@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +22,9 @@
  */
 
 #include <config.h>
+
 #include <xapian/error.h>
+#include <xapian/unicode.h>
 #include "omdebug.h"
 #include <xapian/postingiterator.h>
 #include <xapian/termiterator.h>
@@ -31,6 +34,10 @@
 #include "alltermslist.h"
 #include "multialltermslist.h"
 #include "database.h"
+#include "editdistance.h"
+#include "ortermlist.h"
+
+#include <stdlib.h> // For abs().
 
 #include <vector>
 
@@ -101,8 +108,6 @@ PostingIterator
 Database::postlist_begin(const string &tname) const
 {
     DEBUGAPICALL(PostingIterator, "Database::postlist_begin", tname);
-    if (tname.empty())
-	throw InvalidArgumentError("Zero length terms are invalid");
 
     // Don't bother checking that the term exists first.  If it does, we
     // just end up doing more work, and if it doesn't, we save very little
@@ -134,7 +139,7 @@ Database::termlist_begin(Xapian::docid did) const
     if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
 
     unsigned int multiplier = internal.size();
-    LeafTermList *tl;
+    TermList *tl;
     if (multiplier == 1) {
 	// There's no need for the MultiTermList wrapper in the common case
 	// where we're only dealing with a single database.
@@ -144,25 +149,31 @@ Database::termlist_begin(Xapian::docid did) const
 	Xapian::doccount n = (did - 1) % multiplier; // which actual database
 	Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
 
-	tl = new MultiTermList(internal[n]->open_term_list(m), internal[n], *this);
+	tl = new MultiTermList(internal[n]->open_term_list(m), *this, n);
     }
     RETURN(TermIterator(tl));
 }
 
 TermIterator
 Database::allterms_begin() const
+{
+    return allterms_begin("");
+}
+
+TermIterator
+Database::allterms_begin(const std::string & prefix) const
 {
     DEBUGAPICALL(TermIterator, "Database::allterms_begin", "");
     if (internal.empty()) RETURN(TermIterator(NULL));
 
     if (internal.size() == 1)
-	RETURN(TermIterator(internal[0]->open_allterms()));
+	RETURN(TermIterator(internal[0]->open_allterms(prefix)));
  
     vector<TermList *> lists;
 
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
-	lists.push_back((*i)->open_allterms());
+	lists.push_back((*i)->open_allterms(prefix));
     }
 
     RETURN(TermIterator(new MultiAllTermsList(lists)));
@@ -248,8 +259,8 @@ Xapian::doccount
 Database::get_termfreq(const string & tname) const
 {
     DEBUGAPICALL(Xapian::doccount, "Database::get_termfreq", tname);
-    if (tname.empty())
-	throw InvalidArgumentError("Zero length terms are invalid");
+    if (tname.empty()) RETURN(get_doccount());
+
     Xapian::doccount tf = 0;
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); i++) {
@@ -262,8 +273,7 @@ Xapian::termcount
 Database::get_collection_freq(const string & tname) const
 {
     DEBUGAPICALL(Xapian::termcount, "Database::get_collection_freq", tname);
-    if (tname.empty())
-	throw InvalidArgumentError("Zero length terms are invalid");
+    if (tname.empty()) RETURN(get_doccount());
 
     Xapian::termcount cf = 0;
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
@@ -303,8 +313,9 @@ Database::get_document(Xapian::docid did) const
 bool
 Database::term_exists(const string & tname) const
 {
-    if (tname.empty())
-	throw InvalidArgumentError("Zero length terms are invalid");
+    if (tname.empty()) {
+	return get_doccount() != 0;
+    }
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	if ((*i)->term_exists(tname)) return true;
@@ -329,6 +340,160 @@ Database::get_description() const
     RETURN("Database()");
 }
 
+// Word must have a trigram score at least this close to the best score seen
+// so far.
+#define TRIGRAM_SCORE_THRESHOLD 2
+
+string
+Database::get_spelling_suggestion(const string &word,
+				  unsigned max_edit_distance) const
+{
+    DEBUGLINE(SPELLING, "Database::get_spelling_suggestion(" << word << ", " <<
+			max_edit_distance << ")");
+    AutoPtr<TermList> merger;
+    for (size_t i = 0; i < internal.size(); ++i) {
+	TermList * tl = internal[i]->open_spelling_termlist(word);
+	DEBUGLINE(SPELLING, "Sub db " << i << " tl = " << (void*)tl);
+	if (tl) {
+	    if (merger.get()) {
+		merger = new OrTermList(merger.release(), tl);
+	    } else {
+		merger = tl;
+	    }
+	}
+    }
+    if (!merger.get()) return string();
+
+    // Convert word to UTF-32.
+    vector<unsigned> utf32_word;
+    utf32_word.assign(Utf8Iterator(word), Utf8Iterator());
+
+    vector<unsigned> utf32_term;
+
+    Xapian::termcount best = 1;
+    string result;
+    int edist_best = max_edit_distance;
+    Xapian::doccount freq_best = 0;
+    while (true) {
+	TermList *ret = merger->next();
+	if (ret) merger = ret;
+
+	if (merger->at_end()) break;
+
+	string term = merger->get_termname();
+	Xapian::termcount score = merger->get_wdf();
+
+	DEBUGLINE(SPELLING, "Term \"" << term << "\" ngram score " << score);
+	if (score + TRIGRAM_SCORE_THRESHOLD >= best) {
+	    if (score > best) best = score;
+
+	    // There's no point considering a word where the difference
+	    // in length is greater than the smallest number of edits we've
+	    // found so far.
+
+	    // First check the length of the encoded UTF-8 version of term.
+	    // Each UTF-32 character is 1-4 bytes in UTF-8.
+	    if (abs((long)term.size() - (long)word.size()) > edist_best * 4) {
+		DEBUGLINE(SPELLING, "Lengths much too different");
+		continue;
+	    }
+
+	    // Now convert to UTF-32, and compare the true lengths more
+	    // strictly.
+	    utf32_term.assign(Utf8Iterator(term), Utf8Iterator());
+
+	    if (abs((long)utf32_term.size() - (long)utf32_word.size())
+		    > edist_best) {
+		DEBUGLINE(SPELLING, "Lengths too different");
+		continue;
+	    }
+
+	    int edist = edit_distance_unsigned(&utf32_term[0],
+					       utf32_term.size(),
+					       &utf32_word[0],
+					       utf32_word.size());
+	    DEBUGLINE(SPELLING, "Edit distance " << edist);
+	    // If we have an exact match, return an empty string since there's
+	    // no correction required.
+	    if (edist == 0) return string();
+
+	    if (edist <= edist_best) {
+		Xapian::doccount freq = 0;
+		for (size_t j = 0; j < internal.size(); ++j)
+		    freq += internal[j]->get_spelling_frequency(term);
+
+		DEBUGLINE(SPELLING, "Freq " << freq << " best " << freq_best);
+		if (edist < edist_best || freq > freq_best) {
+		    DEBUGLINE(SPELLING, "Best so far: \"" << term <<
+					"\" edist " << edist << " freq " <<
+					freq);
+		    result = term;
+		    edist_best = edist;
+		    freq_best = freq;
+		}
+	    }
+	}
+    }
+    DEBUGLINE(SPELLING, "Suggesting \"" << result << "\"");
+    return result;
+}
+
+TermIterator
+Database::spellings_begin() const
+{
+    DEBUGAPICALL(TermIterator, "Database::spellings_begin", "");
+    AutoPtr<TermList> merger;
+    for (size_t i = 0; i < internal.size(); ++i) {
+	TermList * tl = internal[i]->open_spelling_wordlist();
+	if (tl) {
+	    if (merger.get()) {
+		merger = new FreqAdderOrTermList(merger.release(), tl);
+	    } else {
+		merger = tl;
+	    }
+	}
+    }
+    RETURN(TermIterator(merger.release()));
+}
+
+TermIterator
+Database::synonyms_begin(const std::string &term) const
+{
+    DEBUGAPICALL(TermIterator, "Database::synonyms_begin", term);
+    AutoPtr<TermList> merger;
+    for (size_t i = 0; i < internal.size(); ++i) {
+	TermList * tl = internal[i]->open_synonym_termlist(term);
+	if (tl) {
+	    if (merger.get()) {
+		merger = new OrTermList(merger.release(), tl);
+	    } else {
+		merger = tl;
+	    }
+	}
+    }
+    RETURN(TermIterator(merger.release()));
+}
+
+TermIterator
+Database::synonym_keys_begin(const std::string &prefix) const
+{
+    DEBUGAPICALL(TermIterator, "Database::synonyms_keys_begin", prefix);
+    AutoPtr<TermList> merger;
+    for (size_t i = 0; i < internal.size(); ++i) {
+	TermList * tl = internal[i]->open_synonym_keylist(prefix);
+	if (tl) {
+	    if (merger.get()) {
+		merger = new OrTermList(merger.release(), tl);
+	    } else {
+		merger = tl;
+	    }
+	}
+    }
+    RETURN(TermIterator(merger.release()));
+}
+
+///////////////////////////////////////////////////////////////////////////
+
 WritableDatabase::WritableDatabase() : Database()
 {
     DEBUGAPICALL(void, "WritableDatabase::WritableDatabase", "");
@@ -407,11 +572,9 @@ void
 WritableDatabase::delete_document(const std::string & unique_term)
 {
     DEBUGAPICALL(void, "WritableDatabase::delete_document", unique_term);
-    if (unique_term.empty()) throw InvalidArgumentError("Empty termnames are invalid");
-    Xapian::PostingIterator p = postlist_begin(unique_term);
-    for ( ; p != postlist_end(unique_term); ++p) {
-	internal[0]->delete_document(*p);
-    }
+    if (unique_term.empty())
+	throw InvalidArgumentError("Empty termnames are invalid");
+    internal[0]->delete_document(unique_term);
 }
 
 void
@@ -424,21 +587,57 @@ WritableDatabase::replace_document(Xapian::docid did, const Document & document)
 }
 
 Xapian::docid
-WritableDatabase::replace_document(const std::string & unique_term, const Document & document)
+WritableDatabase::replace_document(const std::string & unique_term,
+				   const Document & document)
 {
-    DEBUGAPICALL(void, "WritableDatabase::replace_document",
+    DEBUGAPICALL(Xapian::docid, "WritableDatabase::replace_document",
 		 unique_term << ", " << document);
-    if (unique_term.empty()) throw InvalidArgumentError("Empty termnames are invalid");
-    Xapian::PostingIterator p = postlist_begin(unique_term);
-    if (p == postlist_end(unique_term)) {
-	return internal[0]->add_document(document);
-    }
-    Xapian::docid did = *p;
-    internal[0]->replace_document(did, document);
-    while (++p != postlist_end(unique_term)) {
-	internal[0]->delete_document(*p);
-    }
-    return did;
+    if (unique_term.empty())
+	throw InvalidArgumentError("Empty termnames are invalid");
+    RETURN(internal[0]->replace_document(unique_term, document));
+}
+
+void
+WritableDatabase::add_spelling(const std::string & word,
+			       Xapian::termcount freqinc) const
+{
+    DEBUGAPICALL(void, "WritableDatabase::add_spelling",
+		 word << ", " << freqinc);
+    internal[0]->add_spelling(word, freqinc);
+}
+
+void
+WritableDatabase::remove_spelling(const std::string & word,
+				  Xapian::termcount freqdec) const
+{
+    DEBUGAPICALL(void, "WritableDatabase::remove_spelling",
+		 word << ", " << freqdec);
+    internal[0]->remove_spelling(word, freqdec);
+}
+
+void
+WritableDatabase::add_synonym(const std::string & term,
+			      const std::string & synonym) const
+{
+    DEBUGAPICALL(void, "WritableDatabase::add_synonym",
+		 term << ", " << synonym);
+    internal[0]->add_synonym(term, synonym);
+}
+
+void
+WritableDatabase::remove_synonym(const std::string & term,
+				 const std::string & synonym) const
+{
+    DEBUGAPICALL(void, "WritableDatabase::remove_synonym",
+		 term << ", " << synonym);
+    internal[0]->remove_synonym(term, synonym);
+}
+
+void
+WritableDatabase::clear_synonyms(const std::string & term) const
+{
+    DEBUGAPICALL(void, "WritableDatabase::clear_synonyms", term);
+    internal[0]->clear_synonyms(term);
 }
 
 string
