@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2006 Olly Betts
+ * Copyright 2003,2004,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,12 +21,15 @@
  */
 
 #include <config.h>
+
 #include <xapian/document.h>
 #include <xapian/types.h>
 #include "document.h"
 #include "maptermlist.h"
 #include <xapian/error.h>
 #include <xapian/valueiterator.h>
+#include "utils.h"
+
 #include <algorithm>
 #include <string>
 
@@ -131,12 +134,6 @@ Document::add_term(const string & tname, Xapian::termcount wdfinc)
     internal->add_term(tname, wdfinc);
 }
 
-void
-Document::add_term_nopos(const std::string & term, Xapian::termcount wdfinc)
-{
-    Document::add_term(term, wdfinc);
-}
-
 void
 Document::remove_posting(const string & tname, Xapian::termpos tpos,
 			 Xapian::termcount wdfdec)
@@ -196,6 +193,13 @@ Document::values_end() const
     RETURN(ValueIterator(internal->values_count(), *this));
 }
 
+docid
+Document::get_docid() const
+{
+    DEBUGAPICALL(docid, "Document::get_docid", "");
+    RETURN(internal->get_docid());
+}
+
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -204,11 +208,10 @@ void
 OmDocumentTerm::add_position(Xapian::termpos tpos)
 {
     DEBUGAPICALL(void, "OmDocumentTerm::add_position", tpos);
-    
+
     // We generally expect term positions to be added in approximately
     // increasing order, so check the end first
-    Xapian::termpos last = positions.empty() ? 0 : positions.back();
-    if (tpos > last) {
+    if (positions.empty() || tpos > positions.back()) {
 	positions.push_back(tpos);
 	return;
     }
