@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,33 +48,8 @@
 using namespace std;
 
 namespace Xapian {
-   
-ExpandDeciderFilterTerms::ExpandDeciderFilterTerms(TermIterator terms,
-						   TermIterator termsend)
-    : tset(terms, termsend)
-{
-}
 
-int
-ExpandDeciderFilterTerms::operator()(const string &tname) const
-{
-    /* Solaris CC returns an iterator from tset.find() const, and then
-     * doesn't like comparing it to the const_iterator from end().
-     * Therefore make sure we get a const_iterator to do the comparision.
-     */
-    set<string>::const_iterator i = tset.find(tname);
-    return (i == tset.end());
-}
-
-ExpandDeciderAnd::ExpandDeciderAnd(const ExpandDecider *left_,
-                                   const ExpandDecider *right_)
-        : left(left_), right(right_) { }
-
-int
-ExpandDeciderAnd::operator()(const string &tname) const
-{
-    return ((*left)(tname)) && ((*right)(tname));
-}
+MatchDecider::~MatchDecider() { }
 
 // Methods for Xapian::RSet
 
@@ -637,11 +613,14 @@ MSetIterator::get_description() const
 // Methods for Xapian::Enquire::Internal
 
 Enquire::Internal::Internal(const Database &db_, ErrorHandler * errorhandler_)
-  : db(db_), query(), collapse_key(Xapian::valueno(-1)),
+  : db(db_), query(), collapse_key(Xapian::BAD_VALUENO),
     order(Enquire::ASCENDING), percent_cutoff(0), weight_cutoff(0),
-    sort_key(Xapian::valueno(-1)), sort_by(REL), sort_value_forward(true),
-    bias_halflife(0), bias_weight(0), errorhandler(errorhandler_), weight(0)
+    sort_key(Xapian::BAD_VALUENO), sort_by(REL), sort_value_forward(true),
+    errorhandler(errorhandler_), weight(0)
 {
+    if (db.internal.size() == 0) {
+	throw InvalidArgumentError("Can't make an Enquire object from an uninitialised Database object.");
+    }
 }
 
 Enquire::Internal::~Internal()
@@ -666,11 +645,12 @@ Enquire::Internal::get_query()
 MSet
 Enquire::Internal::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 			    Xapian::doccount check_at_least, const RSet *rset,
-			    const MatchDecider *mdecider) const
+			    const MatchDecider *mdecider,
+			    const MatchDecider *matchspy) const
 {
-    DEBUGCALL(API, MSet, "Enquire::Internal::get_mset", first << ", "
-	      << maxitems << ", " << check_at_least << ", " << rset << ", "
-	      << mdecider);
+    DEBUGCALL(API, MSet, "Enquire::Internal::get_mset", first << ", " <<
+	      maxitems << ", " << check_at_least << ", " << rset << ", " <<
+	      mdecider << ", " << matchspy);
 
     if (weight == 0) {
 	weight = new BM25Weight;
@@ -679,20 +659,20 @@ Enquire::Internal::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     MSet retval;
     if (rset == 0) {
 	::MultiMatch match(db, query.internal.get(), qlen, RSet(), collapse_key,
-		       percent_cutoff, weight_cutoff,
-		       order, sort_key, sort_by, sort_value_forward,
-		       bias_halflife, bias_weight, errorhandler,
-		       new LocalStatsGatherer(), weight);
+			   percent_cutoff, weight_cutoff,
+			   order, sort_key, sort_by, sort_value_forward,
+			   errorhandler, new LocalStatsGatherer(), weight);
 	// Run query and put results into supplied Xapian::MSet object.
-	match.get_mset(first, maxitems, check_at_least, retval, mdecider);
+	match.get_mset(first, maxitems, check_at_least, retval, mdecider,
+		       matchspy);
     } else {
 	::MultiMatch match(db, query.internal.get(), qlen, *rset, collapse_key,
-		       percent_cutoff, weight_cutoff,
-		       order, sort_key, sort_by, sort_value_forward,
-		       bias_halflife, bias_weight, errorhandler,
-		       new LocalStatsGatherer(), weight);
+			   percent_cutoff, weight_cutoff,
+			   order, sort_key, sort_by, sort_value_forward,
+			   errorhandler, new LocalStatsGatherer(), weight);
 	// Run query and put results into supplied Xapian::MSet object.
-	match.get_mset(first, maxitems, check_at_least, retval, mdecider);
+	match.get_mset(first, maxitems, check_at_least, retval, mdecider,
+		       matchspy);
     }
 
     Assert(weight->name() != "bool" || retval.get_max_possible() == 0);
@@ -713,7 +693,6 @@ Enquire::Internal::get_eset(Xapian::termcount maxitems,
 {
     ESet retval;
 
-    // FIXME: make expand and rseti take a refcntptr
     OmExpand expand(db);
     RSetI rseti(db, rset);
 
@@ -724,9 +703,8 @@ Enquire::Internal::get_eset(Xapian::termcount maxitems,
      */
     AutoPtr<ExpandDecider> decider_noquery;
     AutoPtr<ExpandDecider> decider_andnoquery;
-    ExpandDeciderAlways decider_always;
 
-    if (!query.empty() && !(flags & Enquire::include_query_terms)) {
+    if (!query.empty() && !(flags & Enquire::INCLUDE_QUERY_TERMS)) {
 	AutoPtr<ExpandDecider> temp1(
 	    new ExpandDeciderFilterTerms(query.get_terms_begin(),
 					 query.get_terms_end()));
@@ -740,12 +718,10 @@ Enquire::Internal::get_eset(Xapian::termcount maxitems,
 	} else {
 	    edecider = decider_noquery.get();
 	}
-    } else if (edecider == 0) {
-	edecider = &decider_always;
     }
 
     expand.expand(maxitems, retval, &rseti, edecider,
-		  bool(flags & Enquire::use_exact_termfreq), k);
+		  bool(flags & Enquire::USE_EXACT_TERMFREQ), k);
 
     return retval;
 }
@@ -891,7 +867,7 @@ Enquire::set_query(const Query & query, termcount len)
 }
 
 const Query &
-Enquire::get_query()
+Enquire::get_query() const
 {
     DEBUGAPICALL(const Xapian::Query &, "Xapian::Enquire::get_query", "");
     try {
@@ -922,12 +898,6 @@ Enquire::set_docid_order(Enquire::docid_order order)
     internal->order = order;
 }
 
-void
-Enquire::set_sort_forward(bool sort_forward)
-{
-    Enquire::set_docid_order(sort_forward ? ASCENDING : DESCENDING);
-}
-
 void
 Enquire::set_cutoff(Xapian::percent percent_cutoff, Xapian::weight weight_cutoff)
 {
@@ -967,42 +937,28 @@ Enquire::set_sort_by_relevance_then_value(Xapian::valueno sort_key,
     internal->sort_value_forward = ascending;
 }
 
-void
-Enquire::set_sorting(Xapian::valueno sort_key, int sort_bands,
-		     bool sort_by_relevance)
-{
-    if (sort_bands > 1) {
-	throw Xapian::UnimplementedError("sort bands are no longer supported");
-    }
-    if (sort_bands == 0 || sort_key == Xapian::valueno(-1)) {
-	Enquire::set_sort_by_relevance();
-    } else if (!sort_by_relevance) {
-	Enquire::set_sort_by_value(sort_key);
-    } else {
-	Enquire::set_sort_by_value_then_relevance(sort_key);
-    }
-}
-
-void
-Enquire::set_bias(Xapian::weight bias_weight, time_t bias_halflife)
-{
-    internal->bias_weight = bias_weight;
-    internal->bias_halflife = bias_halflife;
-}
-
 MSet
 Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		  Xapian::doccount check_at_least, const RSet *rset,
 		  const MatchDecider *mdecider) const
+{
+    return get_mset(first, maxitems, check_at_least, rset, mdecider, NULL);
+}
+
+MSet
+Enquire::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
+		  Xapian::doccount check_at_least, const RSet *rset,
+		  const MatchDecider *mdecider,
+		  const MatchDecider *matchspy) const
 {
     // FIXME: display contents of pointer params, if they're not null.
     DEBUGAPICALL(Xapian::MSet, "Xapian::Enquire::get_mset", first << ", " <<
 		 maxitems << ", " << check_at_least << ", " << rset << ", " <<
-		 mdecider);
+		 mdecider << ", " << matchspy);
 
     try {
 	RETURN(internal->get_mset(first, maxitems, check_at_least, rset,
-				  mdecider));
+				  mdecider, matchspy));
     } catch (Error & e) {
 	if (internal->errorhandler) (*internal->errorhandler)(e);
 	throw;
