@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +22,8 @@
  */
 
 #include <config.h>
+
+#include "omassert.h"
 #include "omdebug.h"
 #include "omqueryinternal.h"
 #include "utils.h"
@@ -43,9 +46,7 @@ Query::add_subquery(const Query & subq)
 {
     DEBUGAPICALL(void, "Xapian::Query::add_subquery", subq);
     Assert(internal.get());
-    if (!subq.internal.get())
-	throw InvalidArgumentError("Can't compose a query from undefined queries");
-    internal->add_subquery(*(subq.internal));
+    internal->add_subquery(subq.internal.get());
 }
 
 /// Add a subquery by pointer
@@ -57,9 +58,7 @@ Query::add_subquery(const Query * subq)
 	throw InvalidArgumentError("Pointer to subquery may not be null");
     }
     Assert(internal.get());
-    if (!subq->internal.get())
-	throw InvalidArgumentError("Can't compose a query from undefined queries");
-    internal->add_subquery(*(subq->internal));
+    internal->add_subquery(subq->internal.get());
 }
 
 /// Add a subquery which is a single term
@@ -68,7 +67,8 @@ Query::add_subquery(const string & tname)
 {
     DEBUGAPICALL(void, "Xapian::Query::add_subquery", tname);
     Assert(internal.get());
-    internal->add_subquery(tname);
+    Query::Internal subqint(tname);
+    internal->add_subquery(&subqint);
 }
 
 /// Setup the internals for the query, with the appropriate operator.
@@ -132,6 +132,14 @@ Query::Query(Query::op op_, Query q) : internal(0)
     }
 }
 
+Query::Query(Query::op op_, Xapian::valueno valno,
+	     const string &begin, const string &end)
+    : internal(new Query::Internal(op_, valno, begin, end))
+{
+    DEBUGAPICALL(void, "Xapian::Query::Query",
+		 op_ << ", " << valno << ", " << begin << ", " << end);
+}
+
 // Copy constructor
 Query::Query(const Query & copyme)
 	: internal(copyme.internal)
@@ -190,12 +198,6 @@ Query::empty() const
     return internal.get() == 0;
 }
 
-bool
-Query::is_empty() const
-{
-    return empty();
-}
-
 Query::Query(Query::op op_, const std::string & left, const std::string & right)
     : internal(0)
 {
@@ -210,4 +212,8 @@ Query::Query(Query::op op_, const std::string & left, const std::string & right)
     }
 }
 
+/* Define static members. */
+Xapian::Query Xapian::Query::MatchAll = Xapian::Query("");
+Xapian::Query Xapian::Query::MatchNothing = Xapian::Query();
+
 }
