@@ -2,8 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
- * Copyright 2006 Richard Boulton
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,8 @@
  */
 
 #include <config.h>
+
+#include "autoptr.h"
 #include "omdebug.h"
 #include "omqueryinternal.h"
 #include "utils.h"
@@ -54,6 +56,7 @@ get_min_subqs(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_NEAR:
 	case Xapian::Query::OP_PHRASE:
 	case Xapian::Query::OP_ELITE_SET:
+	case Xapian::Query::OP_VALUE_RANGE:
 	    return 0;
 	case Xapian::Query::OP_FILTER:
 	case Xapian::Query::OP_AND_MAYBE:
@@ -70,6 +73,7 @@ get_max_subqs(Xapian::Query::Internal::op_t op)
 {
     switch (op) {
 	case Xapian::Query::Internal::OP_LEAF:
+	case Xapian::Query::OP_VALUE_RANGE:
 	    return 0;
 	case Xapian::Query::OP_FILTER:
 	case Xapian::Query::OP_AND_MAYBE:
@@ -98,8 +102,7 @@ is_leaf(Xapian::Query::Internal::op_t op)
 
 /** serialising method, for network matches.
  *
- *  The format is designed to be relatively easy
- *  to parse, as well as encodable in one line of text.
+ *  The format is designed to be relatively easy to parse.
  *
  *  A single-term query becomes `[<encodedtname>@<termpos>#<wqf>'
  *  where:
@@ -116,17 +119,16 @@ is_leaf(Xapian::Query::Internal::op_t op)
  *  be - FIXME).
  */
 string
-Xapian::Query::Internal::serialise() const
+Xapian::Query::Internal::serialise(Xapian::termpos & curpos) const
 {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
-    Xapian::termpos curpos = 1;
     string result;
 
     if (op == Xapian::Query::Internal::OP_LEAF) {
 	result += '[';
 	result += encode_length(tname.length());
 	result += tname;
-       	if (term_pos != curpos) result += '@' + om_tostring(term_pos);
+	if (term_pos != curpos) result += '@' + om_tostring(term_pos);
 	if (wqf != 1) result += '#' + om_tostring(wqf);
 	++curpos;
     } else {
@@ -134,7 +136,7 @@ Xapian::Query::Internal::serialise() const
 	for (subquery_list::const_iterator i = subqs.begin();
 	     i != subqs.end();
 	     ++i) {
-	    result += (*i)->serialise();
+	    result += (*i)->serialise(curpos);
 	}
 	switch (op) {
 	    case Xapian::Query::Internal::OP_LEAF:
@@ -167,10 +169,19 @@ Xapian::Query::Internal::serialise() const
 	    case Xapian::Query::OP_ELITE_SET:
 		result += "*" + om_tostring(parameter);
 		break;
+	    case Xapian::Query::OP_VALUE_RANGE:
+		result += "]";
+		result += encode_length(tname.length());
+		result += tname;
+		result += encode_length(str_parameter.length());
+		result += str_parameter;
+		result += om_tostring(parameter);
+		break;
 	}
     }
     return result;
 #else
+    (void)curpos;
     throw Xapian::InternalError("query serialisation not compiled in");
 #endif
 }
@@ -190,6 +201,7 @@ Xapian::Query::Internal::get_op_name(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_NEAR:            name = "NEAR"; break;
 	case Xapian::Query::OP_PHRASE:          name = "PHRASE"; break;
 	case Xapian::Query::OP_ELITE_SET:       name = "ELITE_SET"; break;
+	case Xapian::Query::OP_VALUE_RANGE:     name = "VALUE_RANGE"; break;
     }
     return name;
 }
@@ -209,9 +221,21 @@ Xapian::Query::Internal::get_description() const
 	    opstr += "wqf=" + om_tostring(wqf);
 	}
 	if (!opstr.empty()) opstr = ":(" + opstr + ")";
+	if (tname.empty()) return "<alldocuments>" + opstr;
 	return tname + opstr;
     }
 
+    if (op == Xapian::Query::OP_VALUE_RANGE) {
+	opstr = get_op_name(op);
+	opstr += ' ';
+	opstr += om_tostring(parameter);
+	opstr += ' ';
+	opstr += tname;
+	opstr += ' ';
+	opstr += str_parameter;
+	return opstr;
+    }
+
     opstr = " " + get_op_name(op) + " ";
     if (op == Xapian::Query::OP_NEAR ||
 	op == Xapian::Query::OP_PHRASE ||
@@ -324,22 +348,22 @@ QUnserial::readquery() {
 	throw Xapian::InvalidArgumentError("Bad serialised query");
     switch (*p++) {
 	case '[': {
-	    size_t length = decode_length(&p, end);
+	    size_t length = decode_length(&p, end, true);
 	    string tname(p, length);
 	    p += length;
 	    Xapian::termpos term_pos = curpos;
 	    Xapian::termcount wqf = 1;
-	    if (p == end)
-		throw Xapian::InvalidArgumentError("Bad serialised query");
-	    if (*p == '@') {
-		char *tmp; // avoid compiler warning
-		term_pos = strtol(p + 1, &tmp, 10);
-		p = tmp;
-	    }
-	    if (*p == '#') {
-		char *tmp; // avoid compiler warning
-		wqf = strtol(p + 1, &tmp, 10);
-		p = tmp;
+	    if (p != end) {
+		if (*p == '@') {
+		    char *tmp; // avoid compiler warning
+		    term_pos = strtol(p + 1, &tmp, 10);
+		    p = tmp;
+		}
+		if (*p == '#') {
+		    char *tmp; // avoid compiler warning
+		    wqf = strtol(p + 1, &tmp, 10);
+		    p = tmp;
+		}
 	    }
 	    ++curpos;
 	    return new Xapian::Query::Internal(tname, wqf, term_pos);
@@ -360,7 +384,7 @@ qint_from_vector(Xapian::Query::op op,
     Xapian::Query::Internal * qint = new Xapian::Query::Internal(op, parameter);
     vector<Xapian::Query::Internal *>::const_iterator i;
     for (i = vec.begin(); i != vec.end(); i++) {
-	qint->add_subquery(**i);
+	qint->add_subquery(*i);
 	delete *i;
     }
     qint->end_construction();
@@ -413,6 +437,19 @@ QUnserial::readcompound() {
 		    p = tmp;
 		    return qint_from_vector(Xapian::Query::OP_ELITE_SET, subqs,
 					    elite_set_size);
+		}
+		case ']': {
+		    size_t len = decode_length(&p, end, true);
+		    string start(p, len);
+		    p += len;
+		    len = decode_length(&p, end, true);
+		    string stop(p, len);
+		    p += len;
+		    char *tmp; // avoid compiler warning
+		    Xapian::valueno valno = strtoul(p, &tmp, 10);
+		    p = tmp;
+		    return new Xapian::Query::Internal(Xapian::Query::OP_VALUE_RANGE, valno,
+						       start, stop);
 	        }
 	        default:
 		    DEBUGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
@@ -458,6 +495,7 @@ Xapian::Query::Internal::swap(Xapian::Query::Internal &other)
     subqs.swap(other.subqs);
     std::swap(parameter, other.parameter);
     std::swap(tname, other.tname);
+    std::swap(str_parameter, other.str_parameter);
     std::swap(term_pos, other.term_pos);
     std::swap(wqf, other.wqf);
 }
@@ -468,6 +506,7 @@ Xapian::Query::Internal::Internal(const Xapian::Query::Internal &copyme)
 	  subqs(),
 	  parameter(copyme.parameter),
 	  tname(copyme.tname),
+	  str_parameter(copyme.str_parameter),
 	  term_pos(copyme.term_pos),
 	  wqf(copyme.wqf)
 {
@@ -490,9 +529,7 @@ Xapian::Query::Internal::Internal(const string & tname_, Xapian::termcount wqf_,
 	  term_pos(term_pos_),
 	  wqf(wqf_)
 {
-    if (tname.empty()) {
-	throw Xapian::InvalidArgumentError("Termnames may not have zero length.");
-    }
+    validate_query();
 }
 
 Xapian::Query::Internal::Internal(op_t op_, Xapian::termcount parameter_)
@@ -507,6 +544,18 @@ Xapian::Query::Internal::Internal(op_t op_, Xapian::termcount parameter_)
 	throw Xapian::InvalidArgumentError("parameter is only meaningful for OP_NEAR, OP_PHRASE, or OP_ELITE_SET");
 }
 
+Xapian::Query::Internal::Internal(op_t op_, Xapian::valueno valno,
+				  const string &begin, const string &end)
+	: op(op_),
+	  parameter(Xapian::termcount(valno)),
+	  tname(begin),
+	  str_parameter(end)
+{
+    if (op != OP_VALUE_RANGE)
+	throw Xapian::InvalidArgumentError("This constructor is only meaningful for OP_VALUE_RANGE");
+    validate_query();
+}
+
 Xapian::Query::Internal::~Internal()
 {
     subquery_list::iterator i;
@@ -519,16 +568,16 @@ Xapian::Query::Internal *
 Xapian::Query::Internal::end_construction()
 {
     DEBUGCALL(API, void, "Xapian::Query::Internal::end_construction", "");
-    prevalidate_query();
+    validate_query();
     Xapian::Query::Internal * qint = simplify_query();
     if (qint) qint->validate_query();
     return qint;
 }
 
 void
-Xapian::Query::Internal::prevalidate_query() const
+Xapian::Query::Internal::validate_query() const
 {
-    DEBUGCALL(API, void, "Xapian::Query::Internal::prevalidate_query", "");
+    DEBUGCALL(API, void, "Xapian::Query::Internal::validate_query", "");
 
     // Check that the number of subqueries is in acceptable limits for this op
     if (subqs.size() < get_min_subqs(op) ||
@@ -540,23 +589,68 @@ Xapian::Query::Internal::prevalidate_query() const
 		om_tostring(subqs.size()) + ".");
     }
 
-    // Check that the termname is not null in a leaf query
-    Assert(!is_leaf(op) || !tname.empty());
-    // Check that the termname is null in a branch query
-    Assert(is_leaf(op) || tname.empty());
+    // Check that the termname is null in a branch query, unless the op
+    // is OP_VALUE_RANGE.
+    Assert(is_leaf(op) || op == OP_VALUE_RANGE || tname.empty());
 }
 
-void
-Xapian::Query::Internal::validate_query() const
+bool
+Xapian::Query::Internal::simplify_matchnothing()
 {
-    DEBUGCALL(API, void, "Xapian::Query::Internal::validate_query", "");
-    prevalidate_query();
-
-    // Check that all subqueries are valid.
-    subquery_list::const_iterator i;
-    for (i = subqs.begin(); i != subqs.end(); ++i) {
-	(**i).validate_query();
+    subquery_list::iterator sq;
+    switch (op) {
+        case OP_PHRASE:
+        case OP_NEAR:
+        case OP_AND:
+        case OP_FILTER:
+            // Doing an "AND" type operation - if we've got any MatchNothing
+            // nodes, we match nothing.
+            for (sq = subqs.begin(); sq != subqs.end(); sq++) {
+                if (*sq == 0) {
+                    for (sq = subqs.begin(); sq != subqs.end(); sq++)
+                        delete *sq;
+                    subqs.clear();
+                    return true;
+                }
+            }
+            break;
+        case OP_ELITE_SET:
+        case OP_OR:
+        case OP_XOR:
+            // Doing an "OR" type operation - if we've got any MatchNothing
+            // subnodes, drop them; except that we mustn't become an empty
+            // node due to this, so we never drop a MatchNothing subnode
+            // if it's the only subnode.
+            sq = subqs.begin();
+            while (sq != subqs.end() && subqs.size() > 1) {
+                if (*sq == 0) {
+                    sq = subqs.erase(sq);
+                } else {
+                    ++sq;
+                }
+            }
+            break;
+        case OP_AND_MAYBE:
+	case OP_AND_NOT:
+            // If left hand side is MatchNothing, we match nothing.
+            // If right hand side is MatchNothing, replace node with LHS.
+            // So, if either node is MatchNothing, replace node with LHS.
+            // Easiest way to do this is to remove the right hand node,
+            // and let simplify_query() perform the replacement of
+            // the unary operator with its one remaining child.
+            Assert(subqs.size() == 2);
+            if (subqs[0] == 0 || subqs[1] == 0) {
+                sq = subqs.begin();
+                ++sq;
+                delete *sq;
+                subqs.erase(sq);
+            }
+            break;
+        case OP_LEAF:
+            // Do nothing.
+            break;
     }
+    return false;
 }
 
 Xapian::Query::Internal *
@@ -564,9 +658,20 @@ Xapian::Query::Internal::simplify_query()
 {
     DEBUGCALL(API, bool, "Xapian::Query::Internal::simplify_query", "");
 
+    // Simplify any MatchNothing nodes.
+    if (simplify_matchnothing()) {
+	return 0;
+    }
+
+    // General simplifications, dependent on operator.
     switch (op) {
 	case OP_LEAF:
 	    return this;
+	case OP_VALUE_RANGE:
+	    // If the start of the range is greater than the end then we won't
+	    // match anything.
+	    if (tname > str_parameter) return 0;
+	    return this;
 	case OP_PHRASE: case OP_NEAR:
 	    // Default to the number of subqueries.
 	    if (!parameter) parameter = subqs.size();
@@ -581,7 +686,7 @@ Xapian::Query::Internal::simplify_query()
 		    parameter = 10;
 		} else {
 		    parameter = Xapian::termcount(ceil(sqrt(double(subqs.size()))));
-		    Assert(parameter >= 10);
+		    Assert(parameter > 10);
 		}
 	    }
 	    break;
@@ -593,8 +698,9 @@ Xapian::Query::Internal::simplify_query()
 	    break;
     }
 
-    // If we have no subqueries, then we're simply an undefined query.
-    if (subqs.empty()) return 0;
+    // If we have no subqueries, then we're an empty query.
+    if (subqs.empty())
+	return 0;
 
     // Any nodes which are valid with only one subquery can be replaced by
     // that solitary subquery.
@@ -636,7 +742,7 @@ Xapian::Query::Internal::collapse_subqs()
     while (sq != subqs.end()) {
 	Assert(*sq != 0);
 	if (is_leaf((*sq)->op)) {
-	    Assert((*sq)->subqs.size() == 0);
+	    Assert((*sq)->subqs.empty());
 	    subqtable::iterator s = sqtab.find(*sq);
 	    if (sqtab.find(*sq) == sqtab.end()) {
 		sqtab.insert(*sq);
@@ -686,7 +792,7 @@ Xapian::Query::Internal::flatten_subqs()
 	    *sq = *j;
 	    *j = 0;
 	    flatten_subqs();
-	    newq.add_subquery(*this);
+	    newq.add_subquery(this);
 	    delete *sq;
 	    *sq = 0;
 	}
@@ -698,16 +804,18 @@ Xapian::Query::Internal::flatten_subqs()
 }
 
 void
-Xapian::Query::Internal::add_subquery(const Xapian::Query::Internal & subq)
+Xapian::Query::Internal::add_subquery(const Xapian::Query::Internal * subq)
 {
     Assert(!is_leaf(op));
-    if (op == subq.op && (op == OP_AND || op == OP_OR || op == OP_XOR)) {
+    if (subq == 0) {
+	subqs.push_back(0);
+    } else if (op == subq->op && (op == OP_AND || op == OP_OR || op == OP_XOR)) {
 	// Distribute the subquery.
-	for (subquery_list::const_iterator i = subq.subqs.begin();
-	     i != subq.subqs.end(); i++) {
-	    add_subquery(**i);
+	for (subquery_list::const_iterator i = subq->subqs.begin();
+	     i != subq->subqs.end(); i++) {
+	    add_subquery(*i);
 	}
     } else {
-	subqs.push_back(new Xapian::Query::Internal(subq));
+	subqs.push_back(new Xapian::Query::Internal(*subq));
     }
 }
