@@ -1,7 +1,7 @@
 /* omtermlistiterator.cc
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2003,2004,2005,2006 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,16 +23,26 @@
 #include <xapian/termiterator.h>
 #include "termlist.h"
 #include "positionlist.h"
+#include "omassert.h"
 #include "omdebug.h"
 
 using namespace std;
 
+// Helper function.
+inline void
+handle_prune(Xapian::Internal::RefCntPtr<TermList>& old, TermList * result)
+{
+    if (result) {
+	old = result;
+    }
+}
+
 Xapian::TermIterator::TermIterator(Internal *internal_)
 	: internal(internal_)
 {
     if (internal.get()) {
 	// A TermList starts before the start, iterators start at the start
-	internal->next();
+	handle_prune(internal, internal->next());
 	if (internal->at_end()) internal = 0;
     }
 }
@@ -92,7 +102,7 @@ Xapian::TermIterator::operator++()
     DEBUGAPICALL(void, "Xapian::TermIterator::operator++", "");
     Assert(internal.get());
     Assert(!internal->at_end());
-    internal->next();
+    handle_prune(internal, internal->next());
     if (internal->at_end()) internal = 0;
     return *this;
 }
@@ -104,7 +114,7 @@ Xapian::TermIterator::skip_to(const string & tname)
     DEBUGAPICALL(void, "Xapian::TermIterator::skip_to", tname);
     if (internal.get()) {
 	Assert(!internal->at_end());
-	internal->skip_to(tname);
+	handle_prune(internal, internal->skip_to(tname));
 	if (internal->at_end()) internal = 0;
     }
 }
