@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,10 @@
  */
 
 #include <config.h>
+
+// We have to use the deprecated Quartz::open() method.
+#define XAPIAN_DEPRECATED(D) D
+
 #include "database.h"
 #include "utils.h" // for om_tostring
 #include <xapian/dbfactory.h>
@@ -33,10 +37,6 @@
 using namespace std;
 
 // Include headers for all the enabled database backends
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-#include "muscat36/da_database.h"
-#include "muscat36/db_database.h"
-#endif
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
 #include "inmemory/inmemory_database.h"
 #endif
@@ -90,37 +90,6 @@ InMemory::open() {
 }
 #endif
 
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-Database
-Muscat36::open_da(const string &R, const string &T, bool heavy_duty) {
-    DEBUGAPICALL_STATIC(Database, "Muscat36::open_da", R << ", " << T << ", " <<
-			heavy_duty);
-    return Database(new DADatabase(R, T, "", heavy_duty));
-}
-
-Database
-Muscat36::open_da(const string &R, const string &T, const string &values,
-		  bool heavy_duty) {
-    DEBUGAPICALL_STATIC(Database, "Muscat36::open_da", R << ", " << T << ", " <<
-			values << ", " << heavy_duty);
-    return Database(new DADatabase(R, T, values, heavy_duty));
-}
-
-Database
-Muscat36::open_db(const string &DB, size_t cache_size) {
-    DEBUGAPICALL_STATIC(Database, "Muscat36::open_db", DB << ", " <<
-			cache_size);
-    return Database(new DBDatabase(DB, "", cache_size));
-}
-
-Database
-Muscat36::open_db(const string &DB, const string &values, size_t cache_size) {
-    DEBUGAPICALL_STATIC(Database, "Muscat36::open_db", DB << ", " << values <<
-			", " << cache_size);
-    return Database(new DBDatabase(DB, values, cache_size));
-}
-#endif
-
 static void
 open_stub(Database *db, const string &file)
 {
@@ -138,16 +107,16 @@ open_stub(Database *db, const string &file)
 	    if (type == "auto") {
 		db->add_database(Database(line));
 		ok = true;
-#ifdef XAPIAN_HAS_QUARTZ_BACKEND
-	    } else if (type == "quartz") {
-		db->add_database(Quartz::open(line));
-		ok = true;
-#endif
 #ifdef XAPIAN_HAS_FLINT_BACKEND
 	    } else if (type == "flint") {
 		db->add_database(Flint::open(line));
 		ok = true;
 #endif
+#ifdef XAPIAN_HAS_QUARTZ_BACKEND
+	    } else if (type == "quartz") {
+		db->add_database(Quartz::open(line));
+		ok = true;
+#endif
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 	    } else if (type == "remote") {
 		string::size_type colon = line.find(':');
@@ -176,9 +145,6 @@ open_stub(Database *db, const string &file)
 		}
 #endif
 	    }
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-	    // FIXME: da and db too, but I'm too slack to do those right now!
-#endif
 	}
 	if (!ok) break;
 	++line_no;
@@ -212,36 +178,15 @@ Database::Database(const string &path)
 	return;
     }
 
-#ifdef XAPIAN_HAS_QUARTZ_BACKEND
-    if (file_exists(path + "/record_DB")) {
-	internal.push_back(new QuartzDatabase(path));
-	return;
-    }
-#endif
 #ifdef XAPIAN_HAS_FLINT_BACKEND
     if (file_exists(path + "/iamflint")) {
 	internal.push_back(new FlintDatabase(path));
 	return;
     }
 #endif
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-    if (file_exists(path + "/R") && file_exists(path + "/T")) {
-	// can't easily tell flimsy from heavyduty so assume hd
-	string keyfile = path + "/keyfile";
-	if (!file_exists(path + "/keyfile")) keyfile = "";
-	internal.push_back(new DADatabase(path + "/R", path + "/T",
-					  keyfile, true));
-	return;
-    }
-    string dbfile = path + "/DB";
-    if (!file_exists(dbfile)) {
-	dbfile += ".da";
-	if (!file_exists(dbfile)) dbfile = "";
-    }
-    if (!dbfile.empty()) {
-	string keyfile = path + "/keyfile";
-	if (!file_exists(path + "/keyfile")) keyfile = "";
-	internal.push_back(new DBDatabase(dbfile, keyfile));
+#ifdef XAPIAN_HAS_QUARTZ_BACKEND
+    if (file_exists(path + "/record_DB")) {
+	internal.push_back(new QuartzDatabase(path));
 	return;
     }
 #endif
@@ -256,14 +201,7 @@ WritableDatabase::WritableDatabase(const std::string &path, int action)
 		 path << ", " << action);
 #if defined XAPIAN_HAS_FLINT_BACKEND && defined XAPIAN_HAS_QUARTZ_BACKEND
     // Both Flint and Quartz are enabled.
-    bool use_flint = false;
-    const char *p = getenv("XAPIAN_PREFER_FLINT");
-    if (p != NULL && *p) {
-	use_flint = !file_exists(path + "/record_DB");
-    } else {
-	use_flint = file_exists(path + "/iamflint");
-    }
-    if (use_flint) {
+    if (!file_exists(path + "/record_DB")) {
 	internal.push_back(new FlintWritableDatabase(path, action, 8192));
     } else {
 	internal.push_back(new QuartzWritableDatabase(path, action, 8192));
@@ -387,6 +325,16 @@ Database::Internal::delete_document(Xapian::docid)
     Assert(false);
 }
 
+void
+Database::Internal::delete_document(const std::string & unique_term)
+{
+    // Default implementation - overridden for remote databases
+    Xapian::Internal::RefCntPtr<LeafPostList> pl(open_post_list(unique_term));
+    while (pl->next(), !pl->at_end()) {
+	delete_document(pl->get_docid());
+    }
+}
+
 void
 Database::Internal::replace_document(Xapian::docid, const Xapian::Document &)
 {
@@ -395,10 +343,96 @@ Database::Internal::replace_document(Xapian::docid, const Xapian::Document &)
 }
 
 Xapian::docid
-Database::Internal::get_lastdocid() const
+Database::Internal::replace_document(const std::string & unique_term,
+				     const Xapian::Document & document)
 {
-    DEBUGCALL(DB, void, "Database::Internal::get_lastdocid", "");
-    throw Xapian::UnimplementedError("Database::Internal::get_lastdocid() not yet implemented");
+    // Default implementation - overridden for remote databases
+    Xapian::Internal::RefCntPtr<LeafPostList> pl(open_post_list(unique_term));
+    pl->next();
+    if (pl->at_end()) {
+	return add_document(document);
+    }
+    Xapian::docid did = pl->get_docid();
+    replace_document(did, document);
+    while (pl->next(), !pl->at_end()) {
+	delete_document(pl->get_docid());
+    }
+    return did;
+}
+
+TermList *
+Database::Internal::open_spelling_termlist(const string &) const
+{
+    // Only implemented for some database backends - others will just not
+    // suggest spelling corrections (or not contribute to them in a multiple
+    // database situation).
+    return NULL;
+}
+
+TermList *
+Database::Internal::open_spelling_wordlist() const
+{
+    // Only implemented for some database backends - others will just not
+    // suggest spelling corrections (or not contribute to them in a multiple
+    // database situation).
+    return NULL;
+}
+
+Xapian::doccount
+Database::Internal::get_spelling_frequency(const string &) const
+{
+    // Only implemented for some database backends - others will just not
+    // suggest spelling corrections (or not contribute to them in a multiple
+    // database situation).
+    return 0;
+}
+
+void
+Database::Internal::add_spelling(const string &, Xapian::termcount) const
+{
+    throw Xapian::UnimplementedError("This backend doesn't implement spelling correction");
+}
+
+void
+Database::Internal::remove_spelling(const string &, Xapian::termcount) const
+{
+    throw Xapian::UnimplementedError("This backend doesn't implement spelling correction");
+}
+
+TermList *
+Database::Internal::open_synonym_termlist(const string &) const
+{
+    // Only implemented for some database backends - others will just not
+    // expand synonyms (or not contribute to them in a multiple database
+    // situation).
+    return NULL;
+}
+
+TermList *
+Database::Internal::open_synonym_keylist(const string &) const
+{
+    // Only implemented for some database backends - others will just not
+    // expand synonyms (or not contribute to them in a multiple database
+    // situation).
+    return NULL;
+}
+
+void
+Database::Internal::add_synonym(const string &, const string &) const
+{
+    throw Xapian::UnimplementedError("This backend doesn't implement synonyms");
+}
+
+void
+Database::Internal::remove_synonym(const string &, const string &) const
+{
+    throw Xapian::UnimplementedError("This backend doesn't implement synonyms");
+}
+
+void
+Database::Internal::clear_synonyms(const string &) const
+{
+    throw Xapian::UnimplementedError("This backend doesn't implement synonyms");
 }
 
 }
