@@ -22,6 +22,9 @@
 #include <config.h>
 
 #include "safeerrno.h"
+#ifdef __WIN32__
+# include "msvc_posix_wrapper.h"
+#endif
 
 #include "flint_btreebase.h"
 #include "flint_io.h"
@@ -97,24 +100,6 @@ FlintTable_base::FlintTable_base()
 {
 }
 
-FlintTable_base::FlintTable_base(const string &name_, char ch)
-	: revision(0),
-	  block_size(0),
-	  root(0),
-	  level(0),
-	  bit_map_size(0),
-	  item_count(0),
-	  last_block(0),
-	  have_fakeroot(false),
-	  sequential(false),
-	  bit_map_low(0)
-{
-    string err_msg;
-    if (!read(name_, ch, err_msg)) {
-	throw Xapian::DatabaseOpeningError(err_msg);
-    }
-}
-
 FlintTable_base::FlintTable_base(const FlintTable_base &other)
 	: revision(other.revision),
 	  block_size(other.block_size),
@@ -199,7 +184,12 @@ bool
 FlintTable_base::read(const string & name, char ch, string &err_msg)
 {
     string basename = name + "base" + ch;
+#ifdef __WIN32__
+    int h = msvc_posix_open(basename.c_str(), O_RDONLY | O_BINARY);
+#else
     int h = open(basename.c_str(), O_RDONLY | O_BINARY);
+#endif
+
     if (h == -1) {
 	err_msg += "Couldn't open " + basename + ": " + strerror(errno) + "\n";
 	return false;
@@ -318,10 +308,16 @@ FlintTable_base::write_to_file(const string &filename)
     buf += pack_uint(have_fakeroot);
     buf += pack_uint(sequential);
     buf += pack_uint(revision);
-    buf.append(reinterpret_cast<const char *>(bit_map), bit_map_size);
+    if (bit_map_size > 0) {
+	buf.append(reinterpret_cast<const char *>(bit_map), bit_map_size);
+    }
     buf += pack_uint(revision);  // REVISION2
 
+#ifdef __WIN32__
+    int h = msvc_posix_open(filename.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY);
+#else
     int h = open(filename.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
+#endif
     if (h < 0) {
 	string message = string("Couldn't open base ")
 		+ filename + " to write: " + strerror(errno);
