@@ -3,7 +3,8 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,8 +33,10 @@
 #include <xapian/error.h>
 #include <xapian/valueiterator.h>
 
+#include "contiguousalldocspostlist.h"
 #include "flint_modifiedpostlist.h"
 #include "flint_postlist.h"
+#include "flint_alldocspostlist.h"
 #include "flint_termlist.h"
 #include "flint_positionlist.h"
 #include "flint_utils.h"
@@ -42,20 +45,10 @@
 #include "flint_document.h"
 #include "flint_alltermslist.h"
 #include "flint_lock.h"
+#include "flint_spellingwordslist.h"
 
 #include <sys/types.h>
-#include <sys/stat.h>
-#ifdef HAVE_SYS_UTSNAME_H
-# include <sys/utsname.h>
-#endif
-
-#ifdef __CYGWIN__
-# include "safewindows.h"
-# include <sys/cygwin.h>
-#elif defined __WIN32__
-# include "safewindows.h"
-# define getpid() GetCurrentProcessId()
-#endif
+#include "safesysstat.h"
 
 #include <list>
 #include <string>
@@ -76,8 +69,10 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 	  positionlist_table(db_dir, readonly),
 	  termlist_table(db_dir, readonly),
 	  value_table(db_dir, readonly),
+	  synonym_table(db_dir, readonly),
+	  spelling_table(db_dir, readonly),
 	  record_table(db_dir, readonly),
-	  lock(db_dir + "/flicklock") // FIXME: typo - should be flintlock!
+	  lock(db_dir + "/flintlock")
 {
     DEBUGCALL(DB, void, "FlintDatabase", flint_dir << ", " << action <<
 	      ", " << block_size);
@@ -86,9 +81,6 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
     // open tables
     if (action == XAPIAN_DB_READONLY) {
 	if (!dbexists) {
-	    // Catch pre-0.6 Xapian databases and give a better error
-	    if (file_exists(db_dir + "/attribute_DB"))
-		throw Xapian::DatabaseOpeningError("Cannot open database at `" + db_dir + "' - it was created by a pre-0.6 version of Xapian");
 	    throw Xapian::DatabaseOpeningError("Cannot open database at `" + db_dir + "' - it does not exist");
 	}
 	// Can still allow searches even if recovery is needed
@@ -121,12 +113,12 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 
 	if (action == Xapian::DB_CREATE) {
 	    throw Xapian::DatabaseCreateError("Can't create new database at `" +
-		    db_dir + "': a database already exists and I was told "
-		    "not to overwrite it");
+					      db_dir + "': a database already exists and I was told "
+					      "not to overwrite it");
 	}
 
 	get_database_write_lock();
-	// if we're overwriting, pretend the db doesn't exists
+	// if we're overwriting, pretend the db doesn't exist
 	// FIXME: if we allow Xapian::DB_OVERWRITE, check it here
 	if (action == Xapian::DB_CREATE_OR_OVERWRITE) {
 	    create_and_open_tables(block_size);
@@ -143,11 +135,7 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 	    postlist_table.get_latest_revision_number()) {
 	    flint_revision_number_t new_revision = get_next_revision_number();
 
-	    postlist_table.commit(new_revision);
-	    positionlist_table.commit(new_revision);
-	    termlist_table.commit(new_revision);
-	    value_table.commit(new_revision);
-	    record_table.commit(new_revision);
+	    set_revision_number(new_revision);
 	}
 	if (record_table.get_doccount() == 0) {
 	    postlist_table.set_total_length_and_lastdocid(0, postlist_table.get_lastdocid());
@@ -158,7 +146,6 @@ FlintDatabase::FlintDatabase(const string &flint_dir, int action,
 FlintDatabase::~FlintDatabase()
 {
     DEBUGCALL(DB, void, "~FlintDatabase", "");
-    if (lock) lock.release();
 }
 
 bool
@@ -166,9 +153,7 @@ FlintDatabase::database_exists() {
     DEBUGCALL(DB, bool, "FlintDatabase::database_exists", "");
     return record_table.exists() &&
 	   postlist_table.exists() &&
-	   positionlist_table.exists() &&
-	   termlist_table.exists() &&
-	   value_table.exists();
+	   termlist_table.exists();
 }
 
 void
@@ -180,25 +165,27 @@ FlintDatabase::create_and_open_tables(unsigned int block_size)
     // Create postlist_table first, and record_table last.  Existence of
     // record_table is considered to imply existence of the database.
     version_file.create();
-    postlist_table.create(block_size);
-    positionlist_table.create(block_size);
-    termlist_table.create(block_size);
-    value_table.create(block_size);
-    record_table.create(block_size);
+    postlist_table.create_and_open(block_size);
+    // The positionlist table is created lazily, but erase it in case we're
+    // overwriting an existing database and it already exists.
+    positionlist_table.erase();
+    positionlist_table.set_block_size(block_size);
+
+    termlist_table.create_and_open(block_size);
+    // The value table is created lazily, but erase it in case we're
+    // overwriting an existing database and it already exists.
+    value_table.erase();
+    value_table.set_block_size(block_size);
+
+    synonym_table.create_and_open(block_size);
+    spelling_table.create_and_open(block_size);
+    record_table.create_and_open(block_size);
 
     Assert(database_exists());
 
-    record_table.open();
-    value_table.open();
-    termlist_table.open();
-    positionlist_table.open();
-    postlist_table.open();
-
     // Check consistency
     flint_revision_number_t revision = record_table.get_open_revision_number();
-    if (revision != value_table.get_open_revision_number() ||
-	revision != termlist_table.get_open_revision_number() ||
-	revision != positionlist_table.get_open_revision_number() ||
+    if (revision != termlist_table.get_open_revision_number() ||
 	revision != postlist_table.get_open_revision_number()) {
 	throw Xapian::DatabaseCreateError("Newly created tables are not in consistent state");
     }
@@ -221,16 +208,25 @@ FlintDatabase::open_tables_consistent()
     record_table.open();
     flint_revision_number_t revision = record_table.get_open_revision_number();
 
+    // In case the positionlist, value, synonym, and/or spelling tables don't
+    // exist yet.
+    unsigned int block_size = record_table.get_block_size();
+    positionlist_table.set_block_size(block_size);
+    value_table.set_block_size(block_size);
+    synonym_table.set_block_size(block_size);
+    spelling_table.set_block_size(block_size);
+
     bool fully_opened = false;
     int tries = 100;
     int tries_left = tries;
     while (!fully_opened && (tries_left--) > 0) {
-	bool opened;
-	opened = value_table.open(revision);
-	if (opened) opened = termlist_table.open(revision);
-	if (opened) opened = positionlist_table.open(revision);
-	if (opened) opened = postlist_table.open(revision);
-	if (opened) {
+	if (spelling_table.open(revision) &&
+	    synonym_table.open(revision) &&
+	    value_table.open(revision) &&
+	    termlist_table.open(revision) &&
+	    positionlist_table.open(revision) &&
+	    postlist_table.open(revision)) {
+	    // Everything now open at the same revision.
 	    fully_opened = true;
 	} else {
 	    // Couldn't open consistent revision: two cases possible:
@@ -252,11 +248,12 @@ FlintDatabase::open_tables_consistent()
 		// is inconsistent.
 		throw Xapian::DatabaseCorruptError("Cannot open tables at consistent revisions");
 	    }
+	    revision = newrevision;
 	}
     }
 
     if (!fully_opened) {
-	throw Xapian::DatabaseOpeningError("Cannot open tables at stable revision - changing too fast");
+	throw Xapian::DatabaseModifiedError("Cannot open tables at stable revision - changing too fast");
     }
 }
 
@@ -266,6 +263,17 @@ FlintDatabase::open_tables(flint_revision_number_t revision)
     DEBUGCALL(DB, void, "FlintDatabase::open_tables", revision);
     version_file.read_and_check();
     record_table.open(revision);
+
+    // In case the positionlist, value, synonym, and/or spelling tables don't
+    // exist yet.
+    unsigned int block_size = record_table.get_block_size();
+    positionlist_table.set_block_size(block_size);
+    value_table.set_block_size(block_size);
+    synonym_table.set_block_size(block_size);
+    spelling_table.set_block_size(block_size);
+
+    spelling_table.open(revision);
+    synonym_table.open(revision);
     value_table.open(revision);
     termlist_table.open(revision);
     positionlist_table.open(revision);
@@ -302,6 +310,8 @@ FlintDatabase::set_revision_number(flint_revision_number_t new_revision)
     positionlist_table.commit(new_revision);
     termlist_table.commit(new_revision);
     value_table.commit(new_revision);
+    synonym_table.commit(new_revision);
+    spelling_table.commit(new_revision);
     record_table.commit(new_revision);
 }
 
@@ -318,8 +328,16 @@ void
 FlintDatabase::get_database_write_lock()
 {
     DEBUGCALL(DB, void, "FlintDatabase::get_database_write_lock", "");
-    if (!lock.lock(true)) {
-	throw Xapian::DatabaseLockError("Unable to acquire database write lock on " + db_dir);
+    FlintLock::reason why = lock.lock(true);
+    if (why != FlintLock::SUCCESS) {
+	string msg("Unable to acquire database write lock on ");
+	msg += db_dir;
+	if (why == FlintLock::INUSE) {
+	    msg += ": already locked";
+	} else if (why == FlintLock::UNSUPPORTED) {
+	    msg += ": locking probably not supported by this FS";
+	}
+	throw Xapian::DatabaseLockError(msg);
     }
 }
 
@@ -331,6 +349,8 @@ FlintDatabase::apply()
 	!positionlist_table.is_modified() &&
 	!termlist_table.is_modified() &&
 	!value_table.is_modified() &&
+	!synonym_table.is_modified() &&
+	!spelling_table.is_modified() &&
 	!record_table.is_modified()) {
 	return;
     }
@@ -339,29 +359,23 @@ FlintDatabase::apply()
     flint_revision_number_t new_revision = get_next_revision_number();
 
     try {
-	postlist_table.commit(new_revision);
-	positionlist_table.commit(new_revision);
-	termlist_table.commit(new_revision);
-	value_table.commit(new_revision);
-	record_table.commit(new_revision);
+	set_revision_number(new_revision);
     } catch (...) {
 	// Modifications failed.  Wipe all the modifications from memory.
-
-	// Reopen tables with old revision number.
-	open_tables(old_revision);
-
-	// Increase revision numbers to new revision number plus one,
-	// writing increased numbers to all tables.
-	new_revision += 1;
-
 	try {
-	    set_revision_number(new_revision);
-
-	    // This cancel() causes any buffered changes to be thrown away,
-	    // and the buffer to be reinitialised with the old entry count.
+	    // Discard any buffered changes and reinitialised cached values
+	    // from the table.
 	    cancel();
-	} catch (const Xapian::Error &) {
-	    throw Xapian::DatabaseError("Modifications failed, and cannot set revision numbers in database to a consistent state");
+
+	    // Reopen tables with old revision number.
+	    open_tables(old_revision);
+
+	    // Increase revision numbers to new revision number plus one,
+	    // writing increased numbers to all tables.
+	    ++new_revision;
+	    set_revision_number(new_revision);
+	} catch (const Xapian::Error &e) {
+	    throw Xapian::DatabaseError("Modifications failed, and cannot set consistent table revision numbers: " + e.get_msg());
 	}
 	throw;
     }
@@ -375,6 +389,8 @@ FlintDatabase::cancel()
     positionlist_table.cancel();
     termlist_table.cancel();
     value_table.cancel();
+    synonym_table.cancel();
+    spelling_table.cancel();
     record_table.cancel();
 }
 
@@ -417,8 +433,7 @@ FlintDatabase::get_termfreq(const string & tname) const
     DEBUGCALL(DB, Xapian::doccount, "FlintDatabase::get_termfreq", tname);
     Assert(!tname.empty());
 
-    FlintPostList pl(NULL, &postlist_table, NULL, tname);
-    RETURN(pl.get_termfreq());
+    RETURN(postlist_table.get_termfreq(tname));
 }
 
 Xapian::termcount
@@ -427,10 +442,7 @@ FlintDatabase::get_collection_freq(const string & tname) const
     DEBUGCALL(DB, Xapian::termcount, "FlintDatabase::get_collection_freq", tname);
     Assert(!tname.empty());
 
-    Xapian::termcount collfreq = 0; // If not found, this value will be unchanged.
-    FlintPostList pl(NULL, &postlist_table, NULL, tname);
-    collfreq = pl.get_collection_freq();
-    RETURN(collfreq);
+    RETURN(postlist_table.get_collection_freq(tname));
 }
 
 bool
@@ -438,10 +450,9 @@ FlintDatabase::term_exists(const string & tname) const
 {
     DEBUGCALL(DB, bool, "FlintDatabase::term_exists", tname);
     Assert(!tname.empty());
-    AutoPtr<FlintCursor> cursor(postlist_table.cursor_get());
     // FIXME: nasty C&P from backends/flint/flint_postlist.cc
     string key = pack_string_preserving_sort(tname);
-    return cursor->find_entry(key);
+    return postlist_table.key_exists(key);
 }
 
 bool
@@ -452,22 +463,29 @@ FlintDatabase::has_positions() const
 
 
 LeafPostList *
-FlintDatabase::do_open_post_list(const string& tname) const
+FlintDatabase::open_post_list(const string& tname) const
 {
-    DEBUGCALL(DB, LeafPostList *, "FlintDatabase::do_open_post_list", tname);
-    Assert(!tname.empty());
-
+    DEBUGCALL(DB, LeafPostList *, "FlintDatabase::open_post_list", tname);
     Xapian::Internal::RefCntPtr<const FlintDatabase> ptrtothis(this);
-    return(new FlintPostList(ptrtothis,
-			      &postlist_table,
-			      &positionlist_table,
-			      tname));
+
+    if (tname.empty()) {
+	Xapian::doccount doccount = get_doccount();
+    	if (postlist_table.get_lastdocid() == doccount) {
+	    RETURN(new ContiguousAllDocsPostList(ptrtothis, doccount));
+	}
+	RETURN(new FlintAllDocsPostList(ptrtothis, &termlist_table, doccount));
+    }
+
+    RETURN(new FlintPostList(ptrtothis,
+			     &postlist_table,
+			     &positionlist_table,
+			     tname));
 }
 
-LeafTermList *
+TermList *
 FlintDatabase::open_term_list(Xapian::docid did) const
 {
-    DEBUGCALL(DB, LeafTermList *, "FlintDatabase::open_term_list", did);
+    DEBUGCALL(DB, TermList *, "FlintDatabase::open_term_list", did);
     Assert(did != 0);
 
     Xapian::Internal::RefCntPtr<const FlintDatabase> ptrtothis(this);
@@ -499,9 +517,9 @@ FlintDatabase::open_position_list(Xapian::docid did,
     if (poslist->get_size() == 0) {
 	// Check that term / document combination exists.
 	// If the doc doesn't exist, this will throw Xapian::DocNotFoundError:
-	AutoPtr<LeafTermList> ltl(open_term_list(did));
-	ltl->skip_to(tname);
-	if (ltl->at_end() || ltl->get_termname() != tname)
+	AutoPtr<TermList> tl(open_term_list(did));
+	tl->skip_to(tname);
+	if (tl->at_end() || tl->get_termname() != tname)
 	    throw Xapian::RangeError("Can't open position list: requested term is not present in document.");
     }
 
@@ -509,13 +527,52 @@ FlintDatabase::open_position_list(Xapian::docid did,
 }
 
 TermList *
-FlintDatabase::open_allterms() const
+FlintDatabase::open_allterms(const string & prefix) const
 {
     DEBUGCALL(DB, TermList *, "FlintDatabase::open_allterms", "");
     RETURN(new FlintAllTermsList(Xapian::Internal::RefCntPtr<const FlintDatabase>(this),
-				 &postlist_table));
+				 &postlist_table, prefix));
 }
 
+TermList *
+FlintDatabase::open_spelling_termlist(const string & word) const
+{
+    return spelling_table.open_termlist(word);
+}
+
+TermList *
+FlintDatabase::open_spelling_wordlist() const
+{
+    FlintCursor * cursor = spelling_table.cursor_get();
+    if (!cursor) return NULL;
+    return new FlintSpellingWordsList(Xapian::Internal::RefCntPtr<const FlintDatabase>(this),
+				      cursor);
+}
+
+Xapian::doccount
+FlintDatabase::get_spelling_frequency(const string & word) const
+{
+    return spelling_table.get_word_frequency(word);
+}
+
+TermList *
+FlintDatabase::open_synonym_termlist(const string & term) const
+{
+    return synonym_table.open_termlist(term);
+}
+
+TermList *
+FlintDatabase::open_synonym_keylist(const string & prefix) const
+{
+    FlintCursor * cursor = synonym_table.cursor_get();
+    if (!cursor) return NULL;
+    return new FlintSynonymTermList(Xapian::Internal::RefCntPtr<const FlintDatabase>(this),
+				    cursor, synonym_table.get_entry_count(),
+				    prefix);
+}
+
+///////////////////////////////////////////////////////////////////////////
+
 size_t FlintWritableDatabase::flush_threshold = 0;
 
 FlintWritableDatabase::FlintWritableDatabase(const string &dir, int action,
@@ -548,7 +605,11 @@ FlintWritableDatabase::flush()
 {
     if (transaction_active())
 	throw Xapian::InvalidOperationError("Can't flush during a transaction");
-    if (changes_made) do_flush_const();
+    if (changes_made ||
+	database_ro.spelling_table.is_modified() ||
+	database_ro.synonym_table.is_modified()) {
+	do_flush_const();
+    }
 }
 
 void
@@ -556,11 +617,14 @@ FlintWritableDatabase::do_flush_const() const
 {
     DEBUGCALL(DB, void, "FlintWritableDatabase::do_flush_const", "");
 
-    database_ro.postlist_table.merge_changes(mod_plists, doclens, freq_deltas);
+    if (changes_made) {
+	database_ro.postlist_table.merge_changes(mod_plists, doclens,
+						 freq_deltas);
 
-    // Update the total document length and last used docid.
-    database_ro.postlist_table.set_total_length_and_lastdocid(total_length,
-							      lastdocid);
+	// Update the total document length and last used docid.
+	database_ro.postlist_table.set_total_length_and_lastdocid(total_length,
+								  lastdocid);
+    }
     database_ro.apply();
     freq_deltas.clear();
     doclens.clear();
@@ -573,6 +637,9 @@ FlintWritableDatabase::add_document(const Xapian::Document & document)
 {
     DEBUGCALL(DB, Xapian::docid,
 	      "FlintWritableDatabase::add_document", document);
+    // Make sure the docid counter doesn't overflow.
+    if (lastdocid == Xapian::docid(-1))
+	throw Xapian::DatabaseError("Run out of docids - you'll have to use copydatabase to eliminate any gaps before you can add more documents");
     // Use the next unused document ID.
     RETURN(add_document_(++lastdocid, document));
 }
@@ -581,6 +648,8 @@ Xapian::docid
 FlintWritableDatabase::add_document_(Xapian::docid did,
 				     const Xapian::Document & document)
 {
+    DEBUGCALL(DB, Xapian::docid,
+	      "FlintWritableDatabase::add_document_", did << ", " << document);
     Assert(did != 0);
     try {
 	// Add the record using that document ID.
@@ -632,6 +701,7 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
 		}
 	    }
 	}
+	DEBUGLINE(DB, "Calculated doclen for new document " << did << " as " << new_doclen);
 
 	// Set the termlist
 	database_ro.termlist_table.set_entries(did,
@@ -639,7 +709,8 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
 		new_doclen, false);
 
 	// Set the new document length
-	doclens.insert(make_pair(did, new_doclen));
+	Assert(doclens.find(did) == doclens.end());
+	doclens[did] = new_doclen;
 	total_length += new_doclen;
     } catch (...) {
 	// If an error occurs while adding a document, or doing any other
@@ -663,7 +734,7 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
     if (++changes_made >= flush_threshold && !transaction_active())
 	do_flush_const();
 
-    return did;
+    RETURN(did);
 }
 
 void
@@ -672,10 +743,12 @@ FlintWritableDatabase::delete_document(Xapian::docid did)
     DEBUGCALL(DB, void, "FlintWritableDatabase::delete_document", did);
     Assert(did != 0);
 
-    try {
-	// Remove the record.
-	database_ro.record_table.delete_record(did);
+    // Remove the record.  If this fails, just propagate the exception since
+    // the state should still be consistent (most likely it's
+    // DocNotFoundError).
+    database_ro.record_table.delete_record(did);
 
+    try {
 	// Remove the values
 	database_ro.value_table.delete_all_values(did);
 
@@ -724,6 +797,9 @@ FlintWritableDatabase::delete_document(Xapian::docid did)
 
 	// Remove the termlist.
 	database_ro.termlist_table.delete_termlist(did);
+
+	// Remove the new doclength.
+	doclens.erase(did);
     } catch (...) {
 	// If an error occurs while deleting a document, or doing any other
 	// transaction, the modifications so far must be cleared before
@@ -856,17 +932,17 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 		    j->second.insert(make_pair(did, make_pair('A', wdf)));
 		}
 
-		// FIXME : this might not work if we replace a positionlist
-		// with itself (e.g. if a document is replaced with itself
-		// with just the values changed)
-		database_ro.positionlist_table.delete_positionlist(did, tname);
-		if (term.positionlist_begin() != term.positionlist_end()) {
+		PositionIterator it = term.positionlist_begin();
+		PositionIterator it_end = term.positionlist_end();
+		if (it != it_end) {
 		    database_ro.positionlist_table.set_positionlist(
-			did, tname,
-			term.positionlist_begin(), term.positionlist_end());
+			did, tname, it, it_end);
+		} else {
+		    database_ro.positionlist_table.delete_positionlist(did, tname);
 		}
 	    }
 	}
+	DEBUGLINE(DB, "Calculated doclen for replacement document " << did << " as " << new_doclen);
 
 	// Set the termlist
 	database_ro.termlist_table.set_entries(did,
@@ -874,7 +950,7 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 		new_doclen, false);
 
 	// Set the new document length
-	doclens.insert(make_pair(did, new_doclen));
+	doclens[did] = new_doclen;
 	total_length += new_doclen;
     } catch (const Xapian::DocNotFoundError &) {
 	(void)add_document_(did, document);
@@ -964,13 +1040,21 @@ FlintWritableDatabase::has_positions() const
 
 
 LeafPostList *
-FlintWritableDatabase::do_open_post_list(const string& tname) const
+FlintWritableDatabase::open_post_list(const string& tname) const
 {
-    DEBUGCALL(DB, LeafPostList *, "FlintWritableDatabase::do_open_post_list", tname);
-    Assert(!tname.empty());
-
+    DEBUGCALL(DB, LeafPostList *, "FlintWritableDatabase::open_post_list", tname);
     Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
 
+    if (tname.empty()) {
+	Xapian::doccount doccount = get_doccount();
+    	if (lastdocid == doccount) {
+	    RETURN(new ContiguousAllDocsPostList(ptrtothis, doccount));
+	}
+	RETURN(new FlintAllDocsPostList(ptrtothis,
+					&database_ro.termlist_table,
+					doccount));
+    }
+
     map<string, map<docid, pair<char, termcount> > >::const_iterator j;
     j = mod_plists.find(tname);
     if (j != mod_plists.end()) {
@@ -989,10 +1073,10 @@ FlintWritableDatabase::do_open_post_list(const string& tname) const
 			     tname));
 }
 
-LeafTermList *
+TermList *
 FlintWritableDatabase::open_term_list(Xapian::docid did) const
 {
-    DEBUGCALL(DB, LeafTermList *, "FlintWritableDatabase::open_term_list", did);
+    DEBUGCALL(DB, TermList *, "FlintWritableDatabase::open_term_list", did);
     Assert(did != 0);
 
     Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
@@ -1025,9 +1109,9 @@ FlintWritableDatabase::open_position_list(Xapian::docid did,
     if (poslist->get_size() == 0) {
 	// Check that term / document combination exists.
 	// If the doc doesn't exist, this will throw Xapian::DocNotFoundError:
-	AutoPtr<LeafTermList> ltl(open_term_list(did));
-	ltl->skip_to(tname);
-	if (ltl->at_end() || ltl->get_termname() != tname)
+	AutoPtr<TermList> tl(open_term_list(did));
+	tl->skip_to(tname);
+	if (tl->at_end() || tl->get_termname() != tname)
 	    throw Xapian::RangeError("Can't open position list: requested term is not present in document.");
     }
 
@@ -1035,7 +1119,7 @@ FlintWritableDatabase::open_position_list(Xapian::docid did,
 }
 
 TermList *
-FlintWritableDatabase::open_allterms() const
+FlintWritableDatabase::open_allterms(const string & prefix) const
 {
     DEBUGCALL(DB, TermList *, "FlintWritableDatabase::open_allterms", "");
     if (transaction_active())
@@ -1044,7 +1128,7 @@ FlintWritableDatabase::open_allterms() const
     // need to flush.
     if (changes_made) do_flush_const();
     RETURN(new FlintAllTermsList(Xapian::Internal::RefCntPtr<const FlintWritableDatabase>(this),
-				 &database_ro.postlist_table));
+				 &database_ro.postlist_table, prefix));
 }
 
 void
@@ -1058,3 +1142,77 @@ FlintWritableDatabase::cancel()
     mod_plists.clear();
     changes_made = 0;
 }
+
+void
+FlintWritableDatabase::add_spelling(const string & word,
+				    Xapian::termcount freqinc) const
+{
+    database_ro.spelling_table.add_word(word, freqinc);
+}
+
+void
+FlintWritableDatabase::remove_spelling(const string & word,
+				       Xapian::termcount freqdec) const
+{
+    database_ro.spelling_table.remove_word(word, freqdec);
+}
+
+TermList *
+FlintWritableDatabase::open_spelling_termlist(const string & word) const
+{
+    return database_ro.spelling_table.open_termlist(word);
+}
+
+TermList *
+FlintWritableDatabase::open_spelling_wordlist() const
+{
+    database_ro.spelling_table.merge_changes();
+    FlintCursor * cursor = database_ro.spelling_table.cursor_get();
+    if (!cursor) return NULL;
+    return new FlintSpellingWordsList(Xapian::Internal::RefCntPtr<const FlintWritableDatabase>(this),
+				      cursor);
+}
+
+Xapian::doccount
+FlintWritableDatabase::get_spelling_frequency(const string & word) const
+{
+    return database_ro.spelling_table.get_word_frequency(word);
+}
+
+TermList *
+FlintWritableDatabase::open_synonym_termlist(const string & term) const
+{
+    return database_ro.synonym_table.open_termlist(term);
+}
+
+TermList *
+FlintWritableDatabase::open_synonym_keylist(const string & prefix) const
+{
+    database_ro.synonym_table.merge_changes();
+    FlintCursor * cursor = database_ro.synonym_table.cursor_get();
+    if (!cursor) return NULL;
+    return new FlintSynonymTermList(Xapian::Internal::RefCntPtr<const FlintWritableDatabase>(this),
+				    cursor,
+				    database_ro.synonym_table.get_entry_count(),
+				    prefix);
+}
+
+void
+FlintWritableDatabase::add_synonym(const string & term,
+				   const string & synonym) const
+{
+    database_ro.synonym_table.add_synonym(term, synonym);
+}
+
+void
+FlintWritableDatabase::remove_synonym(const string & term,
+				      const string & synonym) const
+{
+    database_ro.synonym_table.remove_synonym(term, synonym);
+}
+
+void
+FlintWritableDatabase::clear_synonyms(const string & term) const
+{
+    database_ro.synonym_table.clear_synonyms(term);
+}
