@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,8 @@
 #include "flint_positionlist.h"
 #include "flint_postlist.h"
 #include "flint_record.h"
+#include "flint_spelling.h"
+#include "flint_synonym.h"
 #include "flint_termlist.h"
 #include "flint_values.h"
 #include "flint_version.h"
@@ -81,6 +83,14 @@ class FlintDatabase : public Xapian::Database::Internal {
 	 */
 	FlintValueTable value_table;
 
+	/** Table storing synonym data.
+	 */
+	mutable FlintSynonymTable synonym_table;
+
+	/** Table storing spelling correction data.
+	 */
+	mutable FlintSpellingTable spelling_table;
+
 	/** Table storing records.
 	 *
 	 *  Whenever an update is performed, this table is the last to be
@@ -148,7 +158,7 @@ class FlintDatabase : public Xapian::Database::Internal {
 	 *          result.
 	 */
 	void set_revision_number(flint_revision_number_t new_revision);
-	
+
 	/** Re-open tables to recover from an overwritten condition,
 	 *  or just get most up-to-date version.
 	 */
@@ -176,7 +186,12 @@ class FlintDatabase : public Xapian::Database::Internal {
 	 *  @exception Xapian::DatabaseCorruptError is thrown if there is no
 	 *             consistent revision available.
 	 *
-	 *  @exception Xapian::DatabaseOpeningError thrown if database can't be opened.
+	 *  @exception Xapian::DatabaseOpeningError thrown if database can't
+	 *             be opened.
+	 *
+	 *  @exception Xapian::DatabaseVersionError thrown if database is in an
+	 *             unsupported format.  This implies that the database was
+	 *             created by an older or newer version of Xapian.
 	 *
 	 *  @param dbdir directory holding flint tables
 	 *
@@ -203,12 +218,19 @@ class FlintDatabase : public Xapian::Database::Internal {
 	bool term_exists(const string & tname) const;
 	bool has_positions() const;
 
-	LeafPostList * do_open_post_list(const string & tname) const;
-	LeafTermList * open_term_list(Xapian::docid did) const;
+	LeafPostList * open_post_list(const string & tname) const;
+	TermList * open_term_list(Xapian::docid did) const;
 	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
 	PositionList * open_position_list(Xapian::docid did,
 					  const string & tname) const;
-	TermList * open_allterms() const;
+	TermList * open_allterms(const string & prefix) const;
+
+	TermList * open_spelling_termlist(const string & word) const;
+	TermList * open_spelling_wordlist() const;
+	Xapian::doccount get_spelling_frequency(const string & word) const;
+
+	TermList * open_synonym_termlist(const string & word) const;
+	TermList * open_synonym_keylist(const string & prefix) const;
 	//@}
 };
 
@@ -258,6 +280,15 @@ class FlintWritableDatabase : public Xapian::Database::Internal {
 
 	virtual Xapian::docid add_document(const Xapian::Document & document);
 	Xapian::docid add_document_(Xapian::docid did, const Xapian::Document & document);
+	// Stop the default implementation of delete_document(term) and
+	// replace_document(term) from being hidden.  This isn't really
+	// a problem as we only try to call them through the base class
+	// (where they aren't hidden) but some compilers generate a warning
+	// about the hiding.
+#if (!defined __GNUC__ && !defined _MSC_VER) || __GNUC__ > 2
+	using Xapian::Database::Internal::delete_document;
+	using Xapian::Database::Internal::replace_document;
+#endif
 	virtual void delete_document(Xapian::docid did);
 	virtual void replace_document(Xapian::docid did,
 				      const Xapian::Document & document);
@@ -266,7 +297,12 @@ class FlintWritableDatabase : public Xapian::Database::Internal {
     public:
 	/** Create and open a writable flint database.
 	 *
-	 *  @exception Xapian::DatabaseOpeningError thrown if database can't be opened.
+	 *  @exception Xapian::DatabaseOpeningError thrown if database can't
+	 *             be opened.
+	 *
+	 *  @exception Xapian::DatabaseVersionError thrown if database is in an
+	 *             unsupported format.  This implies that the database was
+	 *             created by an older or newer version of Xapian.
 	 *
 	 *  @param dir directory holding flint tables
 	 */
@@ -286,15 +322,24 @@ class FlintWritableDatabase : public Xapian::Database::Internal {
 	bool term_exists(const string & tname) const;
 	bool has_positions() const;
 
-	LeafPostList * do_open_post_list(const string & tname) const;
-	LeafTermList * open_term_list(Xapian::docid did) const;
+	LeafPostList * open_post_list(const string & tname) const;
+	TermList * open_term_list(Xapian::docid did) const;
 	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
 	PositionList * open_position_list(Xapian::docid did,
 					  const string & tname) const;
-	TermList * open_allterms() const;
-	void begin_transaction_();
-	void commit_transaction_();
-	void cancel_transaction_();
+	TermList * open_allterms(const string & prefix) const;
+
+	void add_spelling(const string & word, Xapian::termcount freqinc) const;
+	void remove_spelling(const string & word, Xapian::termcount freqdec) const;
+	TermList * open_spelling_termlist(const string & word) const;
+	TermList * open_spelling_wordlist() const;
+	Xapian::doccount get_spelling_frequency(const string & word) const;
+
+	TermList * open_synonym_termlist(const string & word) const;
+	TermList * open_synonym_keylist(const string & prefix) const;
+	void add_synonym(const string & word, const string & synonym) const;
+	void remove_synonym(const string & word, const string & synonym) const;
+	void clear_synonyms(const string & word) const;
 	//@}
 };
 
