@@ -1,6 +1,6 @@
 /* flint_lock.h: database locking for flint backend.
  *
- * Copyright (C) 2005,2006 Olly Betts
+ * Copyright (C) 2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -35,7 +35,6 @@
 #include <netinet/in.h>
 #endif
 #include <sys/socket.h>
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #endif
@@ -50,6 +49,12 @@ class FlintLock {
 #endif
 
   public:
+    typedef enum {
+	SUCCESS, // We got the lock!
+	INUSE, // Already locked by someone else.
+	UNSUPPORTED, // Locking probably not supported (e.g. NFS without lockd).
+	UNKNOWN // The attempt failed for some unspecified reason.
+    } reason;
 #if defined __CYGWIN__ || defined __WIN32__
     FlintLock(const std::string &filename_)
 	: filename(filename_), hFile(INVALID_HANDLE_VALUE) { }
@@ -58,8 +63,10 @@ class FlintLock {
     FlintLock(const std::string &filename_) : filename(filename_), fd(-1) { }
     operator bool() { return fd != -1; }
 #endif
+    // Release any lock held when we're destroyed.
+    ~FlintLock() { release(); }
 
-    bool lock(bool exclusive);
+    reason lock(bool exclusive);
     void release();
 };
 
