@@ -1,8 +1,7 @@
 /* flint_postlist.cc: Postlists in a flint database
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005 Olly Betts
+ * Copyright 2002,2003,2004,2005,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,9 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #include <config.h>
@@ -32,6 +30,20 @@
 // next free docid and total length of all documents
 static const string METAINFO_KEY("", 1);
 
+/// Make a key for accessing the postlist.
+static void
+make_key(const string & tname, Xapian::docid did, string & key)
+{
+    key = pack_string_preserving_sort(tname);
+    key += pack_uint_preserving_sort(did);
+}
+
+static void
+make_key(const string & tname, string & key)
+{
+    key = pack_string_preserving_sort(tname);
+}
+
 Xapian::docid
 FlintPostListTable::get_lastdocid() const
 {
@@ -49,6 +61,32 @@ FlintPostListTable::get_lastdocid() const
     RETURN(did);
 }
 
+Xapian::doccount
+FlintPostListTable::get_termfreq(const string & term) const
+{
+    string key, tag;
+    make_key(term, key);
+    if (!get_exact_entry(key, tag)) return 0;
+
+    Xapian::doccount termfreq;
+    const char * p = tag.data();
+    FlintPostList::read_number_of_entries(&p, p + tag.size(), &termfreq, NULL);
+    return termfreq;
+}
+
+Xapian::termcount
+FlintPostListTable::get_collection_freq(const string & term) const
+{
+    string key, tag;
+    make_key(term, key);
+    if (!get_exact_entry(key, tag)) return 0;
+
+    Xapian::termcount collfreq;
+    const char * p = tag.data();
+    FlintPostList::read_number_of_entries(&p, p + tag.size(), NULL, &collfreq);
+    return collfreq;
+}
+
 void
 FlintPostListTable::set_total_length_and_lastdocid(flint_totlen_t totlen,
 						  Xapian::docid did)
@@ -81,20 +119,6 @@ FlintPostListTable::get_total_length() const
     RETURN(totlen);
 }
 
-/// Make a key for accessing the postlist.
-static void
-make_key(const string & tname, Xapian::docid did, string & key)
-{
-    key = pack_string_preserving_sort(tname);
-    key += pack_uint_preserving_sort(did);
-}
-
-static void
-make_key(const string & tname, string & key)
-{
-    key = pack_string_preserving_sort(tname);
-}
-
 // How big should chunks in the posting list be?  (They
 // will grow slightly bigger than this, but not more than a
 // few bytes extra) - FIXME: tune this value to try to
@@ -316,7 +340,8 @@ class PostlistChunkReader {
 	    return wdf;
 	}
 	flint_doclen_t get_doclength() const {
-	    return doclength;
+	    DEBUGCALL(DB, flint_doclen_t, "PostlistChunkReader::get_doclength", "");
+	    RETURN(doclength);
 	}
 
 	bool is_at_end() const {
@@ -710,7 +735,6 @@ FlintPostList::FlintPostList(Xapian::Internal::RefCntPtr<const Xapian::Database:
     int found = cursor->find_entry(key);
     if (!found) {
 	number_of_entries = 0;
-	collection_freq = 0;
 	is_at_end = true;
 	pos = 0;
 	end = 0;
@@ -722,8 +746,7 @@ FlintPostList::FlintPostList(Xapian::Internal::RefCntPtr<const Xapian::Database:
     pos = cursor->current_tag.data();
     end = pos + cursor->current_tag.size();
 
-    did = read_start_of_first_chunk(&pos, end,
-			      &number_of_entries, &collection_freq);
+    did = read_start_of_first_chunk(&pos, end, &number_of_entries, NULL);
     first_did_in_chunk = did;
     last_did_in_chunk = read_start_of_chunk(&pos, end, first_did_in_chunk,
 					    &is_last_chunk);
@@ -879,13 +902,10 @@ FlintPostList::move_to_chunk_containing(Xapian::docid desired_did)
 	// In first chunk
 #ifdef XAPIAN_DEBUG
 	Xapian::doccount old_number_of_entries = number_of_entries;
-	Xapian::termcount old_collection_freq = collection_freq;
-	did = read_start_of_first_chunk(&pos, end, &number_of_entries,
-					&collection_freq);
+	did = read_start_of_first_chunk(&pos, end, &number_of_entries, NULL);
 	Assert(old_number_of_entries == number_of_entries);
-	Assert(old_collection_freq == collection_freq);
 #else
-	did = read_start_of_first_chunk(&pos, end, 0, 0);
+	did = read_start_of_first_chunk(&pos, end, NULL, NULL);
 #endif
     } else {
 	// In normal chunk
@@ -933,25 +953,23 @@ FlintPostList::skip_to(Xapian::docid desired_did, Xapian::weight w_min)
     have_started = true;
 
     // Don't skip back, and don't need to do anything if already there.
-    if (desired_did <= did) RETURN(NULL);
+    if (is_at_end || desired_did <= did) RETURN(NULL);
 
     // Move to correct chunk
     if (!current_chunk_contains(desired_did)) {
 	move_to_chunk_containing(desired_did);
-	// Might be at_end now - this is why we need the test before moving
+	// Might be at_end now, so we need to check before trying to move
 	// forward in chunk.
+	if (is_at_end) RETURN(NULL);
     }
 
     // Move to correct position in chunk
-    if (!is_at_end) {
+    bool have_document = move_forward_in_chunk_to_at_least(desired_did);
 #ifdef XAPIAN_DEBUG
-	bool have_document =
+    Assert(have_document);
 #else
-		(void)
+    (void)have_document;
 #endif
-		move_forward_in_chunk_to_at_least(desired_did);
-	Assert(have_document);
-    }
 
     DEBUGLINE(DB, string("Skipped to ") <<
 	      (is_at_end ? string("end.") : string("docid, wdf, doclength = ") +
@@ -995,7 +1013,7 @@ FlintPostListTable::get_chunk(const string &tname,
 	*to = new PostlistChunkWriter("", true, tname, true);
 	return Xapian::docid(-1);
     }
- 
+
     // See if we're appending - if so we can shortcut by just copying
     // the data part of the chunk wholesale.
     bool is_first_chunk = (keypos == keyend);
@@ -1055,6 +1073,7 @@ FlintPostListTable::merge_changes(
     const map<Xapian::docid, Xapian::termcount> & doclens,
     const map<string, pair<Xapian::termcount_diff, Xapian::termcount_diff> > & freq_deltas)
 {
+    DEBUGCALL(DB, void, "FlintPostListTable::merge_changes", "mod_plists, doclens, freq_deltas");
     map<string, map<Xapian::docid, pair<char, Xapian::termcount> > >::const_iterator i;
     for (i = mod_plists.begin(); i != mod_plists.end(); ++i) {
 	if (i->second.empty()) continue;
@@ -1133,6 +1152,7 @@ FlintPostListTable::merge_changes(
 	    Xapian::docid did = j->first;
 
 next_chunk:
+	    DEBUGLINE(DB, "Updating tname=" << tname << ", did=" << did);
 	    if (from) while (!from->is_at_end()) {
 		Xapian::docid copy_did = from->get_docid();
 		if (copy_did >= did) {
