@@ -1,9 +1,8 @@
 /* flint_postlist.h: Postlists in flint databases
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005 Olly Betts
+ * Copyright 2002,2003,2004,2005,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #ifndef OM_HGUARD_FLINT_POSTLIST_H
@@ -55,8 +53,6 @@ class FlintPostListTable : public FlintTable {
 	 *  @param path_          - Path at which the table is stored.
 	 *  @param readonly_      - whether to open the table for read only
 	 *                          access.
-	 *  @param blocksize_     - Size of blocks to use.  This parameter is
-	 *                          only used when creating the table.
 	 */
 	FlintPostListTable(string path_, bool readonly_)
 	    : FlintTable(path_ + "/postlist.", readonly_) { }
@@ -75,6 +71,18 @@ class FlintPostListTable : public FlintTable {
 	 */
 	flint_totlen_t get_total_length() const;
 
+	/** Returns number of docs indexed by @a term.
+	 *
+	 *  This is the length of the postlist.
+	 */
+	Xapian::doccount get_termfreq(const std::string & term) const;
+
+	/** Returns the number of occurrences of @a term in the database.
+	 *
+	 *  This is the sum of the wdfs in the postlist.
+	 */
+	Xapian::termcount get_collection_freq(const std::string & term) const;
+
 	/** Get the last document id used.
 	 */
 	Xapian::docid get_lastdocid() const;
@@ -143,10 +151,6 @@ class FlintPostList : public LeafPostList {
 	/// The number of entries in the posting list.
 	Xapian::doccount number_of_entries;
 
-	/// The number of occurrences of the term in the posting list.
-	Xapian::termcount collection_freq;
-
-
 	/// Copying is not allowed.
 	FlintPostList(const FlintPostList &);
 
@@ -214,19 +218,14 @@ class FlintPostList : public LeafPostList {
 	 */
 	Xapian::doccount get_termfreq() const { return number_of_entries; }
 
-	/** Returns the number of occurrences of the term in the database.
-	 *
-	 *  This is the sum of the wdfs in the postlist.
-	 */
-	Xapian::termcount get_collection_freq() const { return collection_freq; }
-
 	/// Returns the current docid.
 	Xapian::docid get_docid() const { Assert(have_started); return did; }
 
 	/// Returns the length of current document.
 	Xapian::doclength get_doclength() const {
+	    DEBUGCALL(DB, Xapian::doclength, "FlintPostList::get_doclength", "");
 	    Assert(have_started);
-	    return static_cast<Xapian::doclength>(doclength);
+	    RETURN(static_cast<Xapian::doclength>(doclength));
 	}
 
 	/** Returns the Within Document Frequency of the term in the current
