@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,9 @@
 #include <config.h>
 
 #include "safeerrno.h"
+#ifdef __WIN32__
+# include "msvc_posix_wrapper.h"
+#endif
 
 // Define to use "dangerous" mode - in this mode we write modified btree
 // blocks back in place.  This is somewhat faster (especially once we're
@@ -51,8 +54,6 @@ PREAD_PROTOTYPE
 PWRITE_PROTOTYPE
 #endif
 
-#include <sys/stat.h>
-
 #include <stdio.h>
 #include <string.h>   /* for memmove */
 #include <limits.h>   /* for CHAR_BIT */
@@ -77,6 +78,9 @@ using std::min;
 using std::string;
 using std::vector;
 
+// Only try to compress tags longer than this many bytes.
+const size_t COMPRESS_MIN = 4;
+
 //#define BTREE_DEBUG_FULL 1
 #undef BTREE_DEBUG_FULL
 
@@ -102,37 +106,13 @@ static void report_cursor(int N, Btree * B, Cursor_ * C)
 
 /* Input/output is defined with calls to the basic Unix system interface: */
 
-static int sys_open_to_write_no_except(const string & name)
-{
-    int fd = open(name.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
-    return fd;
-}
-
-static int sys_open_to_write(const string & name)
-{
-    int fd = sys_open_to_write_no_except(name);
-    if (fd < 0) {
-	string message = string("Couldn't open ")
-		+ name + " to write: " + strerror(errno);
-	throw Xapian::DatabaseOpeningError(message);
-    }
-    return fd;
-}
-
-static int sys_open_for_readwrite(const string & name)
-{
-    int fd = open(name.c_str(), O_RDWR | O_BINARY);
-    if (fd < 0) {
-	string message = string("Couldn't open ")
-		+ name + " read/write: " + strerror(errno);
-	throw Xapian::DatabaseOpeningError(message);
-    }
-    return fd;
-}
-
 static void sys_unlink(const string &filename)
 {
+#ifdef __WIN32__
+    if (msvc_posix_unlink(filename.c_str()) == -1) {
+#else
     if (unlink(filename) == -1) {
+#endif
 	string message = "Failed to unlink ";
 	message += filename;
 	message += ": ";
@@ -144,8 +124,10 @@ static void sys_unlink(const string &filename)
 static inline byte *zeroed_new(size_t size)
 {
     byte *temp = new byte[size];
-    if (temp) memset(temp, 0, size);
-
+    // No need to check if temp is NULL, new throws std::bad_alloc if
+    // allocation fails.
+    Assert(temp);
+    memset(temp, 0, size);
     return temp;
 }
 
@@ -297,7 +279,7 @@ FlintTable::write_block(uint4 n, const byte * p) const
     AssertParanoid(base.block_free_at_start(n));
 
     /* write revision is okay */
-    AssertParanoid(REVISION(p) == latest_revision_number + 1);
+    AssertEqParanoid(REVISION(p), latest_revision_number + 1);
 
     if (both_bases) {
 	// Delete the old base before modifying the database.
@@ -586,9 +568,6 @@ FlintTable::split_root(uint4 split_n)
     }
 
     byte * q = zeroed_new(block_size);
-    if (q == 0) {
-	throw std::bad_alloc();
-    }
     C[level].p = q;
     C[level].c = DIR_START;
     C[level].n = base.next_free_block();
@@ -1035,11 +1014,13 @@ void FlintTable::form_key(const string & key) const
 */
 
 bool
-FlintTable::add(const string &key, string tag)
+FlintTable::add(const string &key, string tag, bool already_compressed)
 {
     DEBUGCALL(DB, bool, "FlintTable::add", key << ", " << tag);
     Assert(writable);
 
+    if (handle == -1) create_and_open(block_size);
+
     if (key.size() > FLINT_BTREE_MAX_KEY_LEN) {
 	throw Xapian::InvalidArgumentError(
 		"Key_ too long: length was " +
@@ -1050,6 +1031,72 @@ FlintTable::add(const string &key, string tag)
 
     form_key(key);
 
+    bool compressed = false;
+    if (already_compressed) {
+	compressed = true;
+    } else if (compress_strategy != DONT_COMPRESS && tag.size() > COMPRESS_MIN) {
+	CompileTimeAssert(DONT_COMPRESS != Z_DEFAULT_STRATEGY);
+	CompileTimeAssert(DONT_COMPRESS != Z_FILTERED);
+	CompileTimeAssert(DONT_COMPRESS != Z_HUFFMAN_ONLY);
+#ifdef Z_RLE
+	CompileTimeAssert(DONT_COMPRESS != Z_RLE);
+#endif
+
+	z_stream stream;
+
+	stream.zalloc = reinterpret_cast<alloc_func>(0);
+	stream.zfree = reinterpret_cast<free_func>(0);
+	stream.opaque = (voidpf)0;
+
+	// -15 means raw deflate with 32K LZ77 window (largest)
+	// memLevel 9 is the highest (8 is default)
+	int err;
+	err = deflateInit2(&stream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -15, 9,
+			   compress_strategy);
+	if (err != Z_OK) {
+	    if (err == Z_MEM_ERROR) throw std::bad_alloc();
+	    string msg = "deflateInit2 failed";
+	    if (stream.msg) {
+		msg += " (";
+		msg += stream.msg;
+		msg += ')';
+	    }
+	    throw Xapian::DatabaseError(msg);
+	}
+
+	stream.next_in = (Bytef *)const_cast<char *>(tag.data());
+	stream.avail_in = (uInt)tag.size();
+
+	// If compressed size is >= tag.size(), we don't want to compress.
+	unsigned long blk_len = tag.size() - 1;
+	unsigned char * blk = new unsigned char[blk_len];
+	stream.next_out = blk;
+	stream.avail_out = (uInt)blk_len;
+
+	err = deflate(&stream, Z_FINISH);
+	if (err == Z_STREAM_END) {
+	    // If deflate succeeded, then the output was at least one byte
+	    // smaller than the input.
+	    tag.assign(reinterpret_cast<const char *>(blk), stream.total_out);
+	    compressed = true;
+	    err = deflateEnd(&stream);
+	    if (err != Z_OK) {
+		string msg = "deflateEnd failed";
+		if (stream.msg) {
+		    msg += " (";
+		    msg += stream.msg;
+		    msg += ')';
+		}
+		throw Xapian::DatabaseError(msg);
+	    }
+	} else {
+	    // Deflate failed - presumably the data wasn't compressible.
+	    (void)deflateEnd(&stream);
+	}
+
+	delete [] blk;
+    }
+
     // sort of matching kt.append_chunk(), but setting the chunk
     const size_t cd = kt.key().length() + K1 + I2 + C2 + C2;  // offset to the tag data
     const size_t L = max_item_size - cd; // largest amount of tag data for any chunk
@@ -1097,9 +1144,9 @@ FlintTable::add(const string &key, string tag)
 	size_t l = (i == m ? residue : (i == 1 ? first_L : L));
 	Assert(cd + l <= block_size);
 	Assert(string::size_type(o + l) <= tag.length());
-	kt.set_tag(cd, tag.data() + o, l);
+	kt.set_tag(cd, tag.data() + o, l, compressed);
 	kt.set_component_of(i);
-	
+
 	o += l;
 	residue -= l;
 
@@ -1129,6 +1176,8 @@ FlintTable::del(const string &key)
     DEBUGCALL(DB, bool, "FlintTable::del", key);
     Assert(writable);
 
+    if (handle == -1) RETURN(false);
+
     // We can't delete a key which we is too long for us to store.
     if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
 
@@ -1160,19 +1209,34 @@ FlintTable::get_exact_entry(const string &key, string & tag) const
     RETURN(find_tag(key, &tag));
 }
 
+bool
+FlintTable::key_exists(const string &key) const
+{
+    DEBUGCALL(DB, bool, "FlintTable::key_exists", key);
+    Assert(!key.empty());
+
+    // An oversized key can't exist, so attempting to search for it should fail.
+    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
+
+    form_key(key);
+    RETURN(find(C));
+}
+
 bool
 FlintTable::find_tag(const string &key, string * tag) const
 {
     DEBUGCALL(DB, bool, "FlintTable::find_tag", key << ", &tag");
+    if (handle == -1) RETURN(false);
+
     form_key(key);
     if (!find(C)) RETURN(false);
 
-    read_tag(C, tag);
+    (void)read_tag(C, tag, false);
     RETURN(true);
 }
 
-void
-FlintTable::read_tag(Cursor_ * C_, string *tag) const
+bool
+FlintTable::read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const
 {
     Item_ item(C_[0].p, C_[0].c);
 
@@ -1185,6 +1249,7 @@ FlintTable::read_tag(Cursor_ * C_, string *tag) const
     if (n > 1) tag->reserve((max_item_size - (1 + K1 + I2 + C2 + C2)) * n);
 
     item.append_chunk(tag);
+    bool compressed = item.get_compressed();
 
     for (int i = 2; i <= n; i++) {
 	next(C_, 0);
@@ -1192,6 +1257,86 @@ FlintTable::read_tag(Cursor_ * C_, string *tag) const
     }
     // At this point the cursor is on the last item - calling next will move
     // it to the next key (FlintCursor::get_tag() relies on this).
+    if (!compressed || keep_compressed) return compressed;
+
+    // FIXME: Perhaps we should we decompress each chunk as we read it so we
+    // don't need both the full compressed and uncompressed tags in memory
+    // at once.
+
+    string utag;
+    // May not be enough for a compressed tag, but it's a reasonable guess.
+    utag.reserve(tag->size() + tag->size() / 2);
+
+    Bytef buf[8192];
+
+    z_stream stream;
+    stream.next_out = buf;
+    stream.avail_out = (uInt)sizeof(buf);
+
+    stream.zalloc = reinterpret_cast<alloc_func>(0);
+    stream.zfree = reinterpret_cast<free_func>(0);
+
+    stream.next_in = Z_NULL;
+    stream.avail_in = 0;
+
+    int err = inflateInit2(&stream, -15);
+    if (err != Z_OK) {
+	if (err == Z_MEM_ERROR) throw std::bad_alloc();
+	string msg = "inflateInit2 failed";
+	if (stream.msg) {
+	    msg += " (";
+	    msg += stream.msg;
+	    msg += ')';
+	}
+	throw Xapian::DatabaseError(msg);
+    }
+
+    stream.next_in = (Bytef*)const_cast<char *>(tag->data());
+    stream.avail_in = (uInt)tag->size();
+
+    while (err != Z_STREAM_END) {
+	stream.next_out = buf;
+	stream.avail_out = (uInt)sizeof(buf);
+	err = inflate(&stream, Z_SYNC_FLUSH);
+	if (err == Z_BUF_ERROR && stream.avail_in == 0) {
+	    DEBUGLINE(DB, "Z_BUF_ERROR - faking checksum of " << stream.adler);
+	    Bytef header2[4];
+	    setint4(header2, 0, stream.adler);
+	    stream.next_in = header2;
+	    stream.avail_in = 4;
+	    err = inflate(&stream, Z_SYNC_FLUSH);
+	    if (err == Z_STREAM_END) break;
+	}
+
+	if (err != Z_OK && err != Z_STREAM_END) {
+	    if (err == Z_MEM_ERROR) throw std::bad_alloc();
+	    string msg = "inflate failed";
+	    if (stream.msg) {
+		msg += " (";
+		msg += stream.msg;
+		msg += ')';
+	    }
+	    throw Xapian::DatabaseError(msg);
+	}
+
+	utag.append(reinterpret_cast<const char *>(buf),
+		    stream.next_out - buf);
+    }
+    if (utag.size() != stream.total_out) {
+	string msg = "compressed tag didn't expand to the expected size: ";
+	msg += om_tostring(utag.size());
+	msg += " != ";
+	// OpenBSD's zlib.h uses off_t instead of uLong for total_out.
+	msg += om_tostring((size_t)stream.total_out);
+	throw Xapian::DatabaseCorruptError(msg);
+    }
+
+    err = inflateEnd(&stream);
+    if (err != Z_OK) abort();
+
+    swap(*tag, utag);
+
+    return false;
 }
 
 void
@@ -1204,6 +1349,7 @@ FlintTable::set_full_compaction(bool parity)
 }
 
 FlintCursor * FlintTable::cursor_get() const {
+    if (handle == -1) return NULL;
     // FIXME Ick - casting away const is nasty
     return new FlintCursor(const_cast<FlintTable *>(this));
 }
@@ -1216,23 +1362,32 @@ FlintTable::basic_open(bool revision_supplied, flint_revision_number_t revision_
     int ch = 'X'; /* will be 'A' or 'B' */
 
     {
+	const size_t BTREE_BASES = 2;
 	string err_msg;
-	vector<char> basenames;
-	basenames.push_back('A');
-	basenames.push_back('B');
+	static const char basenames[BTREE_BASES] = { 'A', 'B' };
 
-	vector<FlintTable_base> bases(basenames.size());
-	vector<bool> base_ok(basenames.size());
+	FlintTable_base bases[BTREE_BASES];
+	bool base_ok[BTREE_BASES];
 
+	both_bases = true;
+	bool valid_base = false;
 	{
-	    for (size_t i = 0; i < basenames.size(); ++i) {
-		base_ok[i] = bases[i].read(name, basenames[i], err_msg);
+	    for (size_t i = 0; i < BTREE_BASES; ++i) {
+		bool ok = bases[i].read(name, basenames[i], err_msg);
+		base_ok[i] = ok;
+		if (ok) {
+		    valid_base = true;
+		} else {
+		    both_bases = false;
+		}
 	    }
 	}
 
-	// FIXME: assumption that there are only two bases
-	if (base_ok[0] && base_ok[1]) both_bases = true;
-	if (!base_ok[0] && !base_ok[1]) {
+	if (!valid_base) {
+	    if (handle != -1) {
+		::close(handle);
+		handle = -1;
+	    }
 	    string message = "Error opening table `";
 	    message += name;
 	    message += "':\n";
@@ -1242,7 +1397,7 @@ FlintTable::basic_open(bool revision_supplied, flint_revision_number_t revision_
 
 	if (revision_supplied) {
 	    bool found_revision = false;
-	    for (size_t i = 0; i < basenames.size(); ++i) {
+	    for (size_t i = 0; i < BTREE_BASES; ++i) {
 		if (base_ok[i] && bases[i].get_revision() == revision_) {
 		    ch = basenames[i];
 		    found_revision = true;
@@ -1258,7 +1413,7 @@ FlintTable::basic_open(bool revision_supplied, flint_revision_number_t revision_
 	    }
 	} else {
 	    flint_revision_number_t highest_revision = 0;
-	    for (size_t i = 0; i < basenames.size(); ++i) {
+	    for (size_t i = 0; i < BTREE_BASES; ++i) {
 		if (base_ok[i] && bases[i].get_revision() >= highest_revision) {
 		    ch = basenames[i];
 		    highest_revision = bases[i].get_revision();
@@ -1269,7 +1424,7 @@ FlintTable::basic_open(bool revision_supplied, flint_revision_number_t revision_
 	FlintTable_base *basep = 0;
 	FlintTable_base *other_base = 0;
 
-	for (size_t i = 0; i < basenames.size(); ++i) {
+	for (size_t i = 0; i < BTREE_BASES; ++i) {
 	    DEBUGLINE(UNKNOWN, "Checking (ch == " << ch << ") against "
 		      "basenames[" << i << "] == " << basenames[i]);
 	    DEBUGLINE(UNKNOWN, "bases[" << i << "].get_revision() == " <<
@@ -1315,9 +1470,6 @@ FlintTable::basic_open(bool revision_supplied, flint_revision_number_t revision_
 
     /* kt holds constructed items as well as keys */
     kt = Item_wr_(zeroed_new(block_size));
-    if (kt.get_address() == 0) {
-	throw std::bad_alloc();
-    }
 
     set_max_item_size(BLOCK_CAPACITY);
 
@@ -1372,12 +1524,30 @@ FlintTable::read_root()
 }
 
 bool
-FlintTable::do_open_to_write(bool revision_supplied, flint_revision_number_t revision_)
+FlintTable::do_open_to_write(bool revision_supplied,
+			     flint_revision_number_t revision_,
+			     bool create_db)
 {
-    /* FIXME: do the exception safety the right way, by making all the
-     * parts into sensible objects.
-     */
+    int flags = O_RDWR | O_BINARY;
+    if (create_db) flags |= O_CREAT | O_TRUNC;
+    handle = ::open((name + "DB").c_str(), flags, 0666);
+    if (handle < 0) {
+	// lazy doesn't make a lot of sense with create_db anyway, but ENOENT
+	// with O_CREAT means a parent directory doesn't exist.
+	if (lazy && !create_db && errno == ENOENT) {
+	    revision_number = revision_;
+	    return true;
+	}
+	string message(create_db ? "Couldn't create " : "Couldn't open ");
+	message += name;
+	message += "DB read/write: ";
+	message += strerror(errno);
+	throw Xapian::DatabaseOpeningError(message);
+    }
+
     if (!basic_open(revision_supplied, revision_)) {
+	::close(handle);
+	handle = -1;
 	if (!revision_supplied) {
 	    throw Xapian::DatabaseOpeningError("Failed to open for writing");
 	}
@@ -1389,8 +1559,6 @@ FlintTable::do_open_to_write(bool revision_supplied, flint_revision_number_t rev
 
     writable = true;
 
-    handle = sys_open_for_readwrite(name + "DB");
-
     prev_ptr = &FlintTable::prev_default;
     next_ptr = &FlintTable::next_default;
 
@@ -1408,9 +1576,6 @@ FlintTable::do_open_to_write(bool revision_supplied, flint_revision_number_t rev
     read_root();
 
     buffer = zeroed_new(block_size);
-    if (buffer == 0) {
-	throw std::bad_alloc();
-    }
 
     // swap for writing
     other_base_letter = base_letter == 'A' ? 'B' : 'A';
@@ -1422,7 +1587,8 @@ FlintTable::do_open_to_write(bool revision_supplied, flint_revision_number_t rev
     return true;
 }
 
-FlintTable::FlintTable(string path_, bool readonly_)
+FlintTable::FlintTable(string path_, bool readonly_,
+		       int compress_strategy_, bool lazy_)
 	: revision_number(0),
 	  item_count(0),
 	  block_size(0),
@@ -1446,8 +1612,9 @@ FlintTable::FlintTable(string path_, bool readonly_)
 	  Btree_modified(false),
 	  full_compaction(false),
 	  writable(!readonly_),
-	  dont_close_handle(false),
-	  split_p(0)
+	  split_p(0),
+	  compress_strategy(compress_strategy_),
+	  lazy(lazy_)
 {
     DEBUGCALL(DB, void, "FlintTable::Btree", path_ << ", " << readonly_);
 }
@@ -1459,8 +1626,8 @@ FlintTable::exists() const {
 	    (file_exists(name + "baseA") || file_exists(name + "baseB")));
 }
 
-/** Delete file, throwing an error if can't delete it (but not if it
- *  doesn't exist)
+/** Delete file, throwing an error if we can't delete it (but not if it
+ *  doesn't exist).
  */
 static void
 sys_unlink_if_exists(const string & filename)
@@ -1473,19 +1640,41 @@ sys_unlink_if_exists(const string & filename)
 }
 
 void
-FlintTable::create(unsigned int block_size_)
+FlintTable::erase()
 {
-    DEBUGCALL(DB, void, "FlintTable::create", block_size_);
+    DEBUGCALL(DB, void, "FlintTable::erase", "");
     close();
 
+    sys_unlink_if_exists(name + "baseA");
+    sys_unlink_if_exists(name + "baseB");
+    sys_unlink_if_exists(name + "DB");
+}
+
+void
+FlintTable::set_block_size(unsigned int block_size_)
+{
+    DEBUGCALL(DB, void, "FlintTable::set_block_size", block_size_);
     // Block size must in the range 2048..BYTE_PAIR_RANGE, and a power of two.
     if (block_size_ < 2048 || block_size_ > BYTE_PAIR_RANGE ||
 	(block_size_ & (block_size_ - 1)) != 0) {
 	block_size_ = FLINT_DEFAULT_BLOCK_SIZE;
     }
+    block_size = block_size_;
+}
+
+void
+FlintTable::create_and_open(unsigned int block_size_)
+{
+    DEBUGCALL(DB, void, "FlintTable::create_and_open", block_size_);
+    Assert(writable);
+    close();
+
+    if (block_size_ == 0) abort();
+    set_block_size(block_size_);
 
     // FIXME: it would be good to arrange that this works such that there's
-    // always a valid table in place...
+    // always a valid table in place if you run create_and_open() on an
+    // existing table.
 
     /* write initial values to files */
 
@@ -1499,13 +1688,8 @@ FlintTable::create(unsigned int block_size_)
     /* remove the alternative base file, if any */
     sys_unlink_if_exists(name + "baseB");
 
-    /* create the main file */
-    int h = sys_open_to_write(name + "DB");
-    if (h == -1 || ::close(h) != 0) {
-	string message = "Error creating DB file: ";
-	message += strerror(errno);
-	throw Xapian::DatabaseOpeningError(message);
-    }
+    // Any errors are thrown if revision_supplied is false.
+    (void)do_open_to_write(false, 0, true);
 }
 
 FlintTable::~FlintTable() {
@@ -1519,17 +1703,21 @@ void FlintTable::close() {
     if (handle != -1) {
 	// If an error occurs here, we just ignore it, since we're just
 	// trying to free everything.
-	if (!dont_close_handle) (void)::close(handle);
+	(void)::close(handle);
 	handle = -1;
     }
 
     for (int j = level; j >= 0; j--) {
 	delete [] C[j].p;
+	C[j].p = 0;
     }
     delete [] split_p;
+    split_p = 0;
 
     delete [] kt.get_address();
+    kt = 0;
     delete [] buffer;
+    buffer = 0;
 }
 
 void
@@ -1542,6 +1730,8 @@ FlintTable::commit(flint_revision_number_t revision)
 	throw Xapian::DatabaseError("New revision too low");
     }
 
+    if (handle == -1) return;
+
     // FIXME: this doesn't work (probably because the table revisions get
     // out of step) but it's wasteful to keep applying changes to value
     // and position if they're never used...
@@ -1555,7 +1745,7 @@ FlintTable::commit(flint_revision_number_t revision)
     }
 
     if (!flint_io_sync(handle)) {
-	if (!dont_close_handle) (void)::close(handle);
+	(void)::close(handle);
 	handle = -1;
 	throw Xapian::DatabaseError("Can't commit new revision - failed to flush DB to disk");
     }
@@ -1609,6 +1799,8 @@ FlintTable::cancel()
     DEBUGCALL(DB, void, "FlintTable::cancel", "");
     Assert(writable);
 
+    if (handle == -1) return;
+
     // This causes problems: if (!Btree_modified) return;
 
     string err_msg;
@@ -1643,20 +1835,36 @@ FlintTable::cancel()
 
 /************ B-tree reading ************/
 
-void
+bool
 FlintTable::do_open_to_read(bool revision_supplied, flint_revision_number_t revision_)
 {
-    if (!basic_open(revision_supplied, revision_)) {
-	throw Xapian::DatabaseOpeningError("Failed to open table for reading");
-    }
-
     handle = ::open((name + "DB").c_str(), O_RDONLY | O_BINARY);
     if (handle < 0) {
-	string message = string("Couldn't open ")
-		+ name + " to read: " + strerror(errno);
+	if (lazy) {
+	    // This table is optional when reading!
+	    revision_number = revision_;
+	    return true;
+	}
+	string message("Couldn't open ");
+	message += name;
+	message += "DB to read: ";
+	message += strerror(errno);
 	throw Xapian::DatabaseOpeningError(message);
     }
 
+    if (!basic_open(revision_supplied, revision_)) {
+	::close(handle);
+	handle = -1;
+	if (revision_supplied) {
+	    // The requested revision was not available.
+	    // This could be because the database was modified underneath us, or
+	    // because a base file is missing.  Return false, and work out what
+	    // the problem was at a higher level.
+	    return false;
+	}
+	throw Xapian::DatabaseOpeningError("Failed to open table for reading");
+    }
+
     if (sequential) {
 	prev_ptr = &FlintTable::prev_for_sequential;
 	next_ptr = &FlintTable::next_for_sequential;
@@ -1674,6 +1882,7 @@ FlintTable::do_open_to_read(bool revision_supplied, flint_revision_number_t revi
     }
 
     read_root();
+    return true;
 }
 
 void
@@ -1684,11 +1893,12 @@ FlintTable::open()
     close();
 
     if (!writable) {
-	do_open_to_read(false, 0);
+	// Any errors are thrown if revision_supplied is false
+	(void)do_open_to_read(false, 0);
 	return;
     }
 
-    // Any errors are thrown if revision_supplied is false
+    // Any errors are thrown if revision_supplied is false.
     (void)do_open_to_write(false, 0);
 }
 
@@ -1700,9 +1910,13 @@ FlintTable::open(flint_revision_number_t revision)
     close();
 
     if (!writable) {
-	do_open_to_read(true, revision);
-	AssertEq(revision_number, revision);
-	RETURN(true);
+	if (do_open_to_read(true, revision)) {
+	    AssertEq(revision_number, revision);
+	    RETURN(true);
+	} else {
+	    close();
+	    RETURN(false);
+	}
     }
 
     if (!do_open_to_write(true, revision)) {
