@@ -1,7 +1,7 @@
-/* btree.h: Btree implementation
+/* flint_table.h: Btree implementation
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,8 @@
 #ifndef OM_HGUARD_FLINT_TABLE_H
 #define OM_HGUARD_FLINT_TABLE_H
 
+#include <xapian/visibility.h>
+
 #include <algorithm>
 #include <string>
 using std::string;
@@ -31,6 +33,10 @@ using std::string;
 #include "flint_btreeutil.h"
 #include "flint_cursor.h"
 
+#include <zlib.h>
+
+#define DONT_COMPRESS -1
+
 /** The largest possible value of a key_len.
  *
  *  This gives the upper limit of the size of a key that may be stored in the
@@ -81,7 +87,7 @@ const string::size_type FLINT_BTREE_MAX_KEY_LEN = 252;
    components_of(p, c) returns the number marked 'C' above,
 */
 
-class Key_ {
+class XAPIAN_VISIBILITY_DEFAULT Key_ {
     const byte *p;
 public:
     explicit Key_(const byte * p_) : p(p_) { }
@@ -113,7 +119,8 @@ public:
     Item_base_(T p_, int c) : p(p_ + getint2(p_, c)) { }
     Item_base_(T p_) : p(p_) { }
     T get_address() const { return p; }
-    int size() const { return getint2(p, 0); } /* I in diagram above */
+    int size() const { return getint2(p, 0) & 0x7fff; } /* I in diagram above */
+    bool get_compressed() const { return *p & 0x80; }
     int component_of() const {
 	return getint2(p, getK(p, I2) + I2 - C2);
     }
@@ -199,9 +206,10 @@ public:
 	set_component_of(1);
     }
     // FIXME passing cd here is icky
-    void set_tag(int cd, const char *start, int len) {
+    void set_tag(int cd, const char *start, int len, bool compressed) {
 	memmove(p + cd, start, len);
 	set_size(cd + len);
+	if (compressed) *p |= 0x80;
     }
     void fake_root_item() {
 	set_key_len(K1 + C2);   // null key length
@@ -236,7 +244,7 @@ public:
  *  Tags which are null strings _are_ valid, and are different from a
  *  tag simply not being in the table.
  */
-class FlintTable {
+class XAPIAN_VISIBILITY_DEFAULT FlintTable {
     friend class FlintCursor; /* Should probably fix this. */
     private:
 	/// Copying not allowed
@@ -248,17 +256,21 @@ class FlintTable {
     public:
 	/** Create a new Btree object.
 	 *
-	 *  This does not create the table on disk - the create() method must
-	 *  be called to create the table on disk.
+	 *  This does not create the table on disk - the create_and_open()
+	 *  method must be called to create the table on disk.
 	 *
-	 *  This also does not open the table - the open() method must be
-	 *  called before use is made of the table.
+	 *  This also does not open the table - either the create_and_open()
+	 *  or open() methods must be called before use is made of the table.
 	 *
-	 *  @param path_          - Path at which the table is stored.
-	 *  @param readonly_      - whether to open the table for read only
-	 *                          access.
+	 *  @param path_	Path at which the table is stored.
+	 *  @param readonly_	whether to open the table for read only access.
+	 *  @param compress_strategy_	DONT_COMPRESS, Z_DEFAULT_STRATEGY,
+	 *				Z_FILTERED, Z_HUFFMAN_ONLY, or Z_RLE.
+	 *  @param lazy		If true, don't create the table until it's
+	 *			needed.
 	 */
-	FlintTable(string path_, bool readonly_);
+	FlintTable(string path_, bool readonly_,
+		   int compress_strategy_ = DONT_COMPRESS, bool lazy = false);
 
 	/** Close the Btree.
 	 *
@@ -342,6 +354,19 @@ class FlintTable {
 	 */
 	bool get_exact_entry(const string & key, string & tag) const;
 
+	/** Check if a key exists in the Btree.
+	 *
+	 *  This is just like get_exact_entry() except it doesn't read the tag
+	 *  value so is more efficient if you only want to check that the key
+	 *  exists.
+	 *
+	 *  @param key  The key to look for in the table.
+	 *
+	 *  @return true if key is found in table,
+	 *          false if key is not found in table.
+	 */
+	bool key_exists(const string &key) const;
+
 	/** Find a key in the Btree and read its tag.
 	 *
 	 *  If the key is found the tag is copied to tag.  If the key is not
@@ -357,8 +382,14 @@ class FlintTable {
 	bool find_tag(const string &key, string * tag) const;
 
 	/** Read the tag value for the key pointed to by cursor C_.
+	 *
+	 *  @param keep_compressed  Don't uncompress the tag - e.g. useful
+	 *			    if it's just being opaquely copied.
+	 *
+	 *  @return	true if current_tag holds compressed data (always
+	 *		false if keep_compressed was false).
 	 */
-	void read_tag(Cursor_ * C_, string *tag) const;
+	bool read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const;
 
 	/** Add a key/tag pair to the table, replacing any existing pair with
 	 *  the same key.
@@ -368,17 +399,20 @@ class FlintTable {
 	 *  previous commit() will be lost.
 	 *
 	 *  If key is empty, then the null item is replaced.  If key.length()
-	 *  exceeds the the limit on key size, false is returned.
+	 *  exceeds the limit on key size, false is returned.
 	 *
 	 *  e.g.    ok = btree.add("TODAY", "Mon 9 Oct 2000");
 	 *
 	 *  @param key   The key to store in the table.
 	 *  @param tag   The tag to store in the table.
+	 *  @param already_compressed	true if tag is already compressed,
+	 *		for example because it is being opaquely copied
+	 *		(default: false).
 	 *
 	 *  @return true if the operation completed successfully, false
 	 *          otherwise.
 	 */
-	bool add(const string &key, string tag);
+	bool add(const string &key, string tag, bool already_compressed = false);
 
 	/** Delete an entry from the table.
 	 *
@@ -399,7 +433,24 @@ class FlintTable {
 	 */
 	bool del(const string &key);
 
-	/** Create a new empty btree structure on disk.
+	/// Erase this table from disk.
+	void erase();
+
+	/** Set the block size.
+	 *
+	 *  It's only safe to do this before the table is created.
+	 */
+	void set_block_size(unsigned int block_size_);
+
+	/** Get the block size.
+	 */
+	unsigned int get_block_size() const { return block_size; }
+
+	/** Create a new empty btree structure on disk and open it at the
+	 *  initial revision.
+	 *
+	 *  The table must be writable - it doesn't make sense to create
+	 *  a table that is read-only!
 	 *
 	 *  The block size must be less than 64K, where K = 1024. It is unwise
 	 *  to use a small block size (less than 1024 perhaps), so we enforce a
@@ -408,15 +459,17 @@ class FlintTable {
 	 *  Example:
 	 *
 	 *    Btree btree("X-");
-	 *    btree.create(8192);  // files will be X-DB, X-baseA (and X-baseB)
+	 *    btree.create_and_open(8192);
+	 *    // Files will be X-DB, X-baseA (and X-baseB).
 	 *
 	 *  @param blocksize     - Size of blocks to use.
 	 *
-	 *  @exception Xapian::DatabaseCreateError if the table can't be created.
+	 *  @exception Xapian::DatabaseCreateError if the table can't be
+	 *	created.
 	 *  @exception Xapian::InvalidArgumentError if the requested blocksize
-	 *  is unsuitable.
+	 *	is unsuitable.
 	 */
-	void create(unsigned int blocksize);
+	void create_and_open(unsigned int blocksize);
 
 	void set_full_compaction(bool parity);
 
@@ -483,15 +536,19 @@ class FlintTable {
 
     protected:
 
-	/** Perform the opening operation to read.
-	 */
-	void do_open_to_read(bool revision_supplied, flint_revision_number_t revision_);
-
 	/** Perform the opening operation to read.
 	 *
 	 *  Return true iff the open succeeded.
 	 */
-	bool do_open_to_write(bool revision_supplied, flint_revision_number_t revision_);
+	bool do_open_to_read(bool revision_supplied, flint_revision_number_t revision_);
+
+	/** Perform the opening operation to write.
+	 *
+	 *  Return true iff the open succeeded.
+	 */
+	bool do_open_to_write(bool revision_supplied,
+			      flint_revision_number_t revision_,
+			      bool create_db = false);
 	bool basic_open(bool revision_supplied, flint_revision_number_t revision);
 
 	bool find(Cursor_ *) const;
@@ -594,9 +651,6 @@ class FlintTable {
 	/// Set to true when the database is opened to write.
 	bool writable;
 
-	/// Set to true if we shouldn't close handle ourselves.
-	bool dont_close_handle;
-
 	/* B-tree navigation functions */
 	bool prev(Cursor_ *C_, int j) const { return (this->*prev_ptr)(C_, j); }
 	bool next(Cursor_ *C_, int j) const { return (this->*next_ptr)(C_, j); }
@@ -626,6 +680,12 @@ class FlintTable {
 	 */
 	byte * split_p;
 
+	/** DONT_COMPRESS or Z_DEFAULT_COMPRESSION, Z_HUFFMAN_ONLY, Z_RLE. */
+	int compress_strategy;
+
+	/// If true, don't create the table until it's needed.
+	bool lazy;
+
 	/* Debugging methods */
 //	void report_block_full(int m, int n, const byte * p);
 };
