@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +26,8 @@
 
 #include "omdebug.h"
 
+#include "emptypostlist.h"
+#include "expandweight.h"
 #include "inmemory_database.h"
 #include "inmemory_document.h"
 #include "inmemory_alltermslist.h"
@@ -209,16 +212,14 @@ InMemoryTermList::get_approx_size() const
     return terms;
 }
 
-OmExpandBits
-InMemoryTermList::get_weighting() const
+void
+InMemoryTermList::accumulate_stats(Xapian::Internal::ExpandStats & stats) const
 {
     Assert(started);
     Assert(!at_end());
-    Assert(wt != NULL);
-
-    return wt->get_bits(InMemoryTermList::get_wdf(), document_length,
-			InMemoryTermList::get_termfreq(),
-			db->get_doccount());
+    stats.accumulate(InMemoryTermList::get_wdf(), document_length,
+		     InMemoryTermList::get_termfreq(),
+		     db->get_doccount());
 }
 
 string
@@ -260,6 +261,89 @@ InMemoryTermList::positionlist_begin() const
     return Xapian::PositionIterator(db->open_position_list(did, (*pos).tname));
 }
 
+/////////////////////////////
+// InMemoryAllDocsPostList //
+/////////////////////////////
+
+InMemoryAllDocsPostList::InMemoryAllDocsPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_)
+	: did(0), db(db_)
+{
+}
+
+Xapian::doccount
+InMemoryAllDocsPostList::get_termfreq() const
+{
+    return db->totdocs;
+}
+
+Xapian::docid
+InMemoryAllDocsPostList::get_docid() const
+{
+    Assert(did > 0);
+    Assert(did <= db->termlists.size());
+    Assert(db->termlists[did - 1].is_valid);
+    return did;
+}
+
+Xapian::doclength
+InMemoryAllDocsPostList::get_doclength() const
+{
+    return db->get_doclength(did);
+}
+
+Xapian::termcount
+InMemoryAllDocsPostList::get_wdf() const
+{
+    return 1;
+}
+
+PositionList *
+InMemoryAllDocsPostList::read_position_list()
+{
+    throw Xapian::UnimplementedError("Can't open position list for all docs iterator");
+}
+
+PositionList *
+InMemoryAllDocsPostList::open_position_list() const
+{
+    throw Xapian::UnimplementedError("Can't open position list for all docs iterator");
+}
+
+PostList *
+InMemoryAllDocsPostList::next(Xapian::weight /*w_min*/)
+{
+    Assert(!at_end());
+    do {
+       ++did;
+    } while (did <= db->termlists.size() && !db->termlists[did - 1].is_valid);
+    return NULL;
+}
+
+PostList *
+InMemoryAllDocsPostList::skip_to(Xapian::docid did_, Xapian::weight /*w_min*/)
+{
+    Assert(!at_end());
+    if (did <= did_) {
+	did = did_;
+	while (did <= db->termlists.size() && !db->termlists[did - 1].is_valid) {
+	    ++did;
+	}
+    }
+    return NULL;
+}
+
+bool
+InMemoryAllDocsPostList::at_end() const
+{
+    return (did > db->termlists.size());
+}
+
+string
+InMemoryAllDocsPostList::get_description() const
+{
+    return "InMemoryAllDocsPostList" + om_tostring(did);
+}
+
 ///////////////////////////
 // Actual database class //
 ///////////////////////////
@@ -277,16 +361,20 @@ InMemoryDatabase::~InMemoryDatabase()
 }
 
 LeafPostList *
-InMemoryDatabase::do_open_post_list(const string & tname) const
+InMemoryDatabase::open_post_list(const string & tname) const
 {
-    Assert(tname.size() != 0);
+    if (tname.empty()) {
+	if (termlists.empty())
+	    return new EmptyPostList();
+	Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
+	return new InMemoryAllDocsPostList(ptrtothis);
+    }
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
     if (i == postlists.end() || i->second.term_freq == 0)
 	return new EmptyPostList();
 
-    LeafPostList * pl;
-    pl = new InMemoryPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this),
-			      i->second);
+    Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
+    LeafPostList * pl = new InMemoryPostList(ptrtothis, i->second);
     Assert(!pl->at_end());
     return pl;
 }
@@ -342,7 +430,7 @@ InMemoryDatabase::get_doclength(Xapian::docid did) const
     return doclengths[did - 1];
 }
 
-LeafTermList *
+TermList *
 InMemoryDatabase::open_term_list(Xapian::docid did) const
 {
     if (did == 0) throw Xapian::InvalidArgumentError("Docid 0 invalid");
@@ -458,9 +546,12 @@ InMemoryDatabase::replace_document(Xapian::docid did,
 	totdocs--;
     } else if (did > termlists.size()) {
 	termlists.resize(did);
+	termlists[did - 1].is_valid = true;
 	doclengths.resize(did);
 	doclists.resize(did);
 	valuelists.resize(did);
+    } else {
+	termlists[did - 1].is_valid = true;
     }
 
     vector<InMemoryTermEntry>::const_iterator i;
@@ -480,7 +571,6 @@ InMemoryDatabase::replace_document(Xapian::docid did,
 	    ++posting;
 	}
     }
-    termlists[did - 1] = InMemoryDoc();
 
     doclengths[did - 1] = 0;
     doclists[did - 1] = document.get_data();
@@ -515,6 +605,7 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
 	add_values(did, values);
     }
 
+    InMemoryDoc doc(true);
     Xapian::TermIterator i = document.termlist_begin();
     Xapian::TermIterator i_end = document.termlist_end();
     for ( ; i != i_end; ++i) {
@@ -527,11 +618,11 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
 
 	if (j == j_end) {
 	    /* Make sure the posting exists, even without a position. */
-	    make_posting(*i, did, 0, i.get_wdf(), false);
+	    make_posting(&doc, *i, did, 0, i.get_wdf(), false);
 	} else {
 	    positions_present = true;
 	    for ( ; j != j_end; ++j) {
-		make_posting(*i, did, *j, i.get_wdf());
+		make_posting(&doc, *i, did, *j, i.get_wdf());
 	    }
 	}
 
@@ -541,6 +632,7 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
 	postlists[*i].collection_freq += i.get_wdf();
 	++postlists[*i].term_freq;
     }
+    swap(termlists[did - 1], doc);
 
     totdocs++;
 }
@@ -554,21 +646,23 @@ InMemoryDatabase::make_term(const string & tname)
 Xapian::docid
 InMemoryDatabase::make_doc(const string & docdata)
 {
-    termlists.push_back(InMemoryDoc());
+    termlists.push_back(InMemoryDoc(true));
     doclengths.push_back(0);
     doclists.push_back(docdata);
 
-    AssertParanoid(termlists.size() == doclengths.size());
+    AssertEqParanoid(termlists.size(), doclengths.size());
 
     return termlists.size();
 }
 
-void InMemoryDatabase::make_posting(const string & tname,
+void InMemoryDatabase::make_posting(InMemoryDoc * doc,
+				    const string & tname,
 				    Xapian::docid did,
 				    Xapian::termpos position,
 				    Xapian::termcount wdf,
 				    bool use_position)
 {
+    Assert(doc);
     Assert(postlists.find(tname) != postlists.end());
     Assert(did > 0 && did <= termlists.size());
     Assert(did > 0 && did <= doclengths.size());
@@ -595,13 +689,13 @@ void InMemoryDatabase::make_posting(const string & tname,
     termentry.wdf = wdf;
 
     // Now record the termentry
-    termlists[did - 1].add_posting(termentry);
+    doc->add_posting(termentry);
 }
 
 bool
 InMemoryDatabase::term_exists(const string & tname) const
 {
-    Assert(tname.size() != 0);
+    Assert(!tname.empty());
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
     if (i == postlists.end()) return false;
     return (i->second.term_freq != 0);
@@ -614,8 +708,9 @@ InMemoryDatabase::has_positions() const
 }
 
 TermList *
-InMemoryDatabase::open_allterms() const
+InMemoryDatabase::open_allterms(const string & prefix) const
 {
     return new InMemoryAllTermsList(&postlists,
-				    Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this));
+				    Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this),
+				    prefix);
 }
