@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -113,8 +114,12 @@ class InMemoryDoc {
 	// Sorted list of terms indexing this document.
 	vector<InMemoryTermEntry> terms;
 
-	/* Initialise valid */
-	InMemoryDoc() : is_valid(true) {}
+	/* Initialise invalid by default, so that resizing the termlist array
+	 * doesn't create valid documents. */
+	InMemoryDoc() : is_valid(false) {}
+
+	// Initialise specifying validity.
+	InMemoryDoc(bool is_valid_) : is_valid(is_valid_) {}
 
 	void add_posting(const InMemoryTermEntry & post);
 };
@@ -159,8 +164,37 @@ class InMemoryPostList : public LeafPostList {
 	string get_description() const;
 };
 
+/** A PostList over all docs in an inmemory database.
+ */
+class InMemoryAllDocsPostList : public LeafPostList {
+    friend class InMemoryDatabase;
+    private:
+	Xapian::docid did;
+
+	Xapian::Internal::RefCntPtr<const InMemoryDatabase> db;
+
+	InMemoryAllDocsPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db);
+    public:
+	Xapian::doccount get_termfreq() const;
+
+	Xapian::docid       get_docid() const;     // Gets current docid
+	Xapian::doclength   get_doclength() const; // Length of current document
+	Xapian::termcount   get_wdf() const;       // Within Document Frequency
+	PositionList * read_position_list();
+	PositionList * open_position_list() const;
+
+	PostList *next(Xapian::weight w_min);      // Moves to next docid
+
+	PostList *skip_to(Xapian::docid did, Xapian::weight w_min); // Moves to next docid >= specified docid
+
+	// True if we're off the end of the list
+	bool at_end() const;
+
+	string get_description() const;
+};
+
 // Term List
-class InMemoryTermList : public LeafTermList {
+class InMemoryTermList : public TermList {
     friend class InMemoryDatabase;
     private:
 	vector<InMemoryTermEntry>::const_iterator pos;
@@ -178,7 +212,9 @@ class InMemoryTermList : public LeafTermList {
     public:
 	Xapian::termcount get_approx_size() const;
 
-	OmExpandBits get_weighting() const;
+	/// Collate weighting information for the current term.
+	void accumulate_stats(Xapian::Internal::ExpandStats & stats) const;
+
 	string get_termname() const;
 	Xapian::termcount get_wdf() const; // Number of occurrences of term in current doc
 	Xapian::doccount get_termfreq() const;  // Number of docs indexed by term
@@ -193,6 +229,7 @@ class InMemoryTermList : public LeafTermList {
  *  This is a prototype database, mainly used for debugging and testing.
  */
 class InMemoryDatabase : public Xapian::Database::Internal {
+    friend class InMemoryAllDocsPostList;
     private:
 	map<string, InMemoryTerm> postlists;
 	vector<InMemoryDoc> termlists;
@@ -220,7 +257,8 @@ class InMemoryDatabase : public Xapian::Database::Internal {
 	void finish_add_doc(Xapian::docid did, const Xapian::Document &document);
 	void add_values(Xapian::docid did, const map<Xapian::valueno, string> &values_);
 
-	void make_posting(const string & tname,
+	void make_posting(InMemoryDoc * doc,
+			  const string & tname,
 			  Xapian::docid did,
 			  Xapian::termpos position,
 			  Xapian::termcount wdf,
@@ -233,6 +271,15 @@ class InMemoryDatabase : public Xapian::Database::Internal {
 	void cancel();
 
 	Xapian::docid add_document(const Xapian::Document & document);
+	// Stop the default implementation of delete_document(term) and
+	// replace_document(term) from being hidden.  This isn't really
+	// a problem as we only try to call them through the base class
+	// (where they aren't hidden) but some compilers generate a warning
+	// about the hiding.
+#if (!defined __GNUC__ && !defined _MSC_VER) || __GNUC__ > 2
+	using Xapian::Database::Internal::delete_document;
+	using Xapian::Database::Internal::replace_document;
+#endif
 	void delete_document(Xapian::docid did);
 	void replace_document(Xapian::docid did, const Xapian::Document & document);
 	//@}
@@ -258,12 +305,12 @@ class InMemoryDatabase : public Xapian::Database::Internal {
 	bool term_exists(const string & tname) const;
 	bool has_positions() const;
 
-	LeafPostList * do_open_post_list(const string & tname) const;
-	LeafTermList * open_term_list(Xapian::docid did) const;
+	LeafPostList * open_post_list(const string & tname) const;
+	TermList * open_term_list(Xapian::docid did) const;
 	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
 	PositionList * open_position_list(Xapian::docid did,
 					  const string & tname) const;
-	TermList * open_allterms() const;
+	TermList * open_allterms(const string & prefix) const;
 };
 
 #endif /* OM_HGUARD_INMEMORY_DATABASE_H */
