@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,10 @@
 #include <config.h>
 
 #include "safeerrno.h"
+#include "safefcntl.h"
+#ifdef __WIN32__
+# include "msvc_posix_wrapper.h"
+#endif
 
 // Define to use "dangerous" mode - in this mode we write modified btree
 // blocks back in place.  This is somewhat faster (especially once we're
@@ -51,7 +55,7 @@ PREAD_PROTOTYPE
 PWRITE_PROTOTYPE
 #endif
 
-#include <sys/stat.h>
+#include "safeunistd.h"
 
 #include <stdio.h>
 #include <string.h>   /* for memmove */
@@ -72,22 +76,6 @@ PWRITE_PROTOTYPE
 #include <string>
 #include <vector>
 
-#ifdef __WIN32__
-# include <io.h> // for _commit()
-# ifdef _MSC_VER
-// MSVC needs this to get SSIZE_T defined.
-#  include "safewindows.h"
-# endif
-#endif
-
-// Only useful for platforms like Windows which distinguish between text and
-// binary files.
-#ifndef __WIN32__
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-#endif
-
 using std::min;
 using std::string;
 using std::vector;
@@ -119,7 +107,11 @@ static void report_cursor(int N, Btree * B, Cursor * C)
 
 int sys_open_to_read_no_except(const string & name)
 {
+#ifdef __WIN32__
+    int fd = msvc_posix_open(name.c_str(), O_RDONLY | O_BINARY);
+#else
     int fd = open(name.c_str(), O_RDONLY | O_BINARY);
+#endif
     return fd;
 }
 
@@ -136,7 +128,11 @@ int sys_open_to_read(const string & name)
 
 static int sys_open_to_write_no_except(const string & name)
 {
+#ifdef __WIN32__
+    int fd = msvc_posix_open(name.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY);
+#else
     int fd = open(name.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
+#endif
     return fd;
 }
 
@@ -234,7 +230,11 @@ int sys_flush(int h) {
 
 static void sys_unlink(const string &filename)
 {
+#ifdef __WIN32__
+    if (msvc_posix_unlink(filename.c_str()) == -1) {
+#else
     if (unlink(filename) == -1) {
+#endif
 	string message = "Failed to unlink ";
 	message += filename;
 	message += ": ";
@@ -342,7 +342,7 @@ Btree::write_block(uint4 n, const byte * p) const
     AssertParanoid(base.block_free_at_start(n));
 
     /* write revision is okay */
-    AssertParanoid(REVISION(p) == latest_revision_number + 1);
+    AssertEqParanoid(REVISION(p), latest_revision_number + 1);
 
     if (both_bases) {
 	// Delete the old base before modifying the database.
@@ -1264,21 +1264,28 @@ Btree::basic_open(bool revision_supplied, quartz_revision_number_t revision_)
     int ch = 'X'; /* will be 'A' or 'B' */
 
     {
+	const size_t BTREE_BASES = 2;
 	string err_msg;
-	vector<char> basenames;
-	basenames.push_back('A');
-	basenames.push_back('B');
+	static const char basenames[BTREE_BASES] = { 'A', 'B' };
 
-	vector<Btree_base> bases(basenames.size());
-	vector<bool> base_ok(basenames.size());
+	Btree_base bases[BTREE_BASES];
+	bool base_ok[BTREE_BASES];
 
-	for (size_t i = 0; i < basenames.size(); ++i) {
-	    base_ok[i] = bases[i].read(name, basenames[i], err_msg);
+	both_bases = true;
+	bool valid_base = false;
+	{
+	    for (size_t i = 0; i < BTREE_BASES; ++i) {
+		bool ok = bases[i].read(name, basenames[i], err_msg);
+		base_ok[i] = ok;
+		if (ok) {
+		    valid_base = true;
+		} else {
+		    both_bases = false;
+		}
+	    }
 	}
 
-	// FIXME: assumption that there are only two bases
-	if (base_ok[0] && base_ok[1]) both_bases = true;
-	if (!base_ok[0] && !base_ok[1]) {
+	if (!valid_base) {
 	    string message = "Error opening table `";
 	    message += name;
 	    message += "':\n";
@@ -1288,7 +1295,7 @@ Btree::basic_open(bool revision_supplied, quartz_revision_number_t revision_)
 
 	if (revision_supplied) {
 	    bool found_revision = false;
-	    for (size_t i = 0; i < basenames.size(); ++i) {
+	    for (size_t i = 0; i < BTREE_BASES; ++i) {
 		if (base_ok[i] && bases[i].get_revision() == revision_) {
 		    ch = basenames[i];
 		    found_revision = true;
@@ -1304,7 +1311,7 @@ Btree::basic_open(bool revision_supplied, quartz_revision_number_t revision_)
 	    }
 	} else {
 	    quartz_revision_number_t highest_revision = 0;
-	    for (size_t i = 0; i < basenames.size(); ++i) {
+	    for (size_t i = 0; i < BTREE_BASES; ++i) {
 		if (base_ok[i] && bases[i].get_revision() >= highest_revision) {
 		    ch = basenames[i];
 		    highest_revision = bases[i].get_revision();
@@ -1315,7 +1322,7 @@ Btree::basic_open(bool revision_supplied, quartz_revision_number_t revision_)
 	Btree_base *basep = 0;
 	Btree_base *other_base = 0;
 
-	for (size_t i = 0; i < basenames.size(); ++i) {
+	for (size_t i = 0; i < BTREE_BASES; ++i) {
 	    DEBUGLINE(UNKNOWN, "Checking (ch == " << ch << ") against "
 		      "basenames[" << i << "] == " << basenames[i]);
 	    DEBUGLINE(UNKNOWN, "bases[" << i << "].get_revision() == " <<
@@ -1571,11 +1578,15 @@ void Btree::close() {
 
     for (int j = level; j >= 0; j--) {
 	delete [] C[j].p;
+	C[j].p = 0;
     }
     delete [] split_p;
+    split_p = 0;
 
     delete [] kt.get_address();
+    kt = 0;
     delete [] buffer;
+    buffer = 0;
 }
 
 void
@@ -1689,10 +1700,17 @@ Btree::cancel()
 
 /************ B-tree reading ************/
 
-void
+bool
 Btree::do_open_to_read(bool revision_supplied, quartz_revision_number_t revision_)
 {
     if (!basic_open(revision_supplied, revision_)) {
+	if (revision_supplied) {
+	    // The requested revision was not available.
+	    // This could be because the database was modified underneath us, or
+	    // because a base file is missing.  Return false, and work out what
+	    // the problem was at a higher level.
+	    return false;
+	}
 	throw Xapian::DatabaseOpeningError("Failed to open table for reading");
     }
 
@@ -1715,6 +1733,7 @@ Btree::do_open_to_read(bool revision_supplied, quartz_revision_number_t revision
     }
 
     read_root();
+    return true;
 }
 
 void
@@ -1725,7 +1744,8 @@ Btree::open()
     close();
 
     if (!writable) {
-	do_open_to_read(false, 0);
+	// Any errors are thrown if revision_supplied is false
+	(void)do_open_to_read(false, 0);
 	return;
     }
 
@@ -1741,9 +1761,13 @@ Btree::open(quartz_revision_number_t revision)
     close();
 
     if (!writable) {
-	do_open_to_read(true, revision);
-	AssertEq(revision_number, revision);
-	RETURN(true);
+	if (do_open_to_read(true, revision)) {
+	    AssertEq(revision_number, revision);
+	    RETURN(true);
+	} else {
+	    close();
+	    RETURN(false);
+	}
     }
 
     if (!do_open_to_write(true, revision)) {
