@@ -3,7 +3,8 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,6 +34,7 @@
 #include <xapian/valueiterator.h>
 
 #include "quartz_postlist.h"
+#include "quartz_alldocspostlist.h"
 #include "quartz_termlist.h"
 #include "quartz_positionlist.h"
 #include "quartz_utils.h"
@@ -42,7 +44,9 @@
 #include "quartz_alltermslist.h"
 
 #include <sys/types.h>
-#include <sys/stat.h>
+#include "safesysstat.h"
+#include "safefcntl.h"
+#include "safeunistd.h"
 #ifdef HAVE_SYS_UTSNAME_H
 # include <sys/utsname.h>
 #endif
@@ -50,9 +54,6 @@
 #ifdef __CYGWIN__
 # include "safewindows.h"
 # include <sys/cygwin.h>
-#elif defined __WIN32__
-# include "safewindows.h"
-# define getpid() GetCurrentProcessId()
 #endif
 
 #include <list>
@@ -95,7 +96,7 @@ QuartzDatabase::QuartzDatabase(const string &quartz_dir, int action,
 	if (!dbexists) {
 	    // Catch pre-0.6 Xapian databases and give a better error
 	    if (file_exists(db_dir + "/attribute_DB"))
-		throw Xapian::DatabaseOpeningError("Cannot open database at `" + db_dir + "' - it was created by a pre-0.6 version of Xapian");
+		throw Xapian::DatabaseVersionError("Cannot open database at `" + db_dir + "' - it was created by a pre-0.6 version of Xapian");
 	    throw Xapian::DatabaseOpeningError("Cannot open database at `" + db_dir + "' - it does not exist");
 	}
 	// Can still allow searches even if recovery is needed
@@ -106,7 +107,7 @@ QuartzDatabase::QuartzDatabase(const string &quartz_dir, int action,
 	    if (action == Xapian::DB_OPEN) {
 		// Catch pre-0.6 Xapian databases and give a better error
 		if (file_exists(db_dir + "/attribute_DB"))
-		    throw Xapian::DatabaseOpeningError("Cannot open database at `" + db_dir + "' - it was created by a pre-0.6 version of Xapian");
+		    throw Xapian::DatabaseVersionError("Cannot open database at `" + db_dir + "' - it was created by a pre-0.6 version of Xapian");
 		throw Xapian::DatabaseOpeningError("Cannot open database at `" + db_dir + "' - it does not exist");
 	    }
 
@@ -283,12 +284,13 @@ QuartzDatabase::open_tables_consistent()
 		log.make_entry("Cannot open all tables at revision in record table: " + om_tostring(revision));
 		throw Xapian::DatabaseCorruptError("Cannot open tables at consistent revisions");
 	    }
+	    revision = newrevision;
 	}
     }
 
     if (!fully_opened) {
 	log.make_entry("Cannot open all tables in a consistent state - keep changing too fast, giving up after " + om_tostring(tries) + " attempts");
-	throw Xapian::DatabaseOpeningError("Cannot open tables at stable revision - changing too fast");
+	throw Xapian::DatabaseModifiedError("Cannot open tables at stable revision - changing too fast");
     }
 
     log.make_entry("Opened tables at revision " + om_tostring(revision));
@@ -501,9 +503,9 @@ QuartzDatabase::apply()
 	    // and the buffer to be reinitialised with the old entry count.
 	    cancel();
 	} catch (const Xapian::Error & e) {
-	    log.make_entry("Setting revision number failed: " +
-				       e.get_type() + ": " + e.get_msg() + " " +
-				       e.get_context());
+	    string msg("Setting revision number failed: ");
+	    msg += e.get_description();
+	    log.make_entry(msg);
 	    throw Xapian::DatabaseError("Modifications failed, and cannot set revision numbers in database to a consistent state");
 	}
 	throw;
@@ -560,8 +562,8 @@ QuartzDatabase::get_termfreq(const string & tname) const
     DEBUGCALL(DB, Xapian::doccount, "QuartzDatabase::get_termfreq", tname);
     Assert(!tname.empty());
 
-    QuartzPostList pl(NULL, &postlist_table, NULL, tname);
-    RETURN(pl.get_termfreq());
+    RETURN(postlist_table.get_termfreq(tname));
+    RETURN(postlist_table.get_collection_freq(tname));
 }
 
 Xapian::termcount
@@ -570,10 +572,7 @@ QuartzDatabase::get_collection_freq(const string & tname) const
     DEBUGCALL(DB, Xapian::termcount, "QuartzDatabase::get_collection_freq", tname);
     Assert(!tname.empty());
 
-    Xapian::termcount collfreq = 0; // If not found, this value will be unchanged.
-    QuartzPostList pl(NULL, &postlist_table, NULL, tname);
-    collfreq = pl.get_collection_freq();
-    RETURN(collfreq);
+    RETURN(postlist_table.get_collection_freq(tname));
 }
 
 bool
@@ -595,22 +594,27 @@ QuartzDatabase::has_positions() const
 
 
 LeafPostList *
-QuartzDatabase::do_open_post_list(const string& tname) const
+QuartzDatabase::open_post_list(const string& tname) const
 {
-    DEBUGCALL(DB, LeafPostList *, "QuartzDatabase::do_open_post_list", tname);
-    Assert(!tname.empty());
-
+    DEBUGCALL(DB, LeafPostList *, "QuartzDatabase::open_post_list", tname);
     Xapian::Internal::RefCntPtr<const QuartzDatabase> ptrtothis(this);
-    return(new QuartzPostList(ptrtothis,
+
+    if (tname.empty()) {
+	RETURN(new QuartzAllDocsPostList(ptrtothis,
+					 &termlist_table,
+					 get_doccount()));
+    }
+
+    RETURN(new QuartzPostList(ptrtothis,
 			      &postlist_table,
 			      &positionlist_table,
 			      tname));
 }
 
-LeafTermList *
+TermList *
 QuartzDatabase::open_term_list(Xapian::docid did) const
 {
-    DEBUGCALL(DB, LeafTermList *, "QuartzDatabase::open_term_list", did);
+    DEBUGCALL(DB, TermList *, "QuartzDatabase::open_term_list", did);
     Assert(did != 0);
 
     Xapian::Internal::RefCntPtr<const QuartzDatabase> ptrtothis(this);
@@ -642,9 +646,9 @@ QuartzDatabase::open_position_list(Xapian::docid did,
     if (poslist->get_size() == 0) {
 	// Check that term / document combination exists.
 	// If the doc doesn't exist, this will throw Xapian::DocNotFoundError:
-	AutoPtr<LeafTermList> ltl(open_term_list(did));
-	ltl->skip_to(tname);
-	if (ltl->at_end() || ltl->get_termname() != tname)
+	AutoPtr<TermList> tl(open_term_list(did));
+	tl->skip_to(tname);
+	if (tl->at_end() || tl->get_termname() != tname)
 	    throw Xapian::RangeError("Can't open position list: requested term is not present in document.");
     }
 
@@ -652,12 +656,12 @@ QuartzDatabase::open_position_list(Xapian::docid did,
 }
 
 TermList *
-QuartzDatabase::open_allterms() const
+QuartzDatabase::open_allterms(const string & prefix) const
 {
     DEBUGCALL(DB, TermList *, "QuartzDatabase::open_allterms", "");
     AutoPtr<Bcursor> pl_cursor(postlist_table.cursor_get());
     RETURN(new QuartzAllTermsList(Xapian::Internal::RefCntPtr<const QuartzDatabase>(this),
-				  pl_cursor, postlist_table.get_entry_count()));
+				  pl_cursor, postlist_table.get_entry_count(), prefix));
 }
 
 size_t QuartzWritableDatabase::flush_threshold = 0;
@@ -717,6 +721,9 @@ QuartzWritableDatabase::add_document(const Xapian::Document & document)
 {
     DEBUGCALL(DB, Xapian::docid,
 	      "QuartzWritableDatabase::add_document", document);
+    // Make sure the docid counter doesn't overflow.
+    if (lastdocid == Xapian::docid(-1))
+	throw Xapian::DatabaseError("Run out of docids - you'll have to use copydatabase to eliminate any gaps before you can add more documents");
     // Use the next unused document ID.
     RETURN(add_document_(++lastdocid, document));
 }
@@ -783,7 +790,8 @@ QuartzWritableDatabase::add_document_(Xapian::docid did,
 		new_doclen, false);
 
 	// Set the new document length
-	doclens.insert(make_pair(did, new_doclen));
+	Assert(doclens.find(did) == doclens.end());
+	doclens[did] = new_doclen;
 	total_length += new_doclen;
     } catch (...) {
 	// If an error occurs while adding a document, or doing any other
@@ -816,10 +824,12 @@ QuartzWritableDatabase::delete_document(Xapian::docid did)
     DEBUGCALL(DB, void, "QuartzWritableDatabase::delete_document", did);
     Assert(did != 0);
 
-    try {
-	// Remove the record.
-	database_ro.record_table.delete_record(did);
+    // Remove the record.  If this fails, just propagate the exception since
+    // the state should still be consistent (most likely it's
+    // DocNotFoundError).
+    database_ro.record_table.delete_record(did);
 
+    try {
 	// Remove the values
 	database_ro.value_table.delete_all_values(did);
 
@@ -868,6 +878,9 @@ QuartzWritableDatabase::delete_document(Xapian::docid did)
 
 	// Remove the termlist.
 	database_ro.termlist_table.delete_termlist(did);
+
+	// Remove the new doclength.
+	doclens.erase(did);
     } catch (...) {
 	// If an error occurs while deleting a document, or doing any other
 	// transaction, the modifications so far must be cleared before
@@ -1000,14 +1013,13 @@ QuartzWritableDatabase::replace_document(Xapian::docid did,
 		    j->second.insert(make_pair(did, make_pair('A', wdf)));
 		}
 
-		// FIXME : this might not work if we replace a positionlist
-		// with itself (e.g. if a document is replaced with itself
-		// with just the values changed)
-		database_ro.positionlist_table.delete_positionlist(did, tname);
-		if (term.positionlist_begin() != term.positionlist_end()) {
+		PositionIterator it = term.positionlist_begin();
+		PositionIterator it_end = term.positionlist_end();
+		if (it != it_end) {
 		    database_ro.positionlist_table.set_positionlist(
-			did, tname,
-			term.positionlist_begin(), term.positionlist_end());
+			did, tname, it, it_end);
+		} else {
+		    database_ro.positionlist_table.delete_positionlist(did, tname);
 		}
 	    }
 	}
@@ -1018,7 +1030,7 @@ QuartzWritableDatabase::replace_document(Xapian::docid did,
 		new_doclen, false);
 
 	// Set the new document length
-	doclens.insert(make_pair(did, new_doclen));
+	doclens[did] = new_doclen;
 	total_length += new_doclen;
     } catch (const Xapian::DocNotFoundError &) {
 	(void)add_document_(did, document);
@@ -1108,10 +1120,16 @@ QuartzWritableDatabase::has_positions() const
 
 
 LeafPostList *
-QuartzWritableDatabase::do_open_post_list(const string& tname) const
+QuartzWritableDatabase::open_post_list(const string& tname) const
 {
-    DEBUGCALL(DB, LeafPostList *, "QuartzWritableDatabase::do_open_post_list", tname);
-    Assert(!tname.empty());
+    DEBUGCALL(DB, LeafPostList *, "QuartzWritableDatabase::open_post_list", tname);
+    Xapian::Internal::RefCntPtr<const QuartzWritableDatabase> ptrtothis(this);
+
+    if (tname.empty()) {
+	RETURN(new QuartzAllDocsPostList(ptrtothis,
+					 &database_ro.termlist_table,
+					 get_doccount()));
+    }
 
     // Need to flush iff we've got buffered changes to this term's postlist.
     map<string, map<docid, pair<char, termcount> > >::const_iterator j;
@@ -1122,17 +1140,16 @@ QuartzWritableDatabase::do_open_post_list(const string& tname) const
 	do_flush_const();
     }
 
-    Xapian::Internal::RefCntPtr<const QuartzWritableDatabase> ptrtothis(this);
-    return(new QuartzPostList(ptrtothis,
+    RETURN(new QuartzPostList(ptrtothis,
 			      &database_ro.postlist_table,
 			      &database_ro.positionlist_table,
 			      tname));
 }
 
-LeafTermList *
+TermList *
 QuartzWritableDatabase::open_term_list(Xapian::docid did) const
 {
-    DEBUGCALL(DB, LeafTermList *, "QuartzWritableDatabase::open_term_list",
+    DEBUGCALL(DB, TermList *, "QuartzWritableDatabase::open_term_list",
 	      did);
     Assert(did != 0);
 
@@ -1166,9 +1183,9 @@ QuartzWritableDatabase::open_position_list(Xapian::docid did,
     if (poslist->get_size() == 0) {
 	// Check that term / document combination exists.
 	// If the doc doesn't exist, this will throw Xapian::DocNotFoundError:
-	AutoPtr<LeafTermList> ltl(open_term_list(did));
-	ltl->skip_to(tname);
-	if (ltl->at_end() || ltl->get_termname() != tname)
+	AutoPtr<TermList> tl(open_term_list(did));
+	tl->skip_to(tname);
+	if (tl->at_end() || tl->get_termname() != tname)
 	    throw Xapian::RangeError("Can't open position list: requested term is not present in document.");
     }
 
@@ -1176,7 +1193,7 @@ QuartzWritableDatabase::open_position_list(Xapian::docid did,
 }
 
 TermList *
-QuartzWritableDatabase::open_allterms() const
+QuartzWritableDatabase::open_allterms(const string & prefix) const
 {
     DEBUGCALL(DB, TermList *, "QuartzWritableDatabase::open_allterms", "");
     if (transaction_active())
@@ -1186,7 +1203,7 @@ QuartzWritableDatabase::open_allterms() const
     QuartzPostListTable *t = &database_ro.postlist_table;
     AutoPtr<Bcursor> pl_cursor(t->cursor_get());
     RETURN(new QuartzAllTermsList(Xapian::Internal::RefCntPtr<const QuartzWritableDatabase>(this),
-				  pl_cursor, t->get_entry_count()));
+				  pl_cursor, t->get_entry_count(), prefix));
 }
 
 void
