@@ -1,8 +1,7 @@
 /* quartz_postlist.cc: Postlists in quartz databases
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005 Olly Betts
+ * Copyright 2002,2003,2004,2005,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,9 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #include <config.h>
@@ -167,6 +165,32 @@ read_start_of_first_chunk(const char ** posptr,
     RETURN(did);
 }
 
+Xapian::doccount
+QuartzPostListTable::get_termfreq(const string & term) const
+{
+    string key, tag;
+    make_key(term, key);
+    if (!get_exact_entry(key, tag)) return 0;
+
+    Xapian::doccount termfreq;
+    const char * p = tag.data();
+    QuartzPostList::read_number_of_entries(&p, p + tag.size(), &termfreq, NULL);
+    return termfreq;
+}
+
+Xapian::termcount
+QuartzPostListTable::get_collection_freq(const string & term) const
+{
+    string key, tag;
+    make_key(term, key);
+    if (!get_exact_entry(key, tag)) return 0;
+
+    Xapian::termcount collfreq;
+    const char * p = tag.data();
+    QuartzPostList::read_number_of_entries(&p, p + tag.size(), NULL, &collfreq);
+    return collfreq;
+}
+
 static inline void read_did_increase(const char ** posptr,
 			      const char * end,
 			      Xapian::docid * did_ptr)
@@ -657,7 +681,6 @@ QuartzPostList::QuartzPostList(Xapian::Internal::RefCntPtr<const Xapian::Databas
     int found = cursor->find_entry(key);
     if (!found) {
 	number_of_entries = 0;
-	collection_freq = 0;
 	is_at_end = true;
 	pos = 0;
 	end = 0;
@@ -669,8 +692,7 @@ QuartzPostList::QuartzPostList(Xapian::Internal::RefCntPtr<const Xapian::Databas
     pos = cursor->current_tag.data();
     end = pos + cursor->current_tag.size();
 
-    did = read_start_of_first_chunk(&pos, end,
-			      &number_of_entries, &collection_freq);
+    did = read_start_of_first_chunk(&pos, end, &number_of_entries, NULL);
     first_did_in_chunk = did;
     last_did_in_chunk = read_start_of_chunk(&pos, end, first_did_in_chunk,
 	    				    &is_last_chunk);
@@ -826,11 +848,8 @@ QuartzPostList::move_to_chunk_containing(Xapian::docid desired_did)
 	// In first chunk
 #ifdef XAPIAN_DEBUG
 	Xapian::doccount old_number_of_entries = number_of_entries;
-	Xapian::termcount old_collection_freq = collection_freq;
-	did = read_start_of_first_chunk(&pos, end, &number_of_entries,
-					&collection_freq);
+	did = read_start_of_first_chunk(&pos, end, &number_of_entries, NULL);
 	Assert(old_number_of_entries == number_of_entries);
-	Assert(old_collection_freq == collection_freq);
 #else
 	did = read_start_of_first_chunk(&pos, end, 0, 0);
 #endif
@@ -880,25 +899,23 @@ QuartzPostList::skip_to(Xapian::docid desired_did, Xapian::weight w_min)
     have_started = true;
 
     // Don't skip back, and don't need to do anything if already there.
-    if (desired_did <= did) RETURN(NULL);
+    if (is_at_end || desired_did <= did) RETURN(NULL);
 
     // Move to correct chunk
     if (!current_chunk_contains(desired_did)) {
 	move_to_chunk_containing(desired_did);
-	// Might be at_end now - this is why we need the test before moving
+	// Might be at_end now, so we need to check before trying to move
 	// forward in chunk.
+	if (is_at_end) RETURN(NULL);
     }
 
     // Move to correct position in chunk
-    if (!is_at_end) {
+    bool have_document = move_forward_in_chunk_to_at_least(desired_did);
 #ifdef XAPIAN_DEBUG
-	bool have_document =
+    Assert(have_document);
 #else
-		(void)
+    (void)have_document;
 #endif
-		move_forward_in_chunk_to_at_least(desired_did);
-	Assert(have_document);
-    }
 
     DEBUGLINE(DB, string("Skipped to ") <<
 	      (is_at_end ? string("end.") : string("docid, wdf, doclength = ") +
