@@ -2,7 +2,7 @@
  * or btree.  Also check the structures inside the tables.
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,11 @@
  */
 
 #include <config.h>
+
+// We have to use the deprecated Quartz::open() method.
+#define XAPIAN_DEPRECATED(D) D
+#include <xapian.h>
+
 #include <iostream>
 
 #include "autoptr.h"
@@ -29,8 +34,6 @@
 #include "quartz_types.h"
 #include "quartz_utils.h"
 
-#include <xapian.h>
-
 using namespace std;
 
 #define PROG_NAME "quartzcheck"
@@ -97,17 +100,18 @@ main(int argc, char **argv)
     try {
 	size_t errors = 0;
 	struct stat sb;
-	string record_DB(argv[1]);
-	record_DB += "/record_DB";
-	if (stat(record_DB.c_str(), &sb) == 0) {
+	string meta_file(argv[1]);
+	meta_file += "/meta";
+	if (stat(meta_file.c_str(), &sb) == 0) {
 	    // Check a whole quartz database directory.
 	    try {
-		Xapian::Database db(argv[1]);
+		Xapian::Database db = Xapian::Quartz::open(argv[1]);
 		doclens.reserve(db.get_lastdocid());
 	    } catch (const Xapian::Error & e) {
 		// Ignore so we can check a database too broken to open.
 		cout << "Database couldn't be opened for reading: "
-		     << e.get_msg() << "\nContinuing check anyway" << endl;
+		     << e.get_description()
+		     << "\nContinuing check anyway" << endl;
 		++errors;
 	    }
 	    // Assume it's a quartz directory and try to check all the btrees
@@ -138,7 +142,7 @@ main(int argc, char **argv)
 	cerr << argv[0] << ": " << error << endl;
 	exit(1);
     } catch (const Xapian::Error &error) {
-	cerr << argv[0] << ": " << error.get_msg() << endl;
+	cerr << argv[0] << ": " << error.get_description() << endl;
 	exit(1);
     } catch (...) {
 	cerr << argv[0] << ": Unknown exception" << endl;
