@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,8 @@
 
 #include <config.h>
 
+#include <stdlib.h>
+
 #include "safeerrno.h"
 
 #include <iostream>
@@ -52,6 +54,7 @@ const int MSECS_ACTIVE_TIMEOUT_DEFAULT = 15000;
 #define OPT_VERSION 2
 
 static const struct option opts[] = {
+    {"interface",	required_argument,	0, 'I'},
     {"port",		required_argument,	0, 'p'},
     {"active-timeout",	required_argument,	0, 'a'},
     {"idle-timeout",	required_argument,	0, 'i'},
@@ -68,6 +71,8 @@ static void show_usage() {
     cout << "Usage: "PROG_NAME" [OPTIONS] DATABASE_DIRECTORY...\n\n"
 "Options:\n"
 "  --port PORTNUM          listen on port PORTNUM for connections (no default)\n"
+"  --interface ADDRESS     listen on the interface associated with name or\n"
+"                          address ADDRESS (default is all interfaces)\n"
 "  --idle-timeout MSECS    set timeout for idle connections (default " << MSECS_IDLE_TIMEOUT_DEFAULT << "ms)\n"
 "  --active-timeout MSECS  set timeout for active connections (default " << MSECS_ACTIVE_TIMEOUT_DEFAULT << "ms)\n"
 "  --timeout MSECS         set both timeout values\n"
@@ -79,6 +84,7 @@ static void show_usage() {
 }
 
 int main(int argc, char **argv) {
+    string host;
     int port = 0;
     int msecs_active_timeout = MSECS_ACTIVE_TIMEOUT_DEFAULT;
     int msecs_idle_timeout   = MSECS_IDLE_TIMEOUT_DEFAULT;
@@ -89,7 +95,7 @@ int main(int argc, char **argv) {
     bool syntax_error = false;
 
     int c;
-    while ((c = gnu_getopt_long(argc, argv, "p:a:i:t:oq", opts, NULL)) != EOF) {
+    while ((c = gnu_getopt_long(argc, argv, "I:p:a:i:t:oq", opts, NULL)) != EOF) {
 	switch (c) {
 	    case OPT_HELP:
 		cout << PROG_NAME" - "PROG_DESC"\n\n";
@@ -98,6 +104,9 @@ int main(int argc, char **argv) {
 	    case OPT_VERSION:
 		cout << PROG_NAME" - "PACKAGE_STRING << endl;
 		exit(0);
+	    case 'I':
+                host.assign(optarg);
+	        break;
 	    case 'p':
                 port = atoi(optarg);
 	        break;
@@ -141,53 +150,44 @@ int main(int argc, char **argv) {
     }
 
     try {
+	vector<string> dbnames;
+	// Try to open the database(s) so we report problems now instead of
+	// waiting for the first connection.
 	if (writable) {
 	    Xapian::WritableDatabase db(argv[optind], Xapian::DB_CREATE_OR_OPEN);
-
-	    if (verbose)
-		cout << "Starting writable server on port " << port << endl;
-
-	    TcpServer server(db, port, msecs_active_timeout,
-			     msecs_idle_timeout, verbose);
-
-	    if (verbose)
-		cout << "Listening..." << endl;
-
-	    register_user_weighting_schemes(server);
-
-	    if (one_shot) {
-		server.run_once();
-	    } else {
-		server.run();
-	    }
+	    dbnames.push_back(argv[optind]);
 	} else {
-	    Xapian::Database db;
 	    while (argv[optind]) {
-		db.add_database(Xapian::Database(argv[optind++]));
-	    }
-
-	    if (verbose)
-		cout << "Starting server on port " << port << endl;
-
-	    TcpServer server(db, port, msecs_active_timeout,
-			     msecs_idle_timeout, verbose);
-
-	    if (verbose)
-		cout << "Listening..." << endl;
-
-	    register_user_weighting_schemes(server);
-
-	    if (one_shot) {
-		server.run_once();
-	    } else {
-		server.run();
+		dbnames.push_back(argv[optind]);
+		Xapian::Database db(argv[optind++]);
 	    }
 	}
+
+	if (verbose) {
+	    cout << "Starting";
+	    if (writable)
+		cout << " writable";
+	    cout << " server on";
+	    if (!host.empty())
+		cout << " host " << host << ",";
+	    cout << " port " << port << endl;
+	}
+
+	TcpServer server(dbnames, host, port, msecs_active_timeout,
+			 msecs_idle_timeout, writable, verbose);
+
+	if (verbose)
+	    cout << "Listening..." << endl;
+
+	register_user_weighting_schemes(server);
+
+	if (one_shot) {
+	    server.run_once();
+	} else {
+	    server.run();
+	}
     } catch (const Xapian::Error &e) {
-	cerr << e.get_type() << ": " << e.get_msg();
-	if (e.get_errno()) cerr << " (errno:" << strerror(e.get_errno()) << ")";
-	cerr << endl;
-	if (e.get_errno() == EADDRINUSE) exit(69); // EX_UNAVAILABLE
+	cerr << e.get_description() << endl;
 	exit(1);
     } catch (const exception &e) {
 	cerr << "Caught standard exception: " << e.what() << endl;
