@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,12 +33,10 @@
 #include <xapian/positioniterator.h>
 #include <xapian/termiterator.h>
 #include "omdebug.h"
-#include "emptypostlist.h"
 
 using namespace std;
 
 class LeafPostList;
-class LeafTermList;
 class RemoteDatabase;
 
 typedef Xapian::TermIterator::Internal TermList;
@@ -82,18 +81,6 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	void dtor_called();
 
-	/** Method definied by subclass to actually open a posting list.
-	 *
-	 *  This is a list of all the documents which contain a given term.
-	 *
-	 *  @param tname  The term whose posting list is being requested.
-	 *
-	 *  @return       A pointer to the newly created posting list.
-	 *                This object must be deleted by the caller after
-	 *                use.
-	 */
-	virtual LeafPostList * do_open_post_list(const string & tname) const = 0;
-
     public:
 	/** Destroy the database.
 	 *
@@ -123,7 +110,7 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 
 	/** Return the last used document id of this (sub) database.
 	 */
-	virtual Xapian::docid get_lastdocid() const;
+	virtual Xapian::docid get_lastdocid() const = 0;
 
 	/** Return the average length of a document in this (sub) database.
 	 *
@@ -179,24 +166,20 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 
 	/** Open a posting list.
 	 *
+	 *  Method defined by subclass to open a posting list.
 	 *  This is a list of all the documents which contain a given term.
 	 *
 	 *  @param tname  The term whose posting list is being requested.
 	 *
 	 *  @return       A pointer to the newly created posting list.
+	 *		  If the term doesn't exist, a new EmptyPostList
+	 *		  object (or an object which behaves the same way) is
+	 *		  returned instead, which makes it easier
+	 *		  to implement a search over multiple databases.
 	 *                This object must be deleted by the caller after
 	 *                use.
 	 */
-	LeafPostList * open_post_list(const string & tname) const {
-	    if (!term_exists(tname)) {
-		DEBUGLINE(MATCH, tname + " is not in database.");
-		// Term doesn't exist in this database.  However, we create
-		// a (empty) postlist for it to help make distributed searching
-		// cleaner (term might exist in other databases).
-		return new EmptyPostList();
-	    }
-	    return do_open_post_list(tname);
-	}
+	virtual LeafPostList * open_post_list(const string & tname) const = 0;
 
 	/** Open a term list.
 	 *
@@ -208,17 +191,18 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 *                This object must be deleted by the caller after
 	 *                use.
 	 */
-	virtual LeafTermList * open_term_list(Xapian::docid did) const = 0;
+	virtual TermList * open_term_list(Xapian::docid did) const = 0;
 
 	/** Open an allterms list.
 	 *
 	 *  This is a list of all the terms in the database
 	 *
+	 *  @param prefix The prefix to restrict the terms to.
 	 *  @return       A pointer to the newly created allterms list.
 	 *                This object must be deleted by the caller after
 	 *                use.
 	 */
-	virtual TermList * open_allterms() const = 0;
+	virtual TermList * open_allterms(const string & prefix) const = 0;
 
 	/** Open a position list for the given term in the given document.
 	 *
@@ -253,6 +237,75 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	virtual Xapian::Document::Internal *
 	open_document(Xapian::docid did, bool lazy = false) const = 0;
 
+	/** Create a termlist tree from trigrams of @a word.
+	 *
+	 *  If there are no trigrams, returns NULL.
+	 */
+	virtual TermList * open_spelling_termlist(const string & word) const;
+
+	/** Return a termlist which returns the words which are spelling
+	 *  correction targets.
+	 *
+	 *  If there are no spelling correction targets, returns NULL.
+	 */
+	virtual TermList * open_spelling_wordlist() const;
+
+	/** Return the number of times @a word was added as a spelling. */
+	virtual Xapian::doccount get_spelling_frequency(const string & word) const;
+
+	/** Add a word to the spelling dictionary.
+	 *
+	 *  If the word is already present, its frequency is increased.
+	 *
+	 *  @param word	    The word to add.
+	 *  @param freqinc  How much to increase its frequency by.
+	 */
+	virtual void add_spelling(const string & word,
+				  Xapian::termcount freqinc) const;
+
+	/** Remove a word from the spelling dictionary.
+	 *
+	 *  The word's frequency is decreased, and if would become zero or less
+	 *  then the word is removed completely.
+	 *
+	 *  @param word	    The word to remove.
+	 *  @param freqdec  How much to decrease its frequency by.
+	 */
+	virtual void remove_spelling(const string & word,
+				     Xapian::termcount freqdec) const;
+
+	/** Open a termlist returning synonyms for a term.
+	 *
+	 *  If @a term has no synonyms, returns NULL.
+	 */
+	virtual TermList * open_synonym_termlist(const string & term) const;
+
+	/** Open a termlist returning each term which has synonyms.
+	 *
+	 *  @param prefix   If non-empty, only terms with this prefix are
+	 *		    returned.
+	 */
+	virtual TermList * open_synonym_keylist(const string & prefix) const;
+
+	/** Add a synonym for a term.
+	 *
+	 *  If @synonym is already a synonym for @a term, then no action is
+	 *  taken.
+	 */
+	virtual void add_synonym(const string & term, const string & synonym) const;
+
+	/** Remove a synonym for a term.
+	 *
+	 *  If @a synonym isn't a synonym for @a term, then no action is taken.
+	 */
+	virtual void remove_synonym(const string & term, const string & synonym) const;
+
+	/** Clear all synonyms for a term.
+	 *
+	 *  If @a term has no synonyms, no action is taken.
+	 */
+	virtual void clear_synonyms(const string & term) const;
+
 	/** Reopen the database to the latest available revision.
 	 *
 	 *  Database backends which don't support simultaneous update and
@@ -303,11 +356,25 @@ class Database::Internal : public Xapian::Internal::RefCntBase {
 	 */
 	virtual void delete_document(Xapian::docid did);
 
+	/** Delete any documents indexed by a term from the database.
+	 *
+	 *  See WritableDatabase::delete_document() for more information.
+	 */
+	virtual void delete_document(const std::string & unique_term);
+
 	/** Replace a given document in the database.
 	 *
 	 *  See WritableDatabase::replace_document() for more information.
 	 */
-	virtual void replace_document(Xapian::docid did, const Xapian::Document & document);
+	virtual void replace_document(Xapian::docid did,
+				      const Xapian::Document & document);
+
+	/** Replace any documents matching a term.
+	 *
+	 *  See WritableDatabase::replace_document() for more information.
+	 */
+	virtual Xapian::docid replace_document(const std::string & unique_term,
+					       const Xapian::Document & document);
 
 	/** Request and later collect a document from the database.
 	 *  Multiple documents can be requested with request_document(),
