@@ -1,7 +1,7 @@
 /* multimatch.h: class for performing a match
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -58,10 +58,6 @@ class MultiMatch
 
 	bool sort_value_forward;
 
-	time_t bias_halflife;
-
-	Xapian::weight bias_weight;
-
 	/// ErrorHandler
 	Xapian::ErrorHandler * errorhandler;
 
@@ -73,6 +69,9 @@ class MultiMatch
 	 */
         bool recalculate_w_max;
 
+	/** Is each sub-database remote? */
+	vector<bool> is_remote;
+
 	/// get the collapse key
 	string get_collapse_key(PostList *pl,
 				Xapian::docid did, Xapian::valueno keyno,
@@ -114,8 +113,6 @@ class MultiMatch
 		   Xapian::valueno sort_key_,
 		   Xapian::Enquire::Internal::sort_setting sort_by_,
 		   bool sort_value_forward_,
-		   time_t bias_halflife_,
-		   Xapian::weight bias_weight_,
 		   Xapian::ErrorHandler * errorhandler,
 		   StatsGatherer * gatherer_,
 		   const Xapian::Weight *wtscheme);
@@ -124,7 +121,8 @@ class MultiMatch
 		      Xapian::doccount maxitems,
 		      Xapian::doccount check_at_least,
 		      Xapian::MSet & mset,
-		      const Xapian::MatchDecider * mdecider);
+		      const Xapian::MatchDecider * mdecider,
+		      const Xapian::MatchDecider * matchspy);
 
 	/** Called by postlists to indicate that they've rearranged themselves
 	 *  and the maxweight now possible is smaller.
