@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,20 +27,15 @@
 #ifndef OM_HGUARD_OMDEBUG_H
 #define OM_HGUARD_OMDEBUG_H
 
-#include "omassert.h"
-
 #ifdef XAPIAN_DEBUG_VERBOSE
 
+#include <xapian/visibility.h>
+
 #include "omstringstream.h"
 
 #include <vector>
 
-#ifdef __WIN32__
-# include "safewindows.h"
-# define getpid() GetCurrentProcessId()
-#else
-# include <unistd.h>
-#endif
+#include "safeunistd.h"
 
 /** The types of debug output.  These are specified within a DEBUGMSG in
  *  the code by the final portion of the name: ie, UNKNOWN, LOCK, etc...
@@ -99,6 +94,12 @@ enum om_debug_types {
     /** Type for messages to do with the remote backend. */
     OM_DEBUG_REMOTE,
 
+    /** Type for messages from the lemon generated QueryParser code. */
+    OM_DEBUG_QUERYPARSER,
+
+    /** Type for messages to do with spelling correction. */
+    OM_DEBUG_SPELLING,
+
     /** A value equal to the number of known types.
      */
     OM_DEBUG_NUMTYPES
@@ -106,7 +107,7 @@ enum om_debug_types {
 
 /** Class to manage verbose debugging output
  */
-class OmDebug {
+class XAPIAN_VISIBILITY_DEFAULT OmDebug {
     private:
 	/// Copying not allowed
 	OmDebug(OmDebug &);
@@ -144,6 +145,7 @@ class OmDebug {
 	~OmDebug();
 };
 
+XAPIAN_VISIBILITY_DEFAULT
 extern OmDebug om_debug;
 
 /** Display a debugging message, if it is of a desired type. */
@@ -190,6 +192,14 @@ class OmDebugCall {
         }
 };
 
+// 2nd level of stringize definition not needed for the use we put this
+// to in this file (since we always use it within a macro here) but
+// is required in general  (#N doesn't work outside a macro definition)
+#ifndef STRINGIZE
+# define STRINGIZE(N) _STRINGIZE(N)
+# define _STRINGIZE(N) #N
+#endif
+
 /** Display a message indicating that a method has been called, and another
  *  message when the method ends.
  */
