@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,79 +23,55 @@
 #ifndef OM_HGUARD_UTILS_H
 #define OM_HGUARD_UTILS_H
 
+#include <xapian/visibility.h>
+
 #include <string>
 using std::string;
 
 #include <stdlib.h>
-#include <sys/stat.h>
 #include <sys/types.h>
-#include "safefcntl.h"
-#ifdef _MSC_VER
-# include <direct.h>
-# include <io.h>
-#else
-# include <unistd.h>
-#endif
-#include <ctype.h>
-
-#ifdef _MSC_VER
-
-#define S_ISREG(m) (((m)&_S_IFMT) == _S_IFREG)
-#define S_ISDIR(m) (((m)&_S_IFMT) == _S_IFDIR)
-
-// MSVC needs this to get SSIZE_T defined.
-#include "safewindows.h"
-
-#undef ssize_t // In case configure already defined it.
-#define ssize_t SSIZE_T
-
-#endif
+#include "safesysstat.h"
+#include "safeunistd.h"
 
 /// Convert a string to a string!
 inline string om_tostring(const string &s) { return s; }
 
 /// Convert an integer to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(int a);
 
 /// Convert an unsigned integer to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(unsigned int a);
 
 /// Convert a long integer to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(long int a);
 
 /// Convert an unsigned long integer to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(unsigned long int a);
 
+#ifdef __WIN32__
+/// Convert a 64 bit integer to a string
+string om_tostring(__int64 a);
+#endif
+
 /// Convert a double to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(double a);
 
 /// Convert a bool to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(bool a);
 
 /// Convert a pointer to a string
+XAPIAN_VISIBILITY_DEFAULT
 string om_tostring(const void * a);
 
-///////////////////////////////////////////
-// Mapping of types as strings to enums  //
-///////////////////////////////////////////
-
-struct StringAndValue {
-    const char * name;
-    int value;
-};
-
-/** Get the value associated with the given string.  If the string
- *  isn't found, the value returned is the value in the terminating
- *  object (which has a zero length string).
- *
- *  Note: this just uses a list of entries, and searches linearly
- *  through them.  Could at make this do a binary chop, but probably
- *  not worth doing so, unless list gets large.
- */
-int map_string_to_value(const StringAndValue * haystack, const string & needle);
-
 /** Return true if the file fname exists.
  */
+XAPIAN_VISIBILITY_DEFAULT
 bool file_exists(const string &fname);
 
 /// Allow atoi to work directly on C++ strings.
@@ -115,37 +91,15 @@ inline int link(const string &o, const string &n) {
 #endif
 
 /// Allow mkdir to work directly on C++ strings.
-#ifdef __WIN32__
-inline int mkdir(const string &filename, int /*mode*/) {
-    return _mkdir(filename.c_str());
-}
-#else
 inline int mkdir(const string &filename, mode_t mode) {
     return mkdir(filename.c_str(), mode);
 }
-#endif
 
 /// Allow stat to work directly on C++ strings.
 inline int stat(const string &filename, struct stat *buf) {
     return stat(filename.c_str(), buf);
 }
 
-/// Touch a file.
-inline void touch(const string &filename) {
-   int fd = open(filename.c_str(), O_CREAT|O_WRONLY, 0644);
-   if (fd >= 0) close(fd);
-}
-
-/// Remove a directory and contents.
-void rmdir(const string &filename);
-
-#ifdef __WIN32__
-inline unsigned int sleep(unsigned int secs) {
-    _sleep(secs * 1000);
-    return 0;
-}
-#endif
-
 // Like C's isXXXXX() but:
 //  (a) always work in the C locale
 //  (b) handle signed char as well as unsigned char
@@ -161,8 +115,11 @@ namespace Xapian {
 	const unsigned char IS_HEX   = 0x08;
 	const unsigned char IS_SIGN  = 0x10;
 	const unsigned char IS_SPACE = 0x20;
+	XAPIAN_VISIBILITY_DEFAULT
 	extern const unsigned char is_tab[];
+	XAPIAN_VISIBILITY_DEFAULT
 	extern const unsigned char lo_tab[];
+	XAPIAN_VISIBILITY_DEFAULT
 	extern const unsigned char up_tab[];
     }
 }
