@@ -1,6 +1,6 @@
 /* quest.cc - Command line search tool using Xapian::QueryParser.
  *
- * Copyright (C) 2004,2005,2006 Olly Betts
+ * Copyright (C) 2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,8 @@
 
 #include <xapian.h>
 
+#include <stdlib.h>
+
 #include <iostream>
 
 #include "gnu_getopt.h"
@@ -49,6 +51,8 @@ static void show_usage() {
 "Options:\n"
 "  -d, --db=DIRECTORY  database to search (multiple databases may be specified)\n"
 "  -m, --msize=MSIZE   maximum number of matches to return\n"
+"  -s, --stemmer=LANG  set the stemming language, the default is 'english'\n"
+"                      (pass 'none' to disable stemming)\n"
 "  -h, --help          display this help and exit\n"
 "  -v, --version       output version information and exit\n";
 }
@@ -59,12 +63,14 @@ main(int argc, char **argv)
     static const struct option long_opts[] = {
 	{ "db",		required_argument, 0, 'd' },
 	{ "msize",	required_argument, 0, 'm' },
+	{ "stemmer",	required_argument, 0, 's' },
 	{ "help",	no_argument, 0, 'h' },
 	{ "version",	no_argument, 0, 'v' },
 	{ NULL,		0, 0, 0}
     };
 
     Xapian::SimpleStopper mystopper(sw, sw + sizeof(sw) / sizeof(sw[0]));
+    Xapian::Stem stemmer("english");
     int msize = 10;
 
     bool have_database = false;
@@ -73,7 +79,7 @@ main(int argc, char **argv)
 	Xapian::Database db;
 
 	int c;
-	while ((c = gnu_getopt_long(argc, argv, "hvm:d:", long_opts, 0)) != EOF)
+	while ((c = gnu_getopt_long(argc, argv, "hvm:d:s:", long_opts, 0)) != EOF)
 	{
 	    switch (c) {
 		case 'm':
@@ -83,6 +89,16 @@ main(int argc, char **argv)
 		    db.add_database(Xapian::Database(optarg));
 		    have_database = true;
 		    break;
+		case 's':
+		    try {
+			stemmer = Xapian::Stem(optarg);
+		    } catch (const Xapian::Error &) {
+			cerr << "Unknown stemming language '" << optarg << "'.\n";
+			cerr << "Available language names are: "
+			    << Xapian::Stem::get_available_languages() << endl;
+			exit(1);
+		    }
+		    break;
 		case 'v':
 		    cout << PROG_NAME" - "PACKAGE_STRING << endl;
 		    exit(0);
@@ -108,7 +124,7 @@ main(int argc, char **argv)
 	    Xapian::QueryParser parser;
 	    parser.set_database(db);
 	    parser.set_default_op(Xapian::Query::OP_OR);
-	    parser.set_stemmer(Xapian::Stem("english"));
+	    parser.set_stemmer(stemmer);
 	    parser.set_stemming_strategy(Xapian::QueryParser::STEM_SOME);
 	    parser.set_stopper(&mystopper);
 	    enquire.set_query(parser.parse_query(argv[optind]));
@@ -125,7 +141,7 @@ main(int argc, char **argv)
 	}
 	cout << flush;
     } catch (const Xapian::Error & err) {
-	cout << err.get_msg() << endl;
+	cout << err.get_description() << endl;
 	exit(1);
     }
 }
