@@ -3,7 +3,8 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,6 +33,7 @@
 #include <xapian/positioniterator.h>
 #include <xapian/postingiterator.h>
 #include <xapian/termiterator.h>
+#include <xapian/visibility.h>
 
 /// The Xapian library lives in the Xapian namespace.
 namespace Xapian {
@@ -47,11 +49,15 @@ class Document;
  *
  *  @exception DatabaseOpeningError may be thrown if the database cannot
  *  be opened (for example, a required file cannot be found).
+ *
+ *  @exception DatabaseVersionError may be thrown if the database is in an
+ *  unsupported format (for example, created by a newer version of Xapian
+ *  which uses an incompatible format).
  */
-class Database {
+class XAPIAN_VISIBILITY_DEFAULT Database {
     public:
 	class Internal;
-	/// @internal Reference counted internals.
+	/// @private @internal Reference counted internals.
 	std::vector<Xapian::Internal::RefCntPtr<Internal> > internal;
 
 	/** Add an existing database (or group of databases) to those
@@ -70,9 +76,9 @@ class Database {
 	 *
 	 * @param path directory that the database is stored in.
 	 */
-	Database(const std::string &path);
+	explicit Database(const std::string &path);
 
-	/** @internal Create a Database from its internals.
+	/** @private @internal Create a Database from its internals.
 	 */
 	explicit Database(Internal *internal);
 
@@ -108,6 +114,11 @@ class Database {
 
 	/** An iterator pointing to the start of the postlist
 	 *  for a given term.
+	 *
+	 *  If the term name is the empty string, the iterator returned
+	 *  will list all the documents in the database.  Such an iterator
+	 *  will always return a WDF value of 1, since there is no obvious
+	 *  meaning for this quantity in this case.
 	 */
 	PostingIterator postlist_begin(const std::string &tname) const;
 
@@ -152,6 +163,25 @@ class Database {
 	    return TermIterator(NULL);
 	}
 
+	/** An iterator which runs across all terms with a given prefix.
+	 *
+	 *  This is functionally similar to getting an iterator with
+	 *  allterms_begin() and then calling skip_to(prefix) on that iterator
+	 *  to move to the start of the prefix, but is more convenient (because
+	 *  it detects the end of the prefixed terms), and may be more
+	 *  efficient than simply calling skip_to() after opening the iterator,
+	 *  particularly for network databases.
+	 *
+	 *  @param prefix The prefix to restrict the returned terms to.
+	 */
+	TermIterator allterms_begin(const std::string & prefix) const;
+
+	/** Corresponding end iterator to allterms_begin(prefix).
+	 */
+	TermIterator allterms_end(const std::string &) const {
+	    return TermIterator(NULL);
+	}
+
 	/// Get the number of documents in the database.
 	Xapian::doccount get_doccount() const;
 
@@ -205,11 +235,59 @@ class Database {
 	 *		could not be found in the database.
 	 */
 	Xapian::Document get_document(Xapian::docid did) const;
+
+	/** Suggest a spelling correction.
+	 *
+	 *  @param word			The potentially misspelled word.
+	 *  @param max_edit_distance	Only consider words which are at most
+	 *	@a max_edit_distance edits from @a word.  An edit is a
+	 *	character insertion, deletion, or the transposition of two
+	 *	adjacent characters (default is 2).
+	 */
+	std::string get_spelling_suggestion(const std::string &word,
+					    unsigned max_edit_distance = 2) const;
+
+	/** An iterator which returns all the spelling correction targets.
+	 *
+	 *  This returns all the words which are considered as targets for the
+	 *  spelling correction algorithm.  The frequency of each word is
+	 *  available as the term frequency of each entry in the returned
+	 *  iterator.
+	 */
+	Xapian::TermIterator spellings_begin() const;
+
+	/// Corresponding end iterator to spellings_begin().
+	Xapian::TermIterator spellings_end() const {
+	    return Xapian::TermIterator(NULL);
+	}
+
+	/** An iterator which returns all the synonyms for a given term.
+	 *
+	 *  @param term	    The term to return synonyms for.
+	 */
+	Xapian::TermIterator synonyms_begin(const std::string &term) const;
+
+	/// Corresponding end iterator to synonyms_begin(term).
+	Xapian::TermIterator synonyms_end(const std::string &) const {
+	    return Xapian::TermIterator(NULL);
+	}
+
+	/** An iterator which returns all terms which have synonyms.
+	 *
+	 *  @param prefix   If non-empty, only terms with this prefix are
+	 *		    returned.
+	 */
+	Xapian::TermIterator synonym_keys_begin(const std::string &prefix = "") const;
+
+	/// Corresponding end iterator to synonym_keys_begin(prefix).
+	Xapian::TermIterator synonym_keys_end(const std::string & = "") const {
+	    return Xapian::TermIterator(NULL);
+	}
 };
 
 /** This class provides read/write access to a database.
  */
-class WritableDatabase : public Database {
+class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
     public:
 	/** Destroy this handle on the database.
 	 *
@@ -241,7 +319,7 @@ class WritableDatabase : public Database {
 	 */
 	WritableDatabase(const std::string &path, int action);
 
-	/** @internal Create an WritableDatabase given its internals.
+	/** @private @internal Create an WritableDatabase given its internals.
 	 */
 	explicit WritableDatabase(Database::Internal *internal);
 
@@ -392,8 +470,11 @@ class WritableDatabase : public Database {
 	 *  document IDs come from a per-database monotonically increasing
 	 *  counter, so IDs from deleted documents won't be reused.
 	 *
-	 *  Note that this does not mean the document will immediately
-	 *  appear in the database; see flush() for more details.
+	 *  If you want to specify the document ID to be used, you should
+	 *  call replace_document() instead.
+	 *
+	 *  Note that changes to the database won't be immediately committed to
+	 *  disk; see flush() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document will either be fully added, or the document
@@ -417,8 +498,8 @@ class WritableDatabase : public Database {
 	 *  This method removes the document with the specified document ID
 	 *  from the database.
 	 *
-	 *  Note that this does not mean the document will immediately
-	 *  disappear from the database; see flush() for more details.
+	 *  Note that changes to the database won't be immediately committed to
+	 *  disk; see flush() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document will either be fully removed, or the document
@@ -457,9 +538,19 @@ class WritableDatabase : public Database {
 	/** Replace a given document in the database.
 	 *
 	 *  This method replaces the document with the specified document ID.
+	 *  If document ID @a did isn't currently used, the document will be
+	 *  added with document ID @a did.
 	 *
-	 *  Note that this does not mean the document will immediately
-	 *  change in the database; see flush() for more details.
+	 *  The monotonic counter used for automatically allocating document
+	 *  IDs is increased so that the next automatically allocated document
+	 *  ID will be did + 1.  Be aware that if you use this method to
+	 *  specify a high document ID for a new document, and also use
+	 *  WritableDatabase::add_document(), Xapian may get to a state where
+	 *  this counter wraps around and will be unable to automatically
+	 *  allocate document IDs!
+	 *
+	 *  Note that changes to the database won't be immediately committed to
+	 *  disk; see flush() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document will either be fully replaced, or the document
@@ -489,8 +580,8 @@ class WritableDatabase : public Database {
 	 *  be mapped to terms in Xapian, although this method probably has
 	 *  other uses.
 	 *
-	 *  Note that this does not mean the document(s) will immediately
-	 *  change in the database; see flush() for more details.
+	 *  Note that changes to the database won't be immediately committed to
+	 *  disk; see flush() for more details.
 	 *
 	 *  As with all database modification operations, the effect is
 	 *  atomic: the document(s) will either be fully replaced, or the
@@ -512,6 +603,48 @@ class WritableDatabase : public Database {
 	Xapian::docid replace_document(const std::string & unique_term,
 				       const Xapian::Document & document);
 
+	/** Add a word to the spelling dictionary.
+	 *
+	 *  If the word is already present, its frequency is increased.
+	 *
+	 *  @param word	    The word to add.
+	 *  @param freqinc  How much to increase its frequency by (default 1).
+	 */
+	void add_spelling(const std::string & word,
+			  Xapian::termcount freqinc = 1) const;
+
+	/** Remove a word from the spelling dictionary.
+	 *
+	 *  The word's frequency is decreased, and if would become zero or less
+	 *  then the word is removed completely.
+	 *
+	 *  @param word	    The word to remove.
+	 *  @param freqdec  How much to decrease its frequency by (default 1).
+	 */
+	void remove_spelling(const std::string & word,
+			     Xapian::termcount freqdec = 1) const;
+
+	/** Add a synonym for a term.
+	 *
+	 *  If @a synonym is already a synonym for @a term, then no action is
+	 *  taken.
+	 */
+	void add_synonym(const std::string & term,
+			 const std::string & synonym) const;
+
+	/** Remove a synonym for a term.
+	 *
+	 *  If @a synonym isn't a synonym for @a term, then no action is taken.
+	 */
+	void remove_synonym(const std::string & term,
+			    const std::string & synonym) const;
+
+	/** Remove all synonyms for a term.
+	 *
+	 *  If @a term has no synonyms, no action is taken.
+	 */
+	void clear_synonyms(const std::string & term) const;
+
 	/** Introspection method.
 	 *
 	 *  @return A string describing this object.
@@ -519,9 +652,13 @@ class WritableDatabase : public Database {
 	std::string get_description() const;
 };
 
+/** Open for read/write; create if no db exists. */
 const int DB_CREATE_OR_OPEN = 1;
+/** Create a new database; fail if db exists. */
 const int DB_CREATE = 2;
+/** Overwrite existing db; create if none exists. */
 const int DB_CREATE_OR_OVERWRITE = 3;
+/** Open for read/write; fail if no db exists. */
 const int DB_OPEN = 4;
 // Can't see any sensible use for this one
 // const int DB_OVERWRITE = XXX;
