@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,19 +29,20 @@
 #include <xapian/base.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
+#include <xapian/visibility.h>
 
 namespace Xapian {
 
 class ValueIterator;
 
 /// A document in the database - holds data, values, terms, and postings
-class Document {
+class XAPIAN_VISIBILITY_DEFAULT Document {
     public:
 	class Internal;
-	/// @internal Reference counted internals.
+	/// @private @internal Reference counted internals.
 	Xapian::Internal::RefCntPtr<Internal> internal;
 
-	/** @internal Constructor is only used by internal classes.
+	/** @private @internal Constructor is only used by internal classes.
 	 *
 	 *  @param internal_ pointer to internal opaque class
 	 */
@@ -63,8 +64,14 @@ class Document {
 	/// Destructor
 	~Document();
 
-	/// Get value by number (>= 0)
-	std::string get_value(Xapian::valueno value) const;
+	/** Get value by number.
+	 *
+	 *  Returns an empty string if no value with the given number is present
+	 *  in the document.
+	 *
+	 *  @param valueno The number of the value.
+	 */
+	std::string get_value(Xapian::valueno valueno) const;
 
 	/** Add a new value.  It will replace any existing value with the
 	 *  same number.
@@ -116,13 +123,6 @@ class Document {
 	 */
 	void add_term(const std::string & tname, Xapian::termcount wdfinc = 1);
 
-	/** Old name for add_term().
-	 *
-	 *  @deprecated	This method is deprecated and present only for backward
-	 *  compatibility.  Use add_term() instead.
-	 */
-	XAPIAN_DEPRECATED(void add_term_nopos(const std::string & term, Xapian::termcount wdfinc = 1));
-
 	/** Remove a posting of a term from the document.
 	 *
 	 *  Note that the term will still index the document even if all
@@ -178,6 +178,17 @@ class Document {
 	/// Equivalent end iterator for values_begin().
 	ValueIterator values_end() const;
 
+	/** Get the document id which is associated with this document (if any).
+	 *
+	 *  NB If multiple databases are being searched together, then this
+	 *  will be the document id in the individual database, not the merged
+	 *  database!
+	 *
+	 *  @return If this document came from a database, return the document
+	 *	    id in that database.  Otherwise, return 0.
+	 */
+	docid get_docid() const;
+
 	/** Introspection method.
 	 *
 	 *  @return  A string representing this Document.
