@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,18 +25,19 @@
 #define XAPIAN_INCLUDED_ENQUIRE_H
 
 #include <string>
-#include <time.h> // for time_t
 
 #include <xapian/base.h>
-#include <xapian/error.h>
+#include <xapian/deprecated.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
+#include <xapian/visibility.h>
 
 namespace Xapian {
 
 class Database;
 class Document;
 class ErrorHandler;
+class ExpandDecider;
 class MSetIterator;
 class Query;
 class Weight;
@@ -44,7 +45,7 @@ class Weight;
 /** A match set (MSet).
  *  This class represents (a portion of) the results of a query.
  */
-class MSet {
+class XAPIAN_VISIBILITY_DEFAULT MSet {
     public:
 	class Internal;
 	/// @internal Reference counted internals.
@@ -65,7 +66,7 @@ class MSet {
 	/// Assignment is allowed (and is cheap).
 	void operator=(const MSet &other);
 
-	/** Fetch the the document info for a set of items in the MSet.
+	/** Fetch the document info for a set of items in the MSet.
 	 *
 	 *  This method causes the documents in the range specified by the
 	 *  iterators to be fetched from the database, and cached in the
@@ -74,7 +75,7 @@ class MSet {
 	 *  subsequent access to the document contents when the documents are
 	 *  stored in a remote database.
 	 *
-	 *  The iterators must be over this Xapian::MSet: undefined behaviour
+	 *  The iterators must be over this Xapian::MSet - undefined behaviour
 	 *  will result otherwise.
 	 *
 	 *  @param begin   MSetIterator for first item to fetch.
@@ -122,7 +123,7 @@ class MSet {
 	 *
 	 *  This corresponds to the parameter "first" specified in
 	 *  Xapian::Enquire::get_mset().  A value of 0 corresponds to the
-	 *  highest result being the first item in the mset.
+	 *  highest result being the first item in the MSet.
 	 */
 	Xapian::doccount get_firstitem() const;
 
@@ -162,7 +163,7 @@ class MSet {
 	 */
 	Xapian::doccount get_matches_upper_bound() const;
 
-	/** The maximum possible weight in the mset.
+	/** The maximum possible weight in the MSet.
 	 *  This weight is likely not to be attained in the set of results,
 	 *  but represents an upper bound on the weight which a document
 	 *  could attain for the given query.
@@ -228,7 +229,7 @@ class MSet {
 	typedef Xapian::doccount size_type;
 	//@}
 
-	/** Returns a string representing the mset.
+	/** Returns a string representing the MSet.
 	 *  Introspection method.
 	 */
 	std::string get_description() const;
@@ -237,7 +238,7 @@ class MSet {
 /** An iterator pointing to items in an MSet.
  *  This is used for access to individual results of a match.
  */
-class MSetIterator {
+class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
     private:
 	friend class MSet;
 	friend bool operator==(const MSetIterator &a, const MSetIterator &b);
@@ -389,7 +390,7 @@ class ESetIterator;
  *  This set represents the results of an expand operation, which is
  *  performed by Xapian::Enquire::get_eset().
  */
-class ESet {
+class XAPIAN_VISIBILITY_DEFAULT ESet {
     public:
 	class Internal;
 	/// @internal Reference counted internals.
@@ -445,7 +446,7 @@ class ESet {
 };
 
 /** Iterate through terms in the ESet */
-class ESetIterator {
+class XAPIAN_VISIBILITY_DEFAULT ESetIterator {
     private:
 	friend class ESet;
 	friend bool operator==(const ESetIterator &a, const ESetIterator &b);
@@ -538,7 +539,7 @@ inline bool operator!=(const ESetIterator &a, const ESetIterator &b)
  *  This is the set of documents which are marked as relevant, for use
  *  in modifying the term weights, and in performing query expansion.
  */
-class RSet {
+class XAPIAN_VISIBILITY_DEFAULT RSet {
     public:
 	/// Class holding details of RSet
 	class Internal;
@@ -580,7 +581,7 @@ class RSet {
 	bool contains(Xapian::docid did) const;
 
 	/// Test if a given document in the relevance set.
-	bool contains(const Xapian::MSetIterator & i) { return contains(*i); }
+	bool contains(const Xapian::MSetIterator & i) const { return contains(*i); }
 
 	/** Introspection method.
 	 *
@@ -591,26 +592,14 @@ class RSet {
 
 /** Base class for matcher decision functor.
  */
-class MatchDecider {
+class XAPIAN_VISIBILITY_DEFAULT MatchDecider {
     public:
-	/** Decide whether we want this document to be in the mset.
+	/** Decide whether we want this document to be in the MSet.
 	 */
-	virtual int operator()(const Xapian::Document &doc) const = 0;
+	virtual bool operator()(const Xapian::Document &doc) const = 0;
 
 	/// Destructor.
-	virtual ~MatchDecider() {}
-};
-
-/** Base class for expand decision functor.
- */
-class ExpandDecider {
-    public:
-	/** Decide whether we want this term to be in the expand set.
-	 */
-	virtual int operator()(const std::string & tname) const = 0;
-
-	/// Destructor.
-	virtual ~ExpandDecider() {}
+	virtual ~MatchDecider();
 };
 
 /** This class provides an interface to the information retrieval
@@ -623,7 +612,7 @@ class ExpandDecider {
  *  @exception Xapian::InvalidArgumentError will be thrown if an invalid
  *  argument is supplied, for example, an unknown database type.
  */
-class Enquire {
+class XAPIAN_VISIBILITY_DEFAULT Enquire {
     private:
 	/// Copies are not allowed.
 	Enquire(const Enquire &);
@@ -642,6 +631,12 @@ class Enquire {
 	 *  opened: you must create a new Xapian::Enquire object to access a
 	 *  different database, or set of databases.
 	 *
+	 *  The database supplied must have been initialised (ie, must not be
+	 *  the result of calling the Database::Database() constructor).  If
+	 *  you need to handle a situation where you have no index gracefully,
+	 *  a database created with InMemory::open() can be passed here,
+	 *  which represents a completely empty database.
+	 *
 	 *  @param database Specification of the database or databases to
 	 *	   use.
 	 *  @param errorhandler_  A pointer to the error handler to use.
@@ -650,8 +645,11 @@ class Enquire {
 	 *	   Xapian::ErrorHandler object after the Xapian::Enquire object
 	 *	   is deleted.  To use no error handler, this parameter
 	 *	   should be 0.
+	 *
+	 *  @exception Xapian::InvalidArgumentError will be thrown if an
+	 *  initialised Database object is supplied.
 	 */
-	Enquire(const Database &databases, ErrorHandler * errorhandler_ = 0);
+	explicit Enquire(const Database &database, ErrorHandler * errorhandler_ = 0);
 
 	/** Close the Xapian::Enquire object.
 	 */
@@ -671,7 +669,7 @@ class Enquire {
 	 *  @exception Xapian::InvalidArgumentError will be thrown if query has
 	 *	       not yet been set.
 	 */
-	const Xapian::Query & get_query();
+	const Xapian::Query & get_query() const;
 
 	/** Set the weighting scheme to use for queries.
 	 *
@@ -683,7 +681,7 @@ class Enquire {
 
 	/** Set the collapse key to use for queries.
 	 *
-	 *  @param collapse_key  value number to collapse on - at most one mset
+	 *  @param collapse_key  value number to collapse on - at most one MSet
 	 *	entry with each particular value will be returned.
 	 *
 	 *	The entry returned will be the best entry with that particular
@@ -705,7 +703,7 @@ class Enquire {
 	 *	you can offer a link to a non-collapsed search restricted to
 	 *	that thread using a boolean filter.
 	 *
-	 *	(default is Xapian::valueno(-1) which means no collapsing).
+	 *	(default is Xapian::BAD_VALUENO which means no collapsing).
 	 */
 	void set_collapse_key(Xapian::valueno collapse_key);
 
@@ -740,26 +738,18 @@ class Enquire {
 	 */
 	void set_docid_order(docid_order order);
 
-	/** For compatibility with Xapian 0.8.5 and earlier.
-	 *
-	 *  @deprecated	This method is now deprecated, use set_docid_order()
-	 *  instead - set_sort_forward(true) -> set_docid_order(ASCENDING)
-	 *  and set_sort_forward(false) -> set_docid_order(DESCENDING).
-	 */
-	XAPIAN_DEPRECATED(void set_sort_forward(bool sort_forward));
-
 	/** Set the percentage and/or weight cutoffs.
 	 *
 	 * @param percent_cutoff Minimum percentage score for returned
 	 *	documents. If a document has a lower percentage score than this,
-	 *	it will not appear in the mset.  If your intention is to return
+	 *	it will not appear in the MSet.  If your intention is to return
 	 *	only matches which contain all the terms in the query, then
 	 *	it's more efficient to use Xapian::Query::OP_AND instead of
 	 *	Xapian::Query::OP_OR in the query than to use set_cutoff(100).
 	 *	(default 0 => no percentage cut-off).
 	 * @param weight_cutoff Minimum weight for a document to be returned.
 	 *	If a document has a lower score that this, it will not appear
-	 *	in the mset.  It is usually only possible to choose an
+	 *	in the MSet.  It is usually only possible to choose an
 	 *	appropriate weight for cutoff based on the results of a
 	 *	previous run of the same query; this is thus mainly useful for
 	 *	alerting operations.  The other potential use is with a user
@@ -768,25 +758,6 @@ class Enquire {
 	 */
 	void set_cutoff(Xapian::percent percent_cutoff, Xapian::weight weight_cutoff = 0);
 
-	/** For compatibility with Xapian 0.8.5 and earlier.
-	 *
-	 *  @deprecated This method is now deprecated, use
-	 *  set_sort_by_relevance(), set_sort_by_value(), or
-	 *  set_sort_by_value_then_relevance() instead.
-	 *
-	 *  set_sorting(KEY, 1) -> set_sort_by_value(KEY)
-	 *
-	 *  set_sorting(KEY, 1, false) -> set_sort_by_value(KEY)
-	 *
-	 *  set_sorting(KEY, 1, true) -> set_sort_by_value_then_relevance(KEY)
-	 *
-	 *  set_sorting(ANYTHING, 0) -> set_sort_by_relevance()
-	 *
-	 *  set_sorting(Xapian::valueno(-1), ANYTHING) -> set_sort_by_relevance()
-	 */
-	XAPIAN_DEPRECATED(void set_sorting(Xapian::valueno sort_key, int sort_bands,
-			  bool sort_by_relevance = false));
-
 	/** Set the sorting to be by relevance only.
 	 *
 	 *  This is the default.
@@ -795,9 +766,11 @@ class Enquire {
 
 	/** Set the sorting to be by value only.
 	 *
-	 * @param sort_key value number to reorder on.  Sorting is with a
-	 *	string compare.  If ascending is true (the default) higher
-	 *	is better; if ascending is false, lower is better.
+	 *  NB sorting of values uses a string comparison, so you'll need to
+	 *  store numbers padded with leading zeros or spaces, or with the
+	 *  number of digits prepended.
+	 *
+	 * @param sort_key  value number to sort on.
 	 *
 	 * @param ascending  If true, documents values which sort higher by
 	 *		 string compare are better.  If false, the sort order
@@ -808,9 +781,11 @@ class Enquire {
 	/** Set the sorting to be by value, then by relevance for documents
 	 *  with the same value.
 	 *
-	 * @param sort_key value number to reorder on.  Sorting is with a
-	 *	string compare.  If ascending is true (the default) higher
-	 *	is better; if ascending is false, lower is better.
+	 *  NB sorting of values uses a string comparison, so you'll need to
+	 *  store numbers padded with leading zeros or spaces, or with the
+	 *  number of digits prepended.
+	 *
+	 * @param sort_key  value number to sort on.
 	 *
 	 * @param ascending  If true, documents values which sort higher by
 	 *		 string compare are better.  If false, the sort order
@@ -820,6 +795,10 @@ class Enquire {
 					      bool ascending = true);
 
 	/** Set the sorting to be by relevance then value.
+	 *
+	 *  NB sorting of values uses a string comparison, so you'll need to
+	 *  store numbers padded with leading zeros or spaces, or with the
+	 *  number of digits prepended.
 	 *
 	 *  Note that with the default BM25 weighting scheme parameters,
 	 *  non-identical documents will rarely have the same weight, so
@@ -828,9 +807,7 @@ class Enquire {
 	 *  BM25 parameter settings (e.g. BM25Weight(1,0,1,0,0)) or custom
 	 *  weighting schemes.
 	 *
-	 * @param sort_key value number to reorder on.  Sorting is with a
-	 *	string compare.  If ascending is true (the default) higher
-	 *	is better; if ascending is false, lower is better.
+	 * @param sort_key  value number to sort on.
 	 *
 	 * @param ascending  If true, documents values which sort higher by
 	 *		 string compare are better.  If false, the sort order
@@ -839,19 +816,6 @@ class Enquire {
 	void set_sort_by_relevance_then_value(Xapian::valueno sort_key,
 					      bool ascending = true);
 
-	/** Set the bias functor parameters.
-	 *
-	 * NB this is a temporary API for this feature.
-	 *
-	 * @param bias_weight Maximum weight bias functor can add (and which is
-	 *	given to document with a time now or in the future).
-	 *
-	 * @param bias_halflife the match bias decays exponentially as you go
-	 *	back in time.  This sets the half-life of this decay in seconds
-	 *	(default 0 => no bias).
-	 */
-	void set_bias(Xapian::weight bias_weight, time_t bias_halflife);
-
 	/** Get (a portion of) the match set for the current query.
 	 *
 	 *  @param first     the first item in the result set to return.
@@ -865,12 +829,19 @@ class Enquire {
 	 *		     the matcher optimises, it won't consider every
 	 *		     document which might match, so the total number
 	 *		     of matches is estimated.  Setting checkatleast
-	 *		     forces it to consider that many matches and so
-	 *		     allows for reliable paging links.
+	 *		     forces it to consider at least this many matches
+	 *		     and so allows for reliable paging links.
 	 *  @param omrset    the relevance set to use when performing the query.
 	 *  @param mdecider  a decision functor to use to decide whether a
-	 *		     given document should be put in the MSet
-	 *
+	 *		     given document should be put in the MSet.
+	 *  @param matchspy  a decision functor to use to decide whether a
+	 *		     given document should be put in the MSet.  The
+	 *		     matchspy is applied to every document which is
+	 *		     a potential candidate for the MSet, so if there are
+	 *		     checkatleast or more such documents, the matchspy
+	 *		     will see at least checkatleast.  The mdecider is
+	 *		     assumed to be a relatively expensive test so may
+	 *		     be applied in a lazier fashion.
 	 *
 	 *  @return	     A Xapian::MSet object containing the results of the
 	 *		     query.
@@ -881,23 +852,41 @@ class Enquire {
 		      Xapian::doccount checkatleast = 0,
 		      const RSet * omrset = 0,
 		      const MatchDecider * mdecider = 0) const;
+	MSet get_mset(Xapian::doccount first, Xapian::doccount maxitems,
+		      Xapian::doccount checkatleast,
+		      const RSet * omrset,
+		      const MatchDecider * mdecider,
+		      const MatchDecider * matchspy) const;
 	MSet get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		      const RSet * omrset,
 		      const MatchDecider * mdecider = 0) const {
 	    return get_mset(first, maxitems, 0, omrset, mdecider);
 	}
 
+	static const int INCLUDE_QUERY_TERMS = 1;
+	static const int USE_EXACT_TERMFREQ = 2;
+#ifndef _MSC_VER
+	/// Deprecated in Xapian 1.0.0, use INCLUDE_QUERY_TERMS instead.
+	XAPIAN_DEPRECATED(static const int include_query_terms) = 1;
+	/// Deprecated in Xapian 1.0.0, use USE_EXACT_TERMFREQ instead.
+	XAPIAN_DEPRECATED(static const int use_exact_termfreq) = 2;
+#else
+	// Work around MSVC stupidity (you get a warning for deprecating a
+	// declaration).
 	static const int include_query_terms = 1;
 	static const int use_exact_termfreq = 2;
+#pragma deprecated("Xapian::Enquire::include_query_terms", "Xapian::Enquire::use_exact_termfreq")
+#endif
+
 	/** Get the expand set for the given rset.
 	 *
 	 *  @param maxitems  the maximum number of items to return.
 	 *  @param omrset    the relevance set to use when performing
 	 *		     the expand operation.
 	 *  @param flags     zero or more of these values |-ed together:
-	 *		      - Xapian::Enquire::include_query_terms query
+	 *		      - Xapian::Enquire::INCLUDE_QUERY_TERMS query
 	 *			terms may be returned from expand
-	 *		      - Xapian::Enquire::use_exact_termfreq for multi
+	 *		      - Xapian::Enquire::USE_EXACT_TERMFREQ for multi
 	 *			dbs, calculate the exact termfreq; otherwise an
 	 *			approximation is used which can greatly improve
 	 *			efficiency, but still returns good results.
@@ -941,7 +930,7 @@ class Enquire {
 	 *  the given document.
 	 *
 	 *  It is possible for the document to have been removed from the
-	 *  database between the time it is returned in an mset, and the
+	 *  database between the time it is returned in an MSet, and the
 	 *  time that this call is made.  If possible, you should specify
 	 *  an MSetIterator instead of a Xapian::docid, since this will enable
 	 *  database backends with suitable support to prevent this
@@ -1021,7 +1010,7 @@ class RemoteServer;
 namespace Xapian {
 
 /// Abstract base class for weighting schemes
-class Weight {
+class XAPIAN_VISIBILITY_DEFAULT Weight {
     friend class Enquire; // So Enquire can clone us
     friend class ::RemoteServer; // So RemoteServer can clone us - FIXME
     public:
@@ -1050,7 +1039,7 @@ class Weight {
 
     public:
 	Weight() { }
-	virtual ~Weight() { }
+	virtual ~Weight();
 
 	/** Create a new weight object of the same type as this and initialise
 	 *  it with the specified statistics.
@@ -1065,14 +1054,7 @@ class Weight {
 	 *  @param tname_     Term which this object is associated with.
 	 */
 	Weight * create(const Internal * internal_, Xapian::doclength querysize_,
-			  Xapian::termcount wqf_, std::string tname_) const {
-	    Weight * wt = clone();
-	    wt->internal = internal_;
-	    wt->querysize = querysize_;
-	    wt->wqf = wqf_;
-	    wt->tname = tname_;
-	    return wt;
-	}
+			Xapian::termcount wqf_, const std::string & tname_) const;
 
 	/** Name of the weighting scheme.
 	 *
@@ -1119,29 +1101,25 @@ class Weight {
 	virtual Xapian::weight get_maxextra() const = 0;
 
 	/// return false if the weight object doesn't need doclength
-	virtual bool get_sumpart_needs_doclength() const { return true; }
+	virtual bool get_sumpart_needs_doclength() const; /* { return true; } */
 };
 
 /// Boolean weighting scheme (everything gets 0)
-class BoolWeight : public Weight {
+class XAPIAN_VISIBILITY_DEFAULT BoolWeight : public Weight {
     public:
-	BoolWeight * clone() const {
-	    return new BoolWeight;
-	}
+	BoolWeight * clone() const;
 	BoolWeight() { }
-	~BoolWeight() { }
-	std::string name() const { return "Bool"; }
-	std::string serialise() const { return ""; }
-	BoolWeight * unserialise(const std::string & /*s*/) const {
-	    return new BoolWeight;
-	}
-	Xapian::weight get_sumpart(Xapian::termcount /*wdf*/, Xapian::doclength /*len*/) const { return 0; }
-	Xapian::weight get_maxpart() const { return 0; }
+	~BoolWeight();
+	std::string name() const;
+	std::string serialise() const;
+	BoolWeight * unserialise(const std::string & s) const;
+	Xapian::weight get_sumpart(Xapian::termcount wdf, Xapian::doclength len) const;
+	Xapian::weight get_maxpart() const;
 
-	Xapian::weight get_sumextra(Xapian::doclength /*len*/) const { return 0; }
-	Xapian::weight get_maxextra() const { return 0; }
+	Xapian::weight get_sumextra(Xapian::doclength len) const;
+	Xapian::weight get_maxextra() const;
 
-	bool get_sumpart_needs_doclength() const { return false; }
+	bool get_sumpart_needs_doclength() const;
 };
 
 /** BM25 weighting scheme
@@ -1156,7 +1134,7 @@ class BoolWeight : public Weight {
  *   - \f$n_{q}\f$ is the size of the query
  *   - \f$k_{1}\f$, \f$k_{2}\f$, \f$k_{3}\f$ and \f$b\f$ are user specified parameters
  */
-class BM25Weight : public Weight {
+class XAPIAN_VISIBILITY_DEFAULT BM25Weight : public Weight {
     private:
 	mutable Xapian::weight termweight;
 	mutable Xapian::doclength lenpart;
@@ -1214,7 +1192,11 @@ class BM25Weight : public Weight {
 	bool get_sumpart_needs_doclength() const;
 };
 
-/** Traditional probabilistic weighting scheme (as used by Muscat 3.6)
+/** Traditional probabilistic weighting scheme.
+ *
+ * This class implements the Traditional Probabilistic Weighting scheme, as
+ * described by the early papers on Probabilistic Retrieval.  BM25 generally
+ * gives better results.
  *
  * The Traditional weighting scheme formula is \f[
  *	\sum_{t}\frac{f_{t,d}}{k.L_{d}+f_{t,d}}.w_{t}
@@ -1227,7 +1209,7 @@ class BM25Weight : public Weight {
  * TradWeight(k) is equivalent to BM25Weight(k, 0, 0, 1, 0), except that
  * the latter returns weights (k+1) times larger.
  */
-class TradWeight : public Weight {
+class XAPIAN_VISIBILITY_DEFAULT TradWeight : public Weight {
     private:
 	mutable Xapian::weight termweight;
 	mutable Xapian::doclength lenpart;
