@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,25 +30,26 @@
 #include <xapian/base.h>
 #include <xapian/types.h>
 #include <xapian/positioniterator.h>
+#include <xapian/visibility.h>
 
 namespace Xapian {
 
 class Database;
 
-/** A wrapper class for a termname which returns the termname if dereferenced
- *  with *.  We need this to implement input_iterator semantics.
+/** @internal A wrapper class for a termname which returns the termname if
+ *  dereferenced with *.  We need this to implement input_iterator semantics.
  */
 class TermNameWrapper {
     private:
 	std::string tname;
     public:
-	TermNameWrapper(const std::string & tname_) : tname(tname_) { }
+	explicit TermNameWrapper(const std::string & tname_) : tname(tname_) { }
 	const std::string & operator*() const { return tname; }
 };
 
 /** An iterator pointing to items in a list of terms.
  */
-class TermIterator {
+class XAPIAN_VISIBILITY_DEFAULT TermIterator {
     public:
 	class Internal;
 	/// @internal Reference counted internals.
