@@ -6,7 +6,7 @@ const char * dummy = {
 "// If using GCC, also check the C++ ABI version is compatible with that used",
 "// to build the library.",
 "//",
-"// Copyright (C) 2002,2004,2005,2006 Olly Betts",
+"// Copyright (C) 2002,2004,2005,2006,2007 Olly Betts",
 "//",
 "// This program is free software; you can redistribute it and/or",
 "// modify it under the terms of the GNU General Public License as",
@@ -96,68 +96,11 @@ const char * dummy = {
 "#endif",
 #endif
 "",
-"#include <xapian/deprecated.h>",
-"",
 "#define XAPIAN_VERSION ",STRING_VERSION,
 "#define XAPIAN_MAJOR_VERSION ",MAJOR_VERSION,
 "#define XAPIAN_MINOR_VERSION ",MINOR_VERSION,
 "#define XAPIAN_REVISION ",REVISION,
 "",
-"namespace Xapian {",
-"/** Report the version string of the library which the program is linked with.",
-"  *",
-"  * This may be different to the version compiled against (given by",
-"  * XAPIAN_VERSION) if shared libraries are being used.",
-"  */",
-"const char * version_string();",
-"",
-"/** For compatibility with Xapian 0.9.5 and earlier.",
-" *",
-" *  @deprecated This function is now deprecated, use Xapian::version_string()",
-" *  instead.",
-" */",
-"XAPIAN_DEPRECATED(const char * xapian_version_string());",
-"",
-"/** Report the major version of the library which the program is linked to.",
-"  * This may be different to the version compiled against (given by",
-"  * XAPIAN_MAJOR_VERSION) if shared libraries are being used.",
-"  */",
-"int major_version();",
-"",
-"/** For compatibility with Xapian 0.9.5 and earlier.",
-" *",
-" *  @deprecated This function is now deprecated, use Xapian::major_version()",
-" *  instead.",
-" */",
-"XAPIAN_DEPRECATED(int xapian_major_version());",
-"",
-"/** Report the minor version of the library which the program is linked to.",
-"  * This may be different to the version compiled against (given by",
-"  * XAPIAN_MINOR_VERSION) if shared libraries are being used.",
-"  */",
-"int minor_version();",
-"",
-"/** For compatibility with Xapian 0.9.5 and earlier.",
-" *",
-" *  @deprecated This function is now deprecated, use Xapian::minor_version()",
-" *  instead.",
-" */",
-"XAPIAN_DEPRECATED(int xapian_minor_version());",
-"",
-"/** Report the revision of the library which the program is linked to.",
-"  * This may be different to the version compiled against (given by",
-"  * XAPIAN_REVISION) if shared libraries are being used.",
-"  */",
-"int revision();",
-"",
-"/** For compatibility with Xapian 0.9.5 and earlier.",
-" *",
-" *  @deprecated This function is now deprecated, use Xapian::revision()",
-" *  instead.",
-" */",
-"XAPIAN_DEPRECATED(int xapian_revision());",
-"}",
-"",
 #ifdef XAPIAN_HAS_FLINT_BACKEND
 "#define XAPIAN_HAS_FLINT_BACKEND 1",
 #else
@@ -173,16 +116,11 @@ const char * dummy = {
 #else
 "/* #undef XAPIAN_HAS_INMEMORY_BACKEND */",
 #endif
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-"#define XAPIAN_HAS_MUSCAT36_BACKEND 1",
-#else
-"/* #undef XAPIAN_HAS_MUSCAT36_BACKEND */",
-#endif
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 "#define XAPIAN_HAS_REMOTE_BACKEND 1",
 #else
 "/* #undef XAPIAN_HAS_REMOTE_BACKEND */",
 #endif
 "",
-"#endif"
+"#endif /* XAPIAN_INCLUDED_VERSION_H */"
 };
