@@ -26,6 +26,7 @@
 
 #include <xapian/enquire.h>
 
+#include "omassert.h"
 #include "omdebug.h"
 #include "serialise-double.h"
 #include "stats.h"
@@ -85,7 +86,7 @@ BM25Weight::calc_termweight() const
 	// termfreq must be at least rtermfreq since there are at least
 	// rtermfreq documents indexed by this term.  And it can't be
 	// more than (dbsize - rsize + rtermfreq) since the number
-	// of releveant documents not indexed by this term can't be
+	// of relevant documents not indexed by this term can't be
 	// more than the number of documents not indexed by this term.
 	Assert(termfreq >= rtermfreq);
 	Assert(termfreq <= dbsize - rsize + rtermfreq);
