@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,11 +31,13 @@
 #include "andnotpostlist.h"
 #include "andmaybepostlist.h"
 #include "filterpostlist.h"
+#include "exactphrasepostlist.h"
 #include "phrasepostlist.h"
 #include "emptypostlist.h"
 #include "leafpostlist.h"
 #include "mergepostlist.h"
 #include "extraweightpostlist.h"
+#include "valuerangepostlist.h"
 
 #include "omqueryinternal.h"
 
@@ -315,6 +317,9 @@ LocalSubMatch::postlist_from_queries(Xapian::Query::Internal::op_t op,
 	    std::vector<PostList *> postlists_orig = postlists;
 	    PostList *res = build_and_tree(postlists, matcher);
 	    // FIXME: handle EmptyPostList return specially?
+	    if (query->parameter == postlists_orig.size()) {
+		RETURN(new ExactPhrasePostList(res, postlists_orig));
+	    }
 	    RETURN(new PhrasePostList(res, query->parameter, postlists_orig));
 	}
 
@@ -382,7 +387,7 @@ LocalSubMatch::postlist_from_query(const Xapian::Query::Internal *query,
     switch (op) {
 	case Xapian::Query::Internal::OP_LEAF: {
 	    // Make a postlist for a single term
-	    Assert(query->subqs.size() == 0);
+	    Assert(query->subqs.empty());
 
 	    // FIXME: pass the weight type and the info needed to create it to
 	    // the postlist instead (why?)
@@ -468,6 +473,9 @@ LocalSubMatch::postlist_from_query(const Xapian::Query::Internal *query,
 					postlist_from_query(query->subqs[1], matcher, is_bool),
 					matcher,
 					db->get_doccount());
+	case Xapian::Query::OP_VALUE_RANGE:
+	    RETURN(new ValueRangePostList(db, Xapian::valueno(query->parameter),
+					  query->tname, query->str_parameter));
     }
     Assert(false);
     RETURN(NULL);
@@ -476,7 +484,11 @@ LocalSubMatch::postlist_from_query(const Xapian::Query::Internal *query,
 void
 LocalSubMatch::register_term(const string &tname)
 {
-    statssource.my_termfreq_is(tname, db->get_termfreq(tname));
+    if (tname.empty()) {
+	statssource.my_termfreq_is(tname, db->get_doccount());
+    } else {
+	statssource.my_termfreq_is(tname, db->get_termfreq(tname));
+    }
 }
 
 ////////////////////////
@@ -502,7 +514,7 @@ LocalSubMatch::prepare_match(bool /*nowait*/)
 }
 
 void
-LocalSubMatch::start_match(Xapian::doccount)
+LocalSubMatch::start_match(Xapian::doccount, Xapian::doccount)
 {
     // Nothing to do here for a local match.
 }
