@@ -3,6 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004,2006 Olly Betts
+ * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -89,6 +90,13 @@ MergePostList::skip_to(Xapian::docid did, Xapian::weight w_min)
     throw Xapian::InvalidOperationError("MergePostList doesn't support skip_to");
 }
 
+Xapian::termcount
+MergePostList::get_wdf() const
+{
+    DEBUGCALL(MATCH, Xapian::termcount, "MergePostList::get_wdf", "");
+    RETURN(plists[current]->get_wdf());
+}
+
 Xapian::doccount
 MergePostList::get_termfreq_max() const
 {
