@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  * Copyright 2003 Orange PCS Ltd
  * Copyright 2003 Sam Liddicott
  *
@@ -35,7 +35,6 @@
 #include "branchpostlist.h"
 #include "leafpostlist.h"
 #include "mergepostlist.h"
-#include "biaspostlist.h"
 
 #include "document.h"
 #include "omqueryinternal.h"
@@ -61,6 +60,17 @@
 
 using namespace std;
 
+const Xapian::Enquire::Internal::sort_setting REL =
+	Xapian::Enquire::Internal::REL;
+const Xapian::Enquire::Internal::sort_setting REL_VAL =
+	Xapian::Enquire::Internal::REL_VAL;
+const Xapian::Enquire::Internal::sort_setting VAL =
+	Xapian::Enquire::Internal::VAL;
+#if 0 // VAL_REL isn't currently used which causes a warning with SGI CC.
+const Xapian::Enquire::Internal::sort_setting VAL_REL =
+	Xapian::Enquire::Internal::VAL_REL;
+#endif
+
 ////////////////////////////////////
 // Initialisation and cleaning up //
 ////////////////////////////////////
@@ -74,8 +84,6 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 		       Xapian::valueno sort_key_,
 		       Xapian::Enquire::Internal::sort_setting sort_by_,
 		       bool sort_value_forward_,
-		       time_t bias_halflife_,
-		       Xapian::weight bias_weight_,
 		       Xapian::ErrorHandler * errorhandler_,
 		       StatsGatherer * gatherer_,
 		       const Xapian::Weight * weight_)
@@ -84,17 +92,15 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 	  weight_cutoff(weight_cutoff_), order(order_),
 	  sort_key(sort_key_), sort_by(sort_by_),
 	  sort_value_forward(sort_value_forward_),
-	  bias_halflife(bias_halflife_), bias_weight(bias_weight_),
-	  errorhandler(errorhandler_), weight(weight_)
+	  errorhandler(errorhandler_), weight(weight_),
+	  is_remote(db.internal.size())
 {
     DEBUGCALL(MATCH, void, "MultiMatch", db_ << ", " << query_ << ", " <<
-	      qlen << ", " <<
-	      omrset << ", " << collapse_key_ << ", " << percent_cutoff_ <<
-	      ", " << weight_cutoff_ << ", " << int(order_) << ", " <<
-	      sort_key_ << ", " << int(sort_by_) <<
-	      ", " << sort_value_forward_ << ", " << bias_halflife_ << ", " <<
-	      bias_weight_ << ", " << errorhandler_ <<
-	      ", [gatherer_], [weight_]");
+	      qlen << ", " << omrset << ", " << collapse_key_ << ", " <<
+	      percent_cutoff_ << ", " << weight_cutoff_ << ", " <<
+	      int(order_) << ", " << sort_key_ << ", " <<
+	      int(sort_by_) << ", " << sort_value_forward_ << ", " <<
+	      errorhandler_ << ", [gatherer_], [weight_]");
     if (!query) return;
 
     query->validate_query();
@@ -120,12 +126,9 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 	}
     }
 
-    vector<Xapian::RSet>::const_iterator subrset = subrsets.begin();
-
-    vector<Xapian::Internal::RefCntPtr<Xapian::Database::Internal> >::const_iterator i;
-    for (i = db.internal.begin(); i != db.internal.end(); ++i) {
-	Assert(subrset != subrsets.end());
-	Xapian::Database::Internal *subdb = (*i).get();
+    Assert(subrsets.size() == db.internal.size());
+    for (size_t i = 0; i != db.internal.size(); ++i) {
+	Xapian::Database::Internal *subdb = db.internal[i].get();
 	Assert(subdb);
 	Xapian::Internal::RefCntPtr<SubMatch> smatch;
 	try {
@@ -133,17 +136,17 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 	    RemoteDatabase *rem_db = subdb->as_remotedatabase();
 	    if (rem_db) {
-		if (bias_halflife) {
-		    throw Xapian::UnimplementedError("bias_halflife and bias_weight not supported with remote backend");
-		}
+		is_remote[i] = true;
 		rem_db->set_query(query, qlen, collapse_key, order, sort_key,
 				  sort_by, sort_value_forward, percent_cutoff,
-				  weight_cutoff, weight, *subrset);
+				  weight_cutoff, weight, subrsets[i]);
+		bool decreasing_relevance =
+		    (sort_by == REL || sort_by == REL_VAL);
 		smatch = new RemoteSubMatch(rem_db, gatherer.get(),
-					    sort_key != Xapian::valueno(-1));
+					    decreasing_relevance);
 	    } else {
 #endif /* XAPIAN_HAS_REMOTE_BACKEND */
-		smatch = new LocalSubMatch(subdb, query, qlen, *subrset, gatherer.get(), weight);
+		smatch = new LocalSubMatch(subdb, query, qlen, subrsets[i], gatherer.get(), weight);
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 	    }
 #endif /* XAPIAN_HAS_REMOTE_BACKEND */
@@ -155,9 +158,7 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 	    smatch = new EmptySubMatch;
 	}
 	leaves.push_back(smatch);
-	++subrset;
     }
-    Assert(subrset == subrsets.end());
 
     gatherer->set_global_stats(omrset.size());
 
@@ -230,7 +231,8 @@ MultiMatch::getorrecalc_maxweight(PostList *pl)
 void
 MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		     Xapian::doccount check_at_least,
-		     Xapian::MSet & mset, const Xapian::MatchDecider *mdecider)
+		     Xapian::MSet & mset, const Xapian::MatchDecider *mdecider,
+		     const Xapian::MatchDecider *matchspy)
 {
     DEBUGCALL(MATCH, void, "MultiMatch::get_mset", first << ", " << maxitems
 	      << ", " << check_at_least << ", ...");
@@ -250,7 +252,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    try {
 		// FIXME: look if we can push the "check_at_least" stuff
 		// into the remote match handling too.
-		(*leaf)->start_match(first + check_at_least);
+		(*leaf)->start_match(first + maxitems, first + check_at_least);
 	    } catch (Xapian::Error & e) {
 		if (errorhandler) {
 		    DEBUGLINE(EXCEPTION, "Calling error handler for "
@@ -291,23 +293,16 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    postlists.push_back(pl);
 	}
     }
+    Assert(!postlists.empty());
 
     // Get a single combined postlist
     PostList *pl;
-    Assert(postlists.size() != 0);
     if (postlists.size() == 1) {
 	pl = postlists.front();
     } else {
 	pl = new MergePostList(postlists, this, errorhandler);
     }
 
-    // FIXME: a temporary bodge to allow this to be used - I'll write
-    // a proper API later, promise - Olly
-    if (bias_halflife) {
-	pl = new BiasPostList(pl, db,
-		new OmBiasFunctor(db, bias_weight, bias_halflife), this);
-    }
-
     DEBUGLINE(MATCH, "pl = (" << pl->get_description() << ")");
 
     // Empty result set
@@ -331,11 +326,11 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     // maxweight).
     if (check_at_least == 0) {
 	delete pl;
-	if (mdecider != NULL) {
+	if (mdecider != NULL || matchspy != NULL) {
 	    // Lower bound must be set to 0 as the match decider could discard
 	    // all hits.
 	    matches_lower_bound = 0;
-	} else if (collapse_key != Xapian::valueno(-1)) {
+	} else if (collapse_key != Xapian::BAD_VALUENO) {
 	    // Lower bound must be set to no more than 1, since it's possible
 	    // that all hits will be collapsed to a single hit.
 	    if (matches_lower_bound > 1) matches_lower_bound = 1;
@@ -354,12 +349,15 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 
     // Set max number of results that we want - this is used to decide
     // when to throw away unwanted items.
-    Xapian::doccount max_msize = first + check_at_least;
+    Xapian::doccount max_msize = first + maxitems;
     items.reserve(max_msize + 1);
 
-    // Set the minimum item, used to compare against to see if an item
-    // should be considered for the mset.
-    Xapian::Internal::MSetItem min_item(weight_cutoff, 0);
+    // Tracks the minimum item currently eligible for the MSet - we compare
+    // candidate items against this.
+    Xapian::Internal::MSetItem min_item(0.0, 0);
+
+    // Minimum weight an item must have to be worth considering.
+    Xapian::weight min_weight = weight_cutoff;
 
     Xapian::weight percent_factor = percent_cutoff / 100.0;
 
@@ -387,20 +385,23 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 
     while (true) {
 	if (recalculate_w_max) {
-	    if (min_item.wt > 0.0 && getorrecalc_maxweight(pl) < min_item.wt) {
+	    if (min_weight > 0.0 && getorrecalc_maxweight(pl) < min_weight) {
 		DEBUGLINE(MATCH, "*** TERMINATING EARLY (1)");
 		break;
 	    }
 	}
 
-	if (next_handling_prune(pl, min_item.wt, this)) {
+	if (next_handling_prune(pl, min_weight, this)) {
 	    DEBUGLINE(MATCH, "*** REPLACING ROOT");
 
-	    // no need for a full recalc (unless we've got to do one because
-	    // of a prune elsewhere) - we're just switching to a subtree
-	    if (getorrecalc_maxweight(pl) < min_item.wt) {
-		DEBUGLINE(MATCH, "*** TERMINATING EARLY (2)");
-		break;
+	    if (min_weight > 0.0) {
+		// No need for a full recalc (unless we've got to do one
+		// because of a prune elsewhere) - we're just switching to a
+		// subtree.
+		if (getorrecalc_maxweight(pl) < min_weight) {
+		    DEBUGLINE(MATCH, "*** TERMINATING EARLY (2)");
+		    break;
+		}
 	    }
 	}
 
@@ -409,47 +410,58 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    break;
 	}
 
-	Xapian::docid did = pl->get_docid();
+	// Only calculate the weight if we need it for mcmp, or there's a
+	// percentage or weight cutoff in effect.  Otherwise we calculate it
+	// below if we haven't already rejected this candidate.
 	Xapian::weight wt = 0.0;
-	// Only calculate the weight if we need it for mcmp - otherwise
-	// we calculate it below only if we keep the item
-	if (min_item.wt > 0.0) wt = pl->get_weight();
+	bool calculated_weight = false;
+	if (sort_by != VAL || min_weight > 0.0) {
+	    wt = pl->get_weight();
+	    if (wt < min_weight) continue;
+	    calculated_weight = true;
+	}
 
+	Xapian::docid did = pl->get_docid();
 	DEBUGLINE(MATCH, "Candidate document id " << did << " wt " << wt);
 	Xapian::Internal::MSetItem new_item(wt, did);
-	if (sort_key != Xapian::valueno(-1)) {
+	if (sort_by != REL) {
 	    const unsigned int multiplier = db.internal.size();
 	    Assert(multiplier != 0);
 	    Xapian::doccount n = (new_item.did - 1) % multiplier; // which actual database
 	    Xapian::docid m = (new_item.did - 1) / multiplier + 1; // real docid in that database
 	    Xapian::Internal::RefCntPtr<Xapian::Document::Internal> doc(db.internal[n]->open_document(m, true));
 	    new_item.sort_key = doc->get_value(sort_key);
-	}
 
-	// Test if item has high enough weight (or sort key) to get into
-	// proto-mset.
-	if (sort_key != Xapian::valueno(-1) || min_item.wt > 0.0)
+	    // We're sorting by value (in part at least), so compare the item
+	    // against the lowest currently in the proto-mset.  If sort_by is
+	    // VAL, then new_item.wt won't yet be set, but that doesn't
+	    // matter since it's not used by the sort function.
 	    if (!mcmp(new_item, min_item)) continue;
+	}
 
 	Xapian::Internal::RefCntPtr<Xapian::Document::Internal> doc;
 
-	// Use the decision functor if any.
-	// FIXME: if results are from MSetPostList then we can omit this step
-	if (mdecider != NULL) {
-	    if (doc.get() == 0) {
-		const unsigned int multiplier = db.internal.size();
-		Assert(multiplier != 0);
-		Xapian::doccount n = (did - 1) % multiplier; // which actual database
-		Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
+	// Use the match spy and/or decision functors (if specified).
+	if (matchspy != NULL || mdecider != NULL) {
+	    const unsigned int multiplier = db.internal.size();
+	    Assert(multiplier != 0);
+	    Xapian::doccount n = (did - 1) % multiplier; // which actual database
+	    // If the results are from a remote database, then the functor will
+	    // already have been applied there so we can skip this step.
+	    if (!is_remote[n]) {
+		if (doc.get() == 0) {
+		    Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
 
-		Xapian::Internal::RefCntPtr<Xapian::Document::Internal> temp(db.internal[n]->open_document(m, true));
-		doc = temp;
+		    Xapian::Internal::RefCntPtr<Xapian::Document::Internal> temp(db.internal[n]->open_document(m, true));
+		    doc = temp;
+		}
+		Xapian::Document mydoc(doc.get());
+		if (matchspy && !matchspy->operator()(mydoc)) continue;
+		if (mdecider && !mdecider->operator()(mydoc)) continue;
 	    }
-	    Xapian::Document mydoc(doc.get());
-	    if (!mdecider->operator()(mydoc)) continue;
 	}
 
-	if (min_item.wt <= 0.0) {
+	if (!calculated_weight) {
 	    // we didn't calculate the weight above, but now we will need it
 	    wt = pl->get_weight();
 	    new_item.wt = wt;
@@ -459,7 +471,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	documents_considered++;
 
 	// Perform collapsing on key if requested.
-	if (collapse_key != Xapian::valueno(-1)) {
+	if (collapse_key != Xapian::BAD_VALUENO) {
 	    new_item.collapse_key = get_collapse_key(pl, did, collapse_key,
 						     doc);
 
@@ -476,8 +488,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		} else {
 		    ++duplicates_found;
 		    Xapian::Internal::MSetItem &old_item = oldkey->second.first;
-		    // FIXME: what about sort_key != Xapian::valueno(-1) case
-		    // here?
+		    // FIXME: what about the (sort_by != REL) case here?
 		    if (mcmp(old_item, new_item)) {
 			DEBUGLINE(MATCH, "collapsem: better exists: " <<
 				  new_item.collapse_key);
@@ -493,28 +504,36 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		    // replacement item
 		    new_item.collapse_count = old_item.collapse_count + 1;
 
-		    // This is best match with this key so far:
-		    // remove the old one from the MSet
-		    if ((sort_key == Xapian::valueno(-1) && min_item.wt <= 0.0) ||
-			mcmp(old_item, min_item)) {
-			// Old one hasn't fallen out of MSet yet
-			// Scan through (unsorted) MSet looking for entry
+		    // There was a previous item in the collapse tab so
+		    // the MSet can't be empty.
+		    Assert(!items.empty());
+
+		    // This is best potential MSet entry with this key which
+		    // we've seen so far.  Check if the previous best entry
+		    // with this key might still be in the proto-MSet.  If it
+		    // might be, we need to check through for it.
+		    if (old_item.wt >= min_weight && mcmp(old_item, min_item)) {
+			// Scan through (unsorted) MSet looking for entry.
 			// FIXME: more efficient way than just scanning?
 			Xapian::docid olddid = old_item.did;
-			DEBUGLINE(MATCH, "collapsem: removing " << olddid <<
-				  ": " << new_item.collapse_key);
 			vector<Xapian::Internal::MSetItem>::iterator i;
-			for (i = items.begin(); i->did != olddid; ++i) {
-			    Assert(i != items.end());
+			for (i = items.begin(); i != items.end(); ++i) {
+			    if (i->did == olddid) {
+				DEBUGLINE(MATCH, "collapsem: removing " <<
+					  olddid << ": " <<
+					  new_item.collapse_key);
+				// We can replace an arbitrary element in
+				// O(log N) but have to do it by hand (in this
+				// case the new elt is bigger, so we just swap
+				// down the tree).
+				// FIXME: implement this, and clean up is_heap
+				// handling
+				*i = new_item;
+				pushback = false;
+				is_heap = false;
+				break;
+			    }
 			}
-			*i = new_item;
-			pushback = false;
-			// We can replace an arbitrary element in O(log N)
-			// but have to do it by hand (in this case the new
-			// elt is bigger, so we just swap down the tree)
-			// FIXME: implement this, and clean up is_heap
-			// handling
-			is_heap = false;
 		    }
 
 		    // Keep the old weight as it is now second best so far
@@ -540,21 +559,21 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		pop_heap<vector<Xapian::Internal::MSetItem>::iterator,
 			 MSetCmp>(items.begin(), items.end(), mcmp);
 		items.pop_back();
-		if (sort_key != Xapian::valueno(-1)) {
-		    Xapian::weight tmp = min_item.wt;
-		    min_item = items.front();
-		    min_item.wt = tmp;
-		} else {
-		    min_item = items.front();
+
+		min_item = items.front();
+		if (sort_by == REL || sort_by == REL_VAL) {
+		    if (docs_matched >= check_at_least) {
+			if (min_item.wt > min_weight) min_weight = min_item.wt;
+		    }
 		}
-		if (getorrecalc_maxweight(pl) < min_item.wt) {
+		if (getorrecalc_maxweight(pl) < min_weight) {
 		    DEBUGLINE(MATCH, "*** TERMINATING EARLY (3)");
 		    break;
 		}
 	    } else {
 		items.push_back(new_item);
 		is_heap = false;
-		if (sort_key == Xapian::valueno(-1) && items.size() == max_msize) {
+		if (sort_by == REL && items.size() == max_msize) {
 		    // We're done if this is a forward boolean match
 		    // (bodgetastic, FIXME better if we can)
 		    if (max_weight == 0 && sort_forward) break;
@@ -567,24 +586,23 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    greatest_wt = wt;
 	    if (percent_cutoff) {
 		Xapian::weight w = wt * percent_factor;
-		if (w > min_item.wt) {
-		    min_item.wt = w;
-		    min_item.did = 0;
+		if (w > min_weight) {
+		    min_weight = w;
 		    if (!is_heap) {
 			is_heap = true;
 			make_heap<vector<Xapian::Internal::MSetItem>::iterator,
 				  MSetCmp>(items.begin(), items.end(), mcmp);
 		    }
-		    while (!items.empty() && items.front().wt < min_item.wt) {
+		    while (!items.empty() && items.front().wt < min_weight) {
 			pop_heap<vector<Xapian::Internal::MSetItem>::iterator,
 				 MSetCmp>(items.begin(), items.end(), mcmp);
-			Assert(items.back().wt < min_item.wt);
+			Assert(items.back().wt < min_weight);
 			items.pop_back();
 		    }
 #ifdef XAPIAN_DEBUG_PARANOID
 		    vector<Xapian::Internal::MSetItem>::const_iterator i;
 		    for (i = items.begin(); i != items.end(); ++i) {
-			Assert(i->wt >= min_item.wt);
+			Assert(i->wt >= min_weight);
 		    }
 #endif
 		}
@@ -602,34 +620,46 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	vector<Xapian::Internal::MSetItem>::const_iterator best;
 	best = min_element(items.begin(), items.end(), mcmp);
 
-	Xapian::termcount matching_terms = 0;
-	map<string,
-	    Xapian::MSet::Internal::TermFreqAndWeight>::const_iterator i;
+	if (termfreqandwts.size() > 1) {
+	    Xapian::termcount matching_terms = 0;
+	    map<string,
+		Xapian::MSet::Internal::TermFreqAndWeight>::const_iterator i;
 
-	Xapian::TermIterator docterms = db.termlist_begin(best->did);
-	Xapian::TermIterator docterms_end = db.termlist_end(best->did);
-	while (docterms != docterms_end) {
-	    i = termfreqandwts.find(*docterms);
+	    Xapian::TermIterator docterms = db.termlist_begin(best->did);
+	    Xapian::TermIterator docterms_end = db.termlist_end(best->did);
+	    while (docterms != docterms_end) {
+		i = termfreqandwts.find(*docterms);
+		if (i != termfreqandwts.end()) {
+		    percent_scale += i->second.termweight;
+		    ++matching_terms;
+		    if (matching_terms == termfreqandwts.size()) break;
+		}
+		++docterms;
+	    }
+	    // Special case for MatchAll queries
+	    i = termfreqandwts.find("");
 	    if (i != termfreqandwts.end()) {
 		percent_scale += i->second.termweight;
 		++matching_terms;
-		if (matching_terms == termfreqandwts.size()) break;
 	    }
-	    ++docterms;
-	}
-	if (matching_terms < termfreqandwts.size()) {
-	    // OK, work out weight corresponding to 100%
-	    double denom = 0;
-	    for (i = termfreqandwts.begin(); i != termfreqandwts.end(); ++i)
-		denom += i->second.termweight;
+	    if (matching_terms < termfreqandwts.size()) {
+		// OK, work out weight corresponding to 100%
+		double denom = 0;
+		for (i = termfreqandwts.begin(); i != termfreqandwts.end(); ++i)
+		    denom += i->second.termweight;
 
-	    DEBUGLINE(MATCH, "denom = " << denom << " percent_scale = " << percent_scale);
-	    Assert(percent_scale <= denom);
-	    denom *= greatest_wt;
-	    Assert(denom > 0);
-	    percent_scale /= denom;
+		DEBUGLINE(MATCH, "denom = " << denom << " percent_scale = " << percent_scale);
+		Assert(percent_scale <= denom);
+		denom *= greatest_wt;
+		Assert(denom > 0);
+		percent_scale /= denom;
+	    } else {
+		// If all the terms match, the 2 sums of weights cancel
+		percent_scale = 1.0 / greatest_wt;
+	    }
 	} else {
-	    // If all the terms match, the 2 sums of weights cancel
+	    // If there's only a single term in the query, the top document
+	    // must score 100%.
 	    percent_scale = 1.0 / greatest_wt;
 	}
 	Assert(percent_scale > 0);
@@ -669,11 +699,11 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    = items.size();
     } else {
 	if (percent_cutoff) {
-	    // another approach: Xapian::doccount new_est = items.size() * (1 - percent_factor) / (1 - min_item.wt / greatest_wt);
+	    // another approach: Xapian::doccount new_est = items.size() * (1 - percent_factor) / (1 - min_weight / greatest_wt);
 	    Xapian::doccount new_est;
 	    new_est = Xapian::doccount((1 - percent_factor) * matches_estimated);
 	    matches_estimated = max(size_t(new_est), items.size());
-	    // and another: items.size() + (1 - greatest_wt * percent_factor / min_item.wt) * (matches_estimated - items.size());
+	    // and another: items.size() + (1 - greatest_wt * percent_factor / min_weight) * (matches_estimated - items.size());
 
 	    // Very likely an underestimate, but we can't really do better without
 	    // checking further matches...  Only possibility would be to track how
@@ -694,7 +724,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	Assert(matches_estimated >= matches_lower_bound);
 	Assert(matches_estimated <= matches_upper_bound);
 
-	if (collapse_key != Xapian::valueno(-1)) {
+	if (collapse_key != Xapian::BAD_VALUENO) {
 	    // Lower bound must be set to no more than the number of collapse
 	    // values we've got, since it's possible that all further hits
 	    // will be collapsed to a single value.
@@ -728,19 +758,6 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 
     DEBUGLINE(MATCH, items.size() << " items in potential mset");
 
-    if (check_at_least > maxitems) {
-	// Remove unwanted trailing entries
-	if (maxitems == 0) {
-	    items.clear();
-	} else if (items.size() > first + maxitems) {
-	    nth_element(items.begin(),
-			items.begin() + first + maxitems,
-			items.end(),
-			mcmp);
-	    // Erase the unwanted trailing items.
-	    items.erase(items.begin() + first + maxitems);
-	}
-    }
     if (first > 0) {
 	// Remove unwanted leading entries
 	if (items.size() <= first) {
@@ -777,7 +794,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     // of an item that has already been pushed-back as we don't know where it is
     // any more.  If we keep or find references we won't need to mess with
     // is_heap so much maybe?
-    if (collapse_key != Xapian::valueno(-1) && /*percent_cutoff &&*/ !items.empty() &&
+    if (collapse_key != Xapian::BAD_VALUENO && /*percent_cutoff &&*/ !items.empty() &&
 	!collapse_tab.empty()) {
 	// Nicked this formula from above, but for some reason percent_scale
 	// has since been multiplied by 100 so we take that into account
