@@ -1,6 +1,5 @@
 /* phrasepostlist.cc: Return only items where terms are near or form a phrase
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004 Olly Betts
@@ -17,14 +16,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #include <config.h>
+
 #include "phrasepostlist.h"
 #include "positionlist.h"
+#include "omassert.h"
 #include "omdebug.h"
 
 #include <algorithm>
@@ -94,6 +94,60 @@ NearPostList::do_test(std::vector<PositionList *> &plists, Xapian::termcount i,
     RETURN(false);
 }
 
+Xapian::termcount
+NearPostList::get_wdf() const
+{
+    // Calculate an estimate for the wdf of a near postlist.
+    //
+    // The natural definition of the wdf for a near postlist is the number of
+    // times the terms occur in a group within the windowsize in the document -
+    // in other words, the number of times a routine like do_test() could find
+    // a match, if it kept going after finding the first one.  However,
+    // calculating this would be expensive (in CPU time at least - the position
+    // lists are probably already read from disk), and the benefit is dubious
+    // (given that the estimate here is probably only going to be used for
+    // calculating the weight for synonym postlists, and it's not clear that
+    // the above definition is exactly appropriate in this situation, anyway).
+    //
+    // Instead, we could do an estimate of this value, based on the the lengths
+    // of the position lists.  Rather than having to open the position lists,
+    // we could use the wdfs, which will be the same value unless the wdfs have
+    // been artificially inflated - in which case we probably want to use the
+    // inflated value anyway (it will have been inflated to represent the fact
+    // that this term is more important than others, so we should make use of
+    // this hint).
+    //
+    // A reasonable way to calculate an estimate would be to consider the
+    // document to be split into W (overlapping) windows, each 1 term apart,
+    // and of the same length as the windowsize used for the phrase.  Then,
+    // calculate the probability that each term is found in this window (as
+    // Prob = wdf / doclen), multiply these probabilities to get the
+    // probability that they're all found in the window, and then multiply by
+    // the windowsize again to get an estimate.
+    //
+    // However, this requires the doclen, which won't always be available (ie,
+    // if the weighting scheme doesn't use it, we won't have read it from
+    // disk).
+    //
+    // Rather than force an access to disk to get the doclen, we use an even
+    // rougher estimate: the minimum wdf in the sub-lists.  This is actually
+    // the upper bound for the wdf (since the phrase can only occur the same
+    // number of times as the least frequent of its constituent terms).
+    //
+    // If this estimate proves to give bad results, we can revisit this and try
+    // a better approximation.
+
+    std::vector<PostList *>::const_iterator i = terms.begin();
+    Xapian::termcount wdf = (*i)->get_wdf();
+    for (; i != terms.end(); i++) {
+	wdf = std::min(wdf, (*i)->get_wdf());
+    }
+
+    // Ensure that we always return a wdf of at least 1, since we know there
+    // was at least one occurrence of the phrase.
+    return std::max(wdf, 1u);
+}
+
 std::string
 NearPostList::get_description() const
 {
@@ -186,6 +240,29 @@ PhrasePostList::do_test(std::vector<PositionList *> &plists, Xapian::termcount i
     RETURN(false);
 }
 
+Xapian::termcount
+PhrasePostList::get_wdf() const
+{
+    // Calculate the an estimate for the wdf of a phrase postlist.
+    //
+    // We use the minimum wdf of a sub-postlist as our estimate.  See the
+    // comment in NearPostList::get_wdf for justification of this estimate.
+    //
+    // We divide the value calculated for a NearPostList by 2, as a very rough
+    // heuristic to represent the fact that the words must occur in order, and
+    // phrases are therefore rarer than near matches.
+
+    std::vector<PostList *>::const_iterator i = terms.begin();
+    Xapian::termcount wdf = (*i)->get_wdf();
+    for (; i != terms.end(); i++) {
+	wdf = std::min(wdf, (*i)->get_wdf());
+    }
+
+    // Ensure that we always return a wdf of at least 1, since we know there
+    // was at least one occurrence of the phrase.
+    return std::max(wdf / 2, 1u);
+}
+
 std::string
 PhrasePostList::get_description() const
 {
