@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,7 +31,6 @@
 #include "utils.h"
 
 #include "apitest.h"
-#include "api_wrdb.h"
 
 #include <list>
 
@@ -54,18 +54,6 @@ print_mset_percentages(const Xapian::MSet &mset)
     }
 }
 
-static Xapian::MSet
-do_get_simple_query_mset(Xapian::Query query, int maxitems = 10, int first = 0)
-{
-    // open the database (in this case a simple text file
-    // we prepared earlier)
-    Xapian::Enquire enquire(get_database("apitest_simpledata"));
-    enquire.set_query(query);
-
-    // retrieve the top results
-    return enquire.get_mset(first, maxitems);
-}
-
 static Xapian::Query
 query(Xapian::Query::op op, string t1 = "", string t2 = "",
       string t3 = "", string t4 = "", string t5 = "",
@@ -74,16 +62,16 @@ query(Xapian::Query::op op, string t1 = "", string t2 = "",
 {
     vector<string> v;
     Xapian::Stem stemmer("english");
-    if (!t1.empty()) v.push_back(stemmer.stem_word(t1));
-    if (!t2.empty()) v.push_back(stemmer.stem_word(t2));
-    if (!t3.empty()) v.push_back(stemmer.stem_word(t3));
-    if (!t4.empty()) v.push_back(stemmer.stem_word(t4));
-    if (!t5.empty()) v.push_back(stemmer.stem_word(t5));
-    if (!t6.empty()) v.push_back(stemmer.stem_word(t6));
-    if (!t7.empty()) v.push_back(stemmer.stem_word(t7));
-    if (!t8.empty()) v.push_back(stemmer.stem_word(t8));
-    if (!t9.empty()) v.push_back(stemmer.stem_word(t9));
-    if (!t10.empty()) v.push_back(stemmer.stem_word(t10));
+    if (!t1.empty()) v.push_back(stemmer(t1));
+    if (!t2.empty()) v.push_back(stemmer(t2));
+    if (!t3.empty()) v.push_back(stemmer(t3));
+    if (!t4.empty()) v.push_back(stemmer(t4));
+    if (!t5.empty()) v.push_back(stemmer(t5));
+    if (!t6.empty()) v.push_back(stemmer(t6));
+    if (!t7.empty()) v.push_back(stemmer(t7));
+    if (!t8.empty()) v.push_back(stemmer(t8));
+    if (!t9.empty()) v.push_back(stemmer(t9));
+    if (!t10.empty()) v.push_back(stemmer(t10));
     return Xapian::Query(op, v.begin(), v.end());
 }
 
@@ -96,23 +84,23 @@ query(Xapian::Query::op op, Xapian::termcount parameter,
 {
     vector<string> v;
     Xapian::Stem stemmer("english");
-    if (!t1.empty()) v.push_back(stemmer.stem_word(t1));
-    if (!t2.empty()) v.push_back(stemmer.stem_word(t2));
-    if (!t3.empty()) v.push_back(stemmer.stem_word(t3));
-    if (!t4.empty()) v.push_back(stemmer.stem_word(t4));
-    if (!t5.empty()) v.push_back(stemmer.stem_word(t5));
-    if (!t6.empty()) v.push_back(stemmer.stem_word(t6));
-    if (!t7.empty()) v.push_back(stemmer.stem_word(t7));
-    if (!t8.empty()) v.push_back(stemmer.stem_word(t8));
-    if (!t9.empty()) v.push_back(stemmer.stem_word(t9));
-    if (!t10.empty()) v.push_back(stemmer.stem_word(t10));
+    if (!t1.empty()) v.push_back(stemmer(t1));
+    if (!t2.empty()) v.push_back(stemmer(t2));
+    if (!t3.empty()) v.push_back(stemmer(t3));
+    if (!t4.empty()) v.push_back(stemmer(t4));
+    if (!t5.empty()) v.push_back(stemmer(t5));
+    if (!t6.empty()) v.push_back(stemmer(t6));
+    if (!t7.empty()) v.push_back(stemmer(t7));
+    if (!t8.empty()) v.push_back(stemmer(t8));
+    if (!t9.empty()) v.push_back(stemmer(t9));
+    if (!t10.empty()) v.push_back(stemmer(t10));
     return Xapian::Query(op, v.begin(), v.end(), parameter);
 }
 
 static Xapian::Query
 query(const string &t)
 {
-    return Xapian::Query(Xapian::Stem("english").stem_word(t));
+    return Xapian::Query(Xapian::Stem("english")(t));
 }
 
 // #######################################################################
@@ -147,25 +135,32 @@ static bool test_zerodocid1()
 // tests that an empty query returns no matches
 static bool test_emptyquery1()
 {
-    Xapian::MSet mymset = do_get_simple_query_mset(Xapian::Query());
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+
+    enquire.set_query(Xapian::Query());
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 0);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 0);
+
     vector<Xapian::Query> v;
-    mymset = do_get_simple_query_mset(Xapian::Query(Xapian::Query::OP_AND,
-					      v.begin(), v.end()));
+    enquire.set_query(Xapian::Query(Xapian::Query::OP_AND, v.begin(), v.end()));
+    mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 0);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 0);
+
     return true;
 }
 
 // tests the document count for a simple query
 static bool test_simplequery1()
 {
-    Xapian::MSet mymset = do_get_simple_query_mset(Xapian::Query("word"));
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("word"));
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 2);
     return true;
 }
@@ -188,18 +183,10 @@ static bool test_simplequery2()
 
     // Check the weights
     Xapian::MSetIterator i = mymset.begin();
-    if (db.get_avlength() != 1.0) {
-	// These weights are for BM25Weight(1,0,1,0.5,0.5)
-	weights_are_equal_enough(i.get_weight(), 1.046482);
-	i++;
-	weights_are_equal_enough(i.get_weight(), 0.640988);
-    } else {
-	// These weights are for BM25Weight(1,0,1,0.5,0.5) when all
-	// doclengths are reported as 1 (e.g. muscat36 backend)
-	weights_are_equal_enough(i.get_weight(), 0.641854);
-	i++;
-	weights_are_equal_enough(i.get_weight(), 0.641854);
-    }
+    // These weights are for BM25Weight(1,0,1,0.5,0.5)
+    TEST_EQUAL_DOUBLE(i.get_weight(), 1.04648168717725);
+    i++;
+    TEST_EQUAL_DOUBLE(i.get_weight(), 0.640987686595914);
 
     return true;
 }
@@ -207,7 +194,9 @@ static bool test_simplequery2()
 // tests for the right document count for another simple query
 static bool test_simplequery3()
 {
-    Xapian::MSet mymset = do_get_simple_query_mset(query("this"));
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(query("this"));
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
 
     // Check that 6 documents were returned.
     TEST_MSET_SIZE(mymset, 6);
@@ -215,6 +204,43 @@ static bool test_simplequery3()
     return true;
 }
 
+// tests for the right document count for a wildcard query
+// FIXME: move this to querytest (and just use an InMemory DB).
+static bool test_wildquery1()
+{
+    Xapian::QueryParser queryparser;
+    unsigned flags = Xapian::QueryParser::FLAG_WILDCARD |
+		     Xapian::QueryParser::FLAG_LOVEHATE;
+    queryparser.set_stemmer(Xapian::Stem("english"));
+    queryparser.set_stemming_strategy(Xapian::QueryParser::STEM_ALL);
+    Xapian::Database db = get_database("apitest_simpledata");
+    queryparser.set_database(db);
+    Xapian::Enquire enquire(db);
+
+    Xapian::Query qobj = queryparser.parse_query("th*", flags);
+    tout << qobj.get_description() << endl;
+    enquire.set_query(qobj);
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
+    // Check that 6 documents were returned.
+    TEST_MSET_SIZE(mymset, 6);
+
+    qobj = queryparser.parse_query("notindb* \"this\"", flags);
+    tout << qobj.get_description() << endl;
+    enquire.set_query(qobj);
+    mymset = enquire.get_mset(0, 10);
+    // Check that 6 documents were returned.
+    TEST_MSET_SIZE(mymset, 6);
+
+    qobj = queryparser.parse_query("+notindb* \"this\"", flags);
+    tout << qobj.get_description() << endl;
+    enquire.set_query(qobj);
+    mymset = enquire.get_mset(0, 10);
+    // Check that 0 documents were returned.
+    TEST_MSET_SIZE(mymset, 0);
+
+    return true;
+}
+
 // tests a query across multiple databases
 static bool test_multidb1()
 {
@@ -325,12 +351,42 @@ static bool test_multidb5()
     return true;
 }
 
-// tests that when specifiying maxitems to get_mset, no more than
+// tests that when specifying maxitems to get_mset, no more than
 // that are returned.
 static bool test_msetmaxitems1()
 {
-    Xapian::MSet mymset = do_get_simple_query_mset(query("this"), 1);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(query("this"));
+    Xapian::MSet mymset = enquire.get_mset(0, 1);
     TEST_MSET_SIZE(mymset, 1);
+
+    mymset = enquire.get_mset(0, 5);
+    TEST_MSET_SIZE(mymset, 5);
+
+    return true;
+}
+
+// tests the returned weights are as expected (regression test for remote
+// backend which was using the average weight rather than the actual document
+// weight for computing weights - fixed in 1.0.0).
+static bool test_expandweights1()
+{
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("this"));
+
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
+
+    Xapian::RSet myrset;
+    Xapian::MSetIterator i = mymset.begin();
+    myrset.add_document(*i);
+    myrset.add_document(*(++i));
+
+    Xapian::ESet myeset = enquire.get_eset(3, myrset);
+    TEST_EQUAL(myeset.size(), 3);
+    TEST_EQUAL_DOUBLE(myeset[0].get_weight(), 6.08904001099445);
+    TEST_EQUAL_DOUBLE(myeset[1].get_weight(), 6.08904001099445);
+    TEST_EQUAL_DOUBLE(myeset[2].get_weight(), 4.73383620844021);
+
     return true;
 }
 
@@ -381,18 +437,14 @@ static bool test_boolquery1()
 // tests that get_mset() specifying "this" works as expected
 static bool test_msetfirst1()
 {
-    Xapian::MSet mymset1 = do_get_simple_query_mset(query("this"), 6, 0);
-    Xapian::MSet mymset2 = do_get_simple_query_mset(query("this"), 3, 3);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(query("this"));
+    Xapian::MSet mymset1 = enquire.get_mset(0, 6);
+    Xapian::MSet mymset2 = enquire.get_mset(3, 3);
     TEST(mset_range_is_same(mymset1, 3, mymset2, 0, 3));
-    return true;
-}
 
-// Regression test - we weren't adjusting the index into items[] by firstitem
-// in api/omenquire.cc.
-static bool test_msetfirst2()
-{
-    Xapian::MSet mymset1 = do_get_simple_query_mset(query("this"), 6, 0);
-    Xapian::MSet mymset2 = do_get_simple_query_mset(query("this"), 3, 3);
+    // Regression test - we weren't adjusting the index into items[] by
+    // firstitem in api/omenquire.cc.
     TEST_EQUAL(mymset1[5].get_document().get_data(),
 	       mymset2[2].get_document().get_data());
     return true;
@@ -401,7 +453,9 @@ static bool test_msetfirst2()
 // tests the converting-to-percent functions
 static bool test_topercent1()
 {
-    Xapian::MSet mymset = do_get_simple_query_mset(query("this"), 20, 0);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(query("this"));
+    Xapian::MSet mymset = enquire.get_mset(0, 20);
 
     int last_pct = 100;
     Xapian::MSetIterator i = mymset.begin();
@@ -421,7 +475,7 @@ static bool test_topercent1()
 
 class myExpandFunctor : public Xapian::ExpandDecider {
     public:
-	int operator()(const string & tname) const {
+	bool operator()(const string & tname) const {
 	    unsigned long sum = 0;
 	    for (string::const_iterator i=tname.begin(); i!=tname.end(); ++i) {
 		sum += *i;
@@ -623,7 +677,7 @@ static bool test_allowqterms1()
         TEST_NOT_EQUAL(*j, "this");
     }
 
-    Xapian::ESet myeset2 = enquire.get_eset(1000, myrset, Xapian::Enquire::include_query_terms);
+    Xapian::ESet myeset2 = enquire.get_eset(1000, myrset, Xapian::Enquire::INCLUDE_QUERY_TERMS);
     j = myeset2.begin();
     for ( ; j != myeset2.end(); ++j) {
         if (*j == "this") break;
@@ -635,7 +689,9 @@ static bool test_allowqterms1()
 // tests that the MSet max_attained works
 static bool test_maxattain1()
 {
-    Xapian::MSet mymset = do_get_simple_query_mset(query("this"), 100, 0);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(query("this"));
+    Xapian::MSet mymset = enquire.get_mset(0, 100);
 
     Xapian::weight mymax = 0;
     Xapian::MSetIterator i = mymset.begin();
@@ -818,22 +874,18 @@ static bool test_poscollapse1()
 	tout << myquery2.get_description() << "\n";
     }
 
-    Xapian::MSet mymset1 = do_get_simple_query_mset(myquery1);
-    Xapian::MSet mymset2 = do_get_simple_query_mset(myquery2);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(myquery1);
+    Xapian::MSet mymset1 = enquire.get_mset(0, 10);
+
+    enquire.set_query(myquery2);
+    Xapian::MSet mymset2 = enquire.get_mset(0, 10);
 
     TEST_EQUAL(mymset1, mymset2);
 
     return true;
 }
 
-// tests that the collapsing on termpos optimisation gives correct query length
-static bool test_poscollapse2()
-{
-    Xapian::Query q(Xapian::Query::OP_OR, Xapian::Query("this", 1, 1), Xapian::Query("this", 1, 1));
-    TEST_EQUAL(q.get_length(), 2);
-    return true;
-}
-
 // test that running a query twice returns the same results
 static bool test_repeatquery1()
 {
@@ -1065,9 +1117,9 @@ static bool test_eliteset3()
     // make a query
     Xapian::Stem stemmer("english");
 
-    string term1 = stemmer.stem_word("word");
-    string term2 = stemmer.stem_word("rubbish");
-    string term3 = stemmer.stem_word("banana");
+    string term1 = stemmer("word");
+    string term2 = stemmer("rubbish");
+    string term3 = stemmer("banana");
 
     vector<string> terms;
     terms.push_back(term1);
@@ -1142,7 +1194,7 @@ static bool test_termlisttermfreq1()
     Xapian::ESet eset2 = enquire.get_eset(1000, rset2);
 
     // search for weight of term 'another'
-    string theterm = stemmer.stem_word("another");
+    string theterm = stemmer("another");
 
     Xapian::weight wt1 = 0;
     Xapian::weight wt2 = 0;
@@ -1185,9 +1237,9 @@ static bool test_qterminfo1()
     // make a query
     Xapian::Stem stemmer("english");
 
-    string term1 = stemmer.stem_word("word");
-    string term2 = stemmer.stem_word("inmemory");
-    string term3 = stemmer.stem_word("flibble");
+    string term1 = stemmer("word");
+    string term2 = stemmer("inmemory");
+    string term3 = stemmer("flibble");
 
     Xapian::Query myquery(Xapian::Query::OP_OR,
 		    Xapian::Query(term1),
@@ -1238,8 +1290,8 @@ static bool test_qterminfo2()
     // make a query
     Xapian::Stem stemmer("english");
 
-    string term1 = stemmer.stem_word("paragraph");
-    string term2 = stemmer.stem_word("another");
+    string term1 = stemmer("paragraph");
+    string term2 = stemmer("another");
 
     Xapian::Query query(Xapian::Query::OP_AND_NOT, term1,
 	    Xapian::Query(Xapian::Query::OP_AND, term1, term2));
@@ -1254,12 +1306,15 @@ static bool test_qterminfo2()
     return true;
 }
 
-// tests that when specifiying that no items are to be returned, those
+// tests that when specifying that no items are to be returned, those
 // statistics which should be the same are.
 static bool test_msetzeroitems1()
 {
-    Xapian::MSet mymset1 = do_get_simple_query_mset(query("this"), 0);
-    Xapian::MSet mymset2 = do_get_simple_query_mset(query("this"), 1);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(query("this"));
+    Xapian::MSet mymset1 = enquire.get_mset(0, 0);
+
+    Xapian::MSet mymset2 = enquire.get_mset(0, 1);
 
     TEST_EQUAL(mymset1.get_max_possible(), mymset2.get_max_possible());
 
@@ -1269,66 +1324,73 @@ static bool test_msetzeroitems1()
 // test that the matches_* of a simple query are as expected
 static bool test_matches1()
 {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
     Xapian::Query myquery;
     Xapian::MSet mymset;
 
     myquery = query("word");
-    mymset = do_get_simple_query_mset(myquery);
+    enquire.set_query(myquery);
+    mymset = enquire.get_mset(0, 10);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_OR, "inmemory", "word");
-    mymset = do_get_simple_query_mset(myquery);
+    enquire.set_query(myquery);
+    mymset = enquire.get_mset(0, 10);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_AND, "inmemory", "word");
-    mymset = do_get_simple_query_mset(myquery);
+    enquire.set_query(myquery);
+    mymset = enquire.get_mset(0, 10);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 0);
 
     myquery = query(Xapian::Query::OP_AND, "simple", "word");
-    mymset = do_get_simple_query_mset(myquery);
+    enquire.set_query(myquery);
+    mymset = enquire.get_mset(0, 10);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_AND, "simple", "word");
-    mymset = do_get_simple_query_mset(myquery, 0);
+    enquire.set_query(myquery);
+    mymset = enquire.get_mset(0, 0);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
-    mymset = do_get_simple_query_mset(myquery, 1);
+    mymset = enquire.get_mset(0, 1);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
-    mymset = do_get_simple_query_mset(myquery, 2);
+    mymset = enquire.get_mset(0, 2);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
     TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
     myquery = query(Xapian::Query::OP_AND, "paragraph", "another");
-    mymset = do_get_simple_query_mset(myquery, 0);
+    enquire.set_query(myquery);
+    mymset = enquire.get_mset(0, 0);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
-    mymset = do_get_simple_query_mset(myquery, 1);
+    mymset = enquire.get_mset(0, 1);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
 
-    mymset = do_get_simple_query_mset(myquery, 2);
+    mymset = enquire.get_mset(0, 2);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
 
-    mymset = do_get_simple_query_mset(myquery, 1, 20);
+    mymset = enquire.get_mset(1, 20);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     TEST_EQUAL(mymset.get_matches_estimated(), 1);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
@@ -1342,8 +1404,11 @@ static bool test_wqf1()
     // Both queries have length 2; in q1 word has wqf=2, in q2 word has wqf=1
     Xapian::Query q1("word", 2);
     Xapian::Query q2("word");
-    Xapian::MSet mset1 = do_get_simple_query_mset(q1);
-    Xapian::MSet mset2 = do_get_simple_query_mset(q2, 2);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(q1);
+    Xapian::MSet mset1 = enquire.get_mset(0, 10);
+    enquire.set_query(q2);
+    Xapian::MSet mset2 = enquire.get_mset(0, 2);
     // Check the weights
     TEST(mset1.begin().get_weight() > mset2.begin().get_weight());
     return true;
@@ -1354,8 +1419,11 @@ static bool test_qlen1()
 {
     Xapian::Query q1("word");
     Xapian::Query q2("word");
-    Xapian::MSet mset1 = do_get_simple_query_mset(q1);
-    Xapian::MSet mset2 = do_get_simple_query_mset(q2, 2);
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(q1);
+    Xapian::MSet mset1 = enquire.get_mset(0, 10);
+    enquire.set_query(q2);
+    Xapian::MSet mset2 = enquire.get_mset(0, 2);
     // Check the weights
     //TEST(mset1.begin().get_weight() < mset2.begin().get_weight());
     TEST(mset1.begin().get_weight() == mset2.begin().get_weight());
@@ -1470,7 +1538,7 @@ static bool test_spaceterms1()
     Xapian::MSetIterator m;
     Xapian::Stem stemmer("english");
 
-    enquire.set_query(stemmer.stem_word("space man"));
+    enquire.set_query(stemmer("space man"));
     mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 1);
     count = 0;
@@ -1482,7 +1550,7 @@ static bool test_spaceterms1()
 	TEST_NOT_EQUAL(mymset.begin().get_document().get_value(value_no), "");
     }
 
-    enquire.set_query(stemmer.stem_word("tab\tby"));
+    enquire.set_query(stemmer("tab\tby"));
     mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 1);
     count = 0;
@@ -1498,7 +1566,7 @@ static bool test_spaceterms1()
 	}
     }
 
-    enquire.set_query(stemmer.stem_word("back\\slash"));
+    enquire.set_query(stemmer("back\\slash"));
     mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 1);
     count = 0;
@@ -1516,9 +1584,9 @@ static bool test_xor1()
     Xapian::Stem stemmer("english");
 
     vector<string> terms;
-    terms.push_back(stemmer.stem_word("this"));
-    terms.push_back(stemmer.stem_word("word"));
-    terms.push_back(stemmer.stem_word("of"));
+    terms.push_back(stemmer("this"));
+    terms.push_back(stemmer("word"));
+    terms.push_back(stemmer("of"));
 
     Xapian::Query query(Xapian::Query::OP_XOR, terms.begin(), terms.end());
     enquire.set_weighting_scheme(Xapian::BoolWeight());
@@ -1557,7 +1625,8 @@ static bool test_emptyop1()
 
     Xapian::Query query1(Xapian::Query::OP_XOR, nullvec.begin(), nullvec.end());
 
-    Xapian::MSet mymset = do_get_simple_query_mset(query1);
+    enquire.set_query(query1);
+    Xapian::MSet mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 0);
     TEST_EXCEPTION(Xapian::InvalidArgumentError, enquire.get_matching_terms_begin(1));
 
@@ -1575,26 +1644,186 @@ static bool test_checkatleast1()
     return true;
 }
 
+// Regression test - if check_at_least was set we returned (check_at_least - 1)
+// results, rather than the requested msize.  Fixed in 1.0.2.
+static bool test_checkatleast2()
+{
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("paragraph"));
+    Xapian::MSet mymset = enquire.get_mset(0, 3, 10);
+    TEST_MSET_SIZE(mymset, 3);
+    TEST(mymset.get_matches_lower_bound() >= 5);
+    
+    mymset = enquire.get_mset(0, 2, 4);
+    TEST_MSET_SIZE(mymset, 2);
+    TEST(mymset.get_matches_lower_bound() >= 4);
+    return true;
+}
+
+// Feature tests - check_at_least with various sorting options.
+static bool test_checkatleast3()
+{
+    Xapian::Enquire enquire(get_database("etext"));
+    enquire.set_query(Xapian::Query("prussian")); // 60 matches.
+
+    for (int order = 0; order < 3; ++order) {
+	switch (order) {
+	    case 0:
+		enquire.set_docid_order(Xapian::Enquire::ASCENDING);
+		break;
+	    case 1:
+		enquire.set_docid_order(Xapian::Enquire::DESCENDING);
+		break;
+	    case 2:
+		enquire.set_docid_order(Xapian::Enquire::DONT_CARE);
+		break;
+	}
+
+	for (int sort = 0; sort < 4; ++sort) {
+	    switch (sort) {
+		case 0:
+		    enquire.set_sort_by_relevance();
+		    break;
+		case 1:
+		    enquire.set_sort_by_value(0);
+		    break;
+		case 2:
+		    enquire.set_sort_by_value_then_relevance(0);
+		    break;
+		case 3:
+		    enquire.set_sort_by_relevance_then_value(0);
+		    break;
+	    }
+
+	    Xapian::MSet mset = enquire.get_mset(0, 100, 500);
+	    TEST_MSET_SIZE(mset, 60);
+	    TEST(mset.get_matches_lower_bound() >= 60);
+
+	    mset = enquire.get_mset(0, 50, 100);
+	    TEST_MSET_SIZE(mset, 50);
+	    TEST(mset.get_matches_lower_bound() >= 60);
+
+	    mset = enquire.get_mset(0, 10, 50);
+	    TEST_MSET_SIZE(mset, 10);
+	    TEST(mset.get_matches_lower_bound() >= 50);
+	}
+    }
+
+    return true;
+}
+
+// tests all document postlists
+static bool test_allpostlist1()
+{
+    Xapian::Database db(get_database("apitest_manydocs"));
+    Xapian::PostingIterator i = db.postlist_begin("");
+    unsigned int j = 1;
+    while (i != db.postlist_end("")) {
+	TEST_EQUAL(*i, j);
+	i++;
+	j++;
+    }
+    TEST_EQUAL(j, 513);
+
+    i = db.postlist_begin("");
+    j = 1;
+    while (i != db.postlist_end("")) {
+	TEST_EQUAL(*i, j);
+	i++;
+	j++;
+	if (j == 50) {
+	    j += 10;
+	    i.skip_to(j);
+	}
+    }
+    TEST_EQUAL(j, 513);
+
+    return true;
+}
+
+static void test_emptyterm1_helper(Xapian::Database & db)
+{
+    // Don't bother with postlist_begin() because allpostlist tests cover that.
+    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.positionlist_begin(1, ""));
+    TEST_EQUAL(db.get_doccount(), db.get_termfreq(""));
+    TEST_EQUAL(db.get_doccount() != 0, db.term_exists(""));
+    TEST_EQUAL(db.get_doccount(), db.get_collection_freq(""));
+}
+
+// tests results of passing an empty term to various methods
+static bool test_emptyterm1()
+{
+    Xapian::Database db(get_database("apitest_manydocs"));
+    TEST_EQUAL(db.get_doccount(), 512);
+    test_emptyterm1_helper(db);
+
+    db = get_database("apitest_onedoc");
+    TEST_EQUAL(db.get_doccount(), 1);
+    test_emptyterm1_helper(db);
+
+    db = get_database("");
+    TEST_EQUAL(db.get_doccount(), 0);
+    test_emptyterm1_helper(db);
+
+    return true;
+}
+
+// Feature test for Query::OP_VALUE_RANGE.
+static bool test_valuerange1() {
+    Xapian::Database db(get_database("apitest_phrase"));
+    Xapian::Enquire enq(db);
+    static const char * vals[] = {
+	"", " ", "a", "aa", "abcd", "e", "g", "h", "hzz", "i", "l", "z", NULL
+    };
+    for (const char **start = vals; *start; ++start) {
+	for (const char **end = vals; *end; ++end) {
+	    Xapian::Query query(Xapian::Query::OP_VALUE_RANGE, 1, *start, *end);
+	    enq.set_query(query);
+	    Xapian::MSet mset = enq.get_mset(0, 20);
+	    // Check that documents in the MSet match the value range filter.
+	    set<Xapian::docid> matched;
+	    Xapian::MSetIterator i;
+	    for (i = mset.begin(); i != mset.end(); ++i) {
+		matched.insert(*i);
+		string value = db.get_document(*i).get_value(1);
+		tout << "'" << *start << "' <= '" << value << "' <= '" << *end << "'" << endl;
+		TEST(value >= *start);
+		TEST(value <= *end);
+	    }
+	    // Check that documents not in the MSet don't match the value range filter.
+	    for (Xapian::docid j = db.get_lastdocid(); j != 0; --j) {
+		if (matched.find(j) == matched.end()) {
+		    string value = db.get_document(j).get_value(1);
+		    tout << value << " < '" << *start << "' or > '" << *end << "'" << endl;
+		    TEST(value < *start || value > *end);
+		}
+	    }
+	}
+    }
+    return true;
+}
+
 // #######################################################################
 // # End of test cases: now we list the tests to run.
 
 /// The tests which work with any backend
 test_desc anydb_tests[] = {
-    {"zerodocid1", 	   test_zerodocid1},
+    {"zerodocid1",	   test_zerodocid1},
     {"emptyquery1",	   test_emptyquery1},
     {"simplequery1",       test_simplequery1},
     {"simplequery2",       test_simplequery2},
     {"simplequery3",       test_simplequery3},
+    {"wildquery1",	   test_wildquery1},
     {"multidb1",           test_multidb1},
     {"multidb2",           test_multidb2},
     {"multidb3",           test_multidb3},
     {"multidb4",           test_multidb4},
     {"multidb5",           test_multidb5},
     {"msetmaxitems1",      test_msetmaxitems1},
+    {"expandweights1",	   test_expandweights1},
     {"expandmaxitems1",    test_expandmaxitems1},
     {"boolquery1",         test_boolquery1},
     {"msetfirst1",         test_msetfirst1},
-    {"msetfirst2",         test_msetfirst2},
     {"topercent1",	   test_topercent1},
     {"expandfunctor1",	   test_expandfunctor1},
     {"pctcutoff1",	   test_pctcutoff1},
@@ -1606,7 +1835,6 @@ test_desc anydb_tests[] = {
     {"getmterms1",	   test_getmterms1},
     {"getmterms2",	   test_getmterms2},
     {"poscollapse1",	   test_poscollapse1},
-    {"poscollapse2",	   test_poscollapse2},
     {"repeatquery1",	   test_repeatquery1},
     {"fetchdocs1",	   test_fetchdocs1},
     {"absentterm1",	   test_absentterm1},
@@ -1636,5 +1864,10 @@ test_desc anydb_tests[] = {
     {"getdoc1",		   test_getdoc1},
     {"emptyop1",	   test_emptyop1},
     {"checkatleast1",	   test_checkatleast1},
+    {"checkatleast2",	   test_checkatleast2},
+    {"checkatleast3",	   test_checkatleast3},
+    {"allpostlist1",	   test_allpostlist1},
+    {"emptyterm1",	   test_emptyterm1},
+    {"valuerange1",	   test_valuerange1},
     {0, 0}
 };
