@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,19 +22,24 @@
  */
 
 #include <config.h>
+
 #include <algorithm>
 #include <fstream>
 #include <map>
 #include <string>
 #include <vector>
 
+// We have to use the deprecated Quartz::open() method.
+#define XAPIAN_DEPRECATED(D) D
 #include <xapian.h>
+
+#include "backendmanager.h"
 #include "testsuite.h"
 #include "testutils.h"
+#include "unixcmds.h"
 #include "utils.h"
 
 #include "apitest.h"
-#include "api_db.h"
 
 #include <list>
 
@@ -42,7 +48,7 @@ using namespace std;
 static Xapian::Query
 query(const string &t)
 {
-    return Xapian::Query(Xapian::Stem("english").stem_word(t));
+    return Xapian::Query(Xapian::Stem("english")(t));
 }
 
 // #######################################################################
@@ -70,7 +76,8 @@ static bool test_stubdb1()
     ofstream out("stubdb1");
     TEST(out.is_open());
     // FIXME: not very reliable...
-    out << "remote :../bin/xapian-progsrv .flint/db=apitest_simpledata\n";
+    out << "remote :" << BackendManager::get_xapian_progsrv_command()
+	<< " .flint/db=apitest_simpledata\n";
     out.close();
 
     {
@@ -85,14 +92,6 @@ static bool test_stubdb1()
 	enquire.set_query(Xapian::Query("word"));
 	enquire.get_mset(0, 10);
     }
-#if 0
-    { // Now deprecated in favour of the previous form.
-	Xapian::Database db = Xapian::Auto::open("stubdb1");
-	Xapian::Enquire enquire(db);
-	enquire.set_query(Xapian::Query("word"));
-	enquire.get_mset(0, 10);
-    }
-#endif
 
     unlink("stubdb1");
 
@@ -106,9 +105,8 @@ class MyErrorHandler : public Xapian::ErrorHandler {
 
 	bool handle_error(Xapian::Error & error) {
 	    ++count;
-	    tout << "Error handling caught: " << error.get_type() << ": " <<
-		    error.get_msg() << ", with context `" <<
-		    error.get_context() << "': count is now " << count << "\n";
+	    tout << "Error handling caught: " << error.get_description()
+		 << ", count is now " << count << "\n";
 	    return true;
 	}
 
@@ -244,29 +242,75 @@ static bool test_multierrhandler1()
 
 class myMatchDecider : public Xapian::MatchDecider {
     public:
-	int operator()(const Xapian::Document &doc) const {
+	bool operator()(const Xapian::Document &doc) const {
 	    // Note that this is not recommended usage of get_data()
 	    return doc.get_data().find("This is") != string::npos;
 	}
 };
 
-// tests the match decision functor
+// Test Xapian::MatchDecider functor.
 static bool test_matchfunctor1()
 {
-    // FIXME: check that the functor works both ways.
-    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enquire(db);
     enquire.set_query(Xapian::Query("this"));
 
     myMatchDecider myfunctor;
 
     Xapian::MSet mymset = enquire.get_mset(0, 100, 0, &myfunctor);
 
+    vector<bool> docid_checked(db.get_lastdocid());
+
+    // Check that we get the expected number of matches, and that they
+    // satisfy the condition.
     Xapian::MSetIterator i = mymset.begin();
     TEST(i != mymset.end());
     TEST_EQUAL(mymset.size(), 3);
     for ( ; i != mymset.end(); ++i) {
 	const Xapian::Document doc(i.get_document());
 	TEST(myfunctor(doc));
+	docid_checked[*i] = true;
+    }
+
+    // Check that the other documents don't satisfy the condition.
+    for (Xapian::docid did = 1; did < docid_checked.size(); ++did) {
+	if (!docid_checked[did]) {
+	    TEST(!myfunctor(db.get_document(did)));
+	}
+    }
+
+    return true;
+}
+
+// Test Xapian::MatchDecider functor used as a match spy.
+static bool test_matchfunctor2()
+{
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enquire(db);
+    enquire.set_query(Xapian::Query("this"));
+
+    myMatchDecider myfunctor;
+
+    Xapian::MSet mymset = enquire.get_mset(0, 100, 0, NULL, &myfunctor);
+
+    vector<bool> docid_checked(db.get_lastdocid());
+
+    // Check that we get the expected number of matches, and that they
+    // satisfy the condition.
+    Xapian::MSetIterator i = mymset.begin();
+    TEST(i != mymset.end());
+    TEST_EQUAL(mymset.size(), 3);
+    for ( ; i != mymset.end(); ++i) {
+	const Xapian::Document doc(i.get_document());
+	TEST(myfunctor(doc));
+	docid_checked[*i] = true;
+    }
+
+    // Check that the other documents don't satisfy the condition.
+    for (Xapian::docid did = 1; did < docid_checked.size(); ++did) {
+	if (!docid_checked[did]) {
+	    TEST(!myfunctor(db.get_document(did)));
+	}
     }
 
     return true;
@@ -462,33 +506,6 @@ static bool test_collapsekey1()
     return true;
 }
 
-// tests the collapse-on-key for DA databases
-static bool test_collapsekey2()
-{
-    Xapian::Enquire enquire(get_database("apitest_simpledata"));
-    enquire.set_query(Xapian::Query("this"));
-
-    Xapian::MSet mymset1 = enquire.get_mset(0, 100);
-    Xapian::doccount mymsize1 = mymset1.size();
-
-    const Xapian::valueno value_no = 0;
-    enquire.set_collapse_key(value_no);
-    Xapian::MSet mymset = enquire.get_mset(0, 100);
-
-    TEST_AND_EXPLAIN(mymsize1 > mymset.size(),
-		     "Had no fewer items when performing collapse: don't know whether it worked.");
-
-    map<string, Xapian::docid> values;
-    Xapian::MSetIterator i = mymset.begin();
-    for ( ; i != mymset.end(); ++i) {
-	string value = i.get_document().get_value(value_no);
-	TEST(values[value] == 0 || value == "");
-	values[value] = *i;
-    }
-
-    return true;
-}
-
 // tests that collapse-on-key modifies the predicted bounds for the number of
 // matches appropriately.
 static bool test_collapsekey3()
@@ -731,6 +748,53 @@ static bool test_allterms5()
     return true;
 }
 
+// test allterms iterators with prefixes
+static bool test_allterms6()
+{
+    Xapian::Database db;
+    db.add_database(get_database("apitest_allterms"));
+    db.add_database(get_database("apitest_allterms2"));
+
+    Xapian::TermIterator ati = db.allterms_begin("three");
+    TEST(ati != db.allterms_end("three"));
+    TEST_EQUAL(*ati, "three");
+    ati.skip_to("three");
+    TEST(ati != db.allterms_end("three"));
+    TEST_EQUAL(*ati, "three");
+    ati++;
+    TEST(ati == db.allterms_end("three"));
+
+    ati = db.allterms_begin("thre");
+    TEST(ati != db.allterms_end("thre"));
+    TEST_EQUAL(*ati, "three");
+    ati.skip_to("three");
+    TEST(ati != db.allterms_end("thre"));
+    TEST_EQUAL(*ati, "three");
+    ati++;
+    TEST(ati == db.allterms_end("thre"));
+
+    ati = db.allterms_begin("f");
+    TEST(ati != db.allterms_end("f"));
+    TEST_EQUAL(*ati, "five");
+    TEST(ati != db.allterms_end("f"));
+    ati.skip_to("three");
+    TEST(ati == db.allterms_end("f"));
+
+    ati = db.allterms_begin("f");
+    TEST(ati != db.allterms_end("f"));
+    TEST_EQUAL(*ati, "five");
+    ati++;
+    TEST(ati != db.allterms_end("f"));
+    TEST_EQUAL(*ati, "four");
+    ati++;
+    TEST(ati == db.allterms_end("f"));
+
+    ati = db.allterms_begin("absent");
+    TEST(ati == db.allterms_end("absent"));
+
+    return true;
+}
+
 // test that searching for a term with a special characters in it works
 static bool test_specialterms1()
 {
@@ -740,7 +804,7 @@ static bool test_specialterms1()
     Xapian::MSetIterator m;
     Xapian::Stem stemmer("english");
 
-    enquire.set_query(stemmer.stem_word("new\nline"));
+    enquire.set_query(stemmer("new\nline"));
     mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 1);
     count = 0;
@@ -759,7 +823,7 @@ static bool test_specialterms1()
 	}
     }
 
-    enquire.set_query(stemmer.stem_word(string("big\0zero", 8)));
+    enquire.set_query(stemmer(string("big\0zero", 8)));
     mymset = enquire.get_mset(0, 10);
     TEST_MSET_SIZE(mymset, 1);
     count = 0;
@@ -863,7 +927,7 @@ static bool test_multiexpand1()
     Xapian::ESet eset2 = enquire2.get_eset(1000, rset2);
 
     // This is the multi database without approximation
-    Xapian::ESet eset3 = enquire2.get_eset(1000, rset2, Xapian::Enquire::use_exact_termfreq);
+    Xapian::ESet eset3 = enquire2.get_eset(1000, rset2, Xapian::Enquire::USE_EXACT_TERMFREQ);
 
     TEST_EQUAL(eset1.size(), eset2.size());
     TEST_EQUAL(eset1.size(), eset3.size());
@@ -892,8 +956,6 @@ static bool test_postlist1()
 {
     Xapian::Database db(get_database("apitest_simpledata"));
 
-    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.postlist_begin(""));
-
     TEST_EQUAL(db.postlist_begin("rosebud"), db.postlist_end("rosebud"));
 
     string s = "let_us_see_if_we_can_break_it_with_a_really_really_long_term.";
@@ -1086,6 +1148,7 @@ static bool test_consistency1()
     enquire.set_query(Xapian::Query(Xapian::Query::OP_OR, Xapian::Query("the"), Xapian::Query("sky")));
     Xapian::doccount lots = 214;
     Xapian::MSet bigmset = enquire.get_mset(0, lots);
+    TEST_EQUAL(bigmset.size(), lots);
     try {
 	for (Xapian::doccount start = 0; start < lots; ++start) {
 	    for (Xapian::doccount size = 0; size < lots - start; ++size) {
@@ -1115,28 +1178,271 @@ static bool test_consistency1()
 // tests that specifying a nonexistent input file throws an exception.
 static bool test_quartzdatabaseopeningerror1()
 {
+    mkdir(".quartz", 0755);
+
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 		   Xapian::Quartz::open(".quartz/nosuchdirectory"));
-    mkdir(".quartz/emptydirectory", 0755);
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+		   Xapian::Quartz::open(".quartz/nosuchdirectory", Xapian::DB_OPEN));
+
+    mkdir(".quartz/emptydirectory", 0700);
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 		   Xapian::Quartz::open(".quartz/emptydirectory"));
+
     touch(".quartz/somefile");
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Quartz::open(".quartz/somefile"));
+	Xapian::Quartz::open(".quartz/somefile"));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_OPEN));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_CREATE));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_CREATE_OR_OPEN));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_CREATE_OR_OVERWRITE));
+
+    return true;
+}
+
+/// Test opening of a quartz database
+static bool test_quartzdatabaseopen1()
+{
+    const char * dbdir = ".quartz/test_quartzdatabaseopen1";
+    mkdir(".quartz", 0755);
+
+    {
+	rm_rf(dbdir);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Quartz::open(dbdir, Xapian::DB_OPEN));
+	Xapian::Quartz::open(dbdir);
+    }
+
+    {
+	rm_rf(dbdir);
+	mkdir(dbdir, 0700);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Quartz::open(dbdir, Xapian::DB_OPEN));
+	Xapian::Quartz::open(dbdir);
+    }
+
+    {
+	rm_rf(dbdir);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE));
+	Xapian::Quartz::open(dbdir);
+    }
+
+    {
+	rm_rf(dbdir);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OPEN));
+	Xapian::Quartz::open(dbdir);
+    }
+
+    {
+	TEST_EXCEPTION(Xapian::DatabaseCreateError,
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE));
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+	Xapian::Quartz::open(dbdir);
+    }
+
+    {
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
+	Xapian::Quartz::open(dbdir);
+    }
+
+    {
+	Xapian::WritableDatabase wdb =
+	    Xapian::Quartz::open(dbdir, Xapian::DB_OPEN);
+	Xapian::Quartz::open(dbdir);
+    }
+
     return true;
 }
 
 // tests that specifying a nonexistent input file throws an exception.
 static bool test_flintdatabaseopeningerror1()
 {
+    mkdir(".flint", 0755);
+
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 		   Xapian::Flint::open(".flint/nosuchdirectory"));
-    mkdir(".flint/emptydirectory", 0755);
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+		   Xapian::Flint::open(".flint/nosuchdirectory", Xapian::DB_OPEN));
+
+    mkdir(".flint/emptydirectory", 0700);
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 		   Xapian::Flint::open(".flint/emptydirectory"));
+
     touch(".flint/somefile");
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 		   Xapian::Flint::open(".flint/somefile"));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+		   Xapian::Flint::open(".flint/somefile", Xapian::DB_OPEN));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE_OR_OPEN));
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE_OR_OVERWRITE));
+
+    return true;
+}
+
+/// Tests that appropriate error is thrown for database format change.
+static bool test_flintdatabaseformaterror1()
+{
+    string dbdir = test_driver::get_srcdir();
+    dbdir += "/testdata/flint-0.9.9";
+
+    // We should get a version error when we try and open the database for
+    // reading now.
+    TEST_EXCEPTION(Xapian::DatabaseVersionError,
+		   Xapian::Database db(dbdir));
+
+    // Also test when explicitly opening as a flint database.
+    TEST_EXCEPTION(Xapian::DatabaseVersionError,
+		   (void)Xapian::Flint::open(dbdir));
+
+    return true;
+}
+
+/// Test that an old database can be successfully overwritten when using
+// Xapian::DB_CREATE_OR_OVERWRITE.
+static bool test_flintdatabaseformaterror2()
+{
+    string flint099 = test_driver::get_srcdir();
+    flint099 += "/testdata/flint-0.9.9";
+
+    mkdir(".flint", 0755);
+    string dbdir = ".flint/test_flintdatabaseformaterror2";
+
+    rm_rf(dbdir);
+    cp_R(flint099, dbdir);
+
+    (void)Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+
+    rm_rf(dbdir);
+    cp_R(flint099, dbdir);
+
+    // Also test when explicitly opening as a flint database.
+    (void)Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+
+    return true;
+}
+
+// regression test for not releasing lock on error.
+static bool test_flintdatabaseformaterror3()
+{
+    string flint099 = test_driver::get_srcdir();
+    flint099 += "/testdata/flint-0.9.9";
+
+    mkdir(".flint", 0755);
+    string dbdir = ".flint/test_flintdatabaseformaterror3";
+
+    rm_rf(dbdir);
+    cp_R(flint099, dbdir);
+
+    TEST_EXCEPTION(Xapian::DatabaseVersionError,
+		   Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OPEN));
+
+    // This used to throw a DatabaseLockError: "Unable to acquire database
+    // write lock on .flint/formatdb: already locked"
+    Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+
+    return true;
+}
+
+// Test that 1.0.2 and later can open 1.0.1 databases.
+static bool test_flintbackwardcompat1()
+{
+    string flint101 = test_driver::get_srcdir();
+    flint101 += "/testdata/flint-1.0.1";
+
+    mkdir(".flint", 0755);
+    string dbdir = ".flint/test_flintbackwardcompat1";
+
+    rm_rf(dbdir);
+    cp_R(flint101, dbdir);
+
+    // Check we can open the older format for reading.
+    {
+	Xapian::Database db(dbdir);
+	TEST_EQUAL(db.get_doccount(), 0);
+    }
+
+    // Check we can open the older format for update.
+    {
+	Xapian::WritableDatabase db(dbdir, Xapian::DB_OPEN);
+	TEST_EQUAL(db.get_doccount(), 0);
+    }
+
+    return true;
+}
+
+/// Test opening of a flint database
+static bool test_flintdatabaseopen1()
+{
+    const string dbdir = ".flint/test_flintdatabaseopen1";
+    mkdir(".flint", 0755);
+
+    {
+	rm_rf(dbdir);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Flint::open(dbdir, Xapian::DB_OPEN));
+	Xapian::Flint::open(dbdir);
+    }
+
+    {
+	rm_rf(dbdir);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE));
+	Xapian::Flint::open(dbdir);
+    }
+
+    {
+	rm_rf(dbdir);
+	Xapian::WritableDatabase wdb =
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+	TEST_EXCEPTION(Xapian::DatabaseLockError,
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OPEN));
+	Xapian::Flint::open(dbdir);
+    }
+
+    {
+	TEST_EXCEPTION(Xapian::DatabaseCreateError,
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE));
+	Xapian::WritableDatabase wdb =
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+	Xapian::Flint::open(dbdir);
+    }
+
+    {
+	Xapian::WritableDatabase wdb =
+	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
+	Xapian::Flint::open(dbdir);
+    }
+
+    {
+	Xapian::WritableDatabase wdb =
+	    Xapian::Flint::open(dbdir, Xapian::DB_OPEN);
+	Xapian::Flint::open(dbdir);
+    }
+
     return true;
 }
 
@@ -1270,6 +1576,74 @@ static bool test_sortrel1()
     return true;
 }
 
+// Test network stats and local stats give the same results.
+static bool test_netstats1()
+{
+    BackendManager local_manager;
+#if defined XAPIAN_HAS_FLINT_BACKEND
+    local_manager.set_dbtype("flint");
+#elif defined XAPIAN_HAS_QUARTZ_BACKEND
+    local_manager.set_dbtype("quartz");
+#else
+    SKIP_TEST("No suitable local database backend enabled");
+#endif
+    local_manager.set_datadir(test_driver::get_srcdir() + "/testdata/");
+
+    const char * words[] = { "paragraph", "word" };
+    Xapian::Query query(Xapian::Query::OP_OR, words, words + 2);
+    const size_t MSET_SIZE = 10;
+
+    Xapian::RSet rset;
+    rset.add_document(4);
+    rset.add_document(9);
+
+    Xapian::MSet mset_alllocal;
+    {
+	Xapian::Database db;
+	db.add_database(local_manager.get_database("apitest_simpledata"));
+	db.add_database(local_manager.get_database("apitest_simpledata2"));
+
+	Xapian::Enquire enq(db);
+	enq.set_query(query);
+	mset_alllocal = enq.get_mset(0, MSET_SIZE, &rset);
+    }
+
+    {
+	Xapian::Database db;
+	db.add_database(local_manager.get_database("apitest_simpledata"));
+	db.add_database(get_database("apitest_simpledata2"));
+
+	Xapian::Enquire enq(db);
+	enq.set_query(query);
+	Xapian::MSet mset = enq.get_mset(0, MSET_SIZE, &rset);
+	TEST_EQUAL(mset, mset_alllocal);
+    }
+
+    {
+	Xapian::Database db;
+	db.add_database(get_database("apitest_simpledata"));
+	db.add_database(local_manager.get_database("apitest_simpledata2"));
+
+	Xapian::Enquire enq(db);
+	enq.set_query(query);
+	Xapian::MSet mset = enq.get_mset(0, MSET_SIZE, &rset);
+	TEST_EQUAL(mset, mset_alllocal);
+    }
+
+    {
+	Xapian::Database db;
+	db.add_database(get_database("apitest_simpledata"));
+	db.add_database(get_database("apitest_simpledata2"));
+
+	Xapian::Enquire enq(db);
+	enq.set_query(query);
+	Xapian::MSet mset = enq.get_mset(0, MSET_SIZE, &rset);
+	TEST_EQUAL(mset, mset_alllocal);
+    }
+
+    return true;
+}
+
 // Coordinate matching - scores 1 for each matching term
 class MyWeight : public Xapian::Weight {
     public:
@@ -1316,22 +1690,39 @@ static bool test_userweight1()
     return true;
 }
 
+// tests MatchAll queries
+// This is a regression test, which failed with assertion failures in
+// revision 9094.
+static bool test_matchall1()
+{
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enquire(db);
+    enquire.set_query(Xapian::Query::MatchAll);
+    Xapian::MSet mset = enquire.get_mset(0, 10);
+    TEST_EQUAL(mset.get_matches_lower_bound(), db.get_doccount());
+
+
+    enquire.set_query(Xapian::Query(Xapian::Query::OP_OR,
+				    Xapian::Query("nosuchterm"),
+				    Xapian::Query::MatchAll));
+    mset = enquire.get_mset(0, 10);
+    TEST_EQUAL(mset.get_matches_lower_bound(), db.get_doccount());
+
+    return true;
+}
+
 // #######################################################################
 // # End of test cases: now we list the tests to run.
 
 /// The tests which require a database which supports values > 0 sensibly
 test_desc multivalue_tests[] = {
     {"collapsekey1",	   test_collapsekey1},
+    // There no longer is a collapsekey2 test!
     {"collapsekey3",	   test_collapsekey3},
     {"collapsekey4",	   test_collapsekey4},
     {0, 0}
 };
 
-test_desc mus36_tests[] = {
-    {"collapsekey2",       test_collapsekey2},
-    {0, 0}
-};
-
 /// The tests which need a backend which supports iterating over all terms
 test_desc allterms_tests[] = {
     {"allterms1",	   test_allterms1},
@@ -1339,6 +1730,7 @@ test_desc allterms_tests[] = {
     {"allterms3",	   test_allterms3},
     {"allterms4",	   test_allterms4},
     {"allterms5",	   test_allterms5},
+    {"allterms6",	   test_allterms6},
     {"specialterms2",	   test_specialterms2},
     {0, 0}
 };
@@ -1364,6 +1756,7 @@ test_desc collfreq_tests[] = {
 
 test_desc localdb_tests[] = {
     {"matchfunctor1",	   test_matchfunctor1},
+    {"matchfunctor2",	   test_matchfunctor2},
     {"msetiterator1",	   test_msetiterator1},
     {"msetiterator2",	   test_msetiterator2},
     {"msetiterator3",	   test_msetiterator3},
@@ -1385,27 +1778,48 @@ test_desc localdb_tests[] = {
     // Would work with remote if we registered the weighting scheme.
     // FIXME: do this so we also test that functionality...
     {"userweight1",	   test_userweight1},
+    {"matchall1",	   test_matchall1},
     {0, 0}
 };
 
 test_desc remotedb_tests[] = {
 // FIXME:    {"multierrhandler1",   test_multierrhandler1},
+    {"msetiterator1",	   test_msetiterator1},
+    {"msetiterator2",	   test_msetiterator2},
+    {"msetiterator3",	   test_msetiterator3},
+    {"esetiterator1",	   test_esetiterator1},
+    {"esetiterator2",	   test_esetiterator2},
+    {"multiexpand1",       test_multiexpand1},
+    {"postlist1",	   test_postlist1},
+    {"postlist2",	   test_postlist2},
+    {"postlist3",	   test_postlist3},
+    {"postlist4",	   test_postlist4},
+    {"postlist5",	   test_postlist5},
+    {"postlist6",	   test_postlist6},
     {"stubdb1",		   test_stubdb1},
     {"keepalive1",	   test_keepalive1},
     {"termstats",	   test_termstats},
     {"sortvalue1",	   test_sortvalue1},
     {"sortrel1",	   test_sortrel1},
+    {"netstats1",	   test_netstats1},
+    {"matchall1",	   test_matchall1},
     {0, 0}
 };
 
 test_desc flint_tests[] = {
     {"flintdatabaseopeningerror1",	test_flintdatabaseopeningerror1},
+    {"flintdatabaseformaterror1",	test_flintdatabaseformaterror1},
+    {"flintdatabaseformaterror2",	test_flintdatabaseformaterror2},
+    {"flintdatabaseformaterror3",	test_flintdatabaseformaterror3},
+    {"flintbackwardcompat1",		test_flintbackwardcompat1},
+    {"flintdatabaseopen1",		test_flintdatabaseopen1},
     {"sortrel1",	   test_sortrel1},
     {0, 0}
 };
 
 test_desc quartz_tests[] = {
     {"quartzdatabaseopeningerror1",	test_quartzdatabaseopeningerror1},
+    {"quartzdatabaseopen1",		test_quartzdatabaseopen1},
     {"sortrel1",	   test_sortrel1},
     {0, 0}
 };
