@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2006 Olly Betts
+ * Copyright 2003,2004,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,9 @@
  */
 
 #include <config.h>
+
 #include <stdlib.h>
+#include <iostream>
 #include <string>
 #include <vector>
 
@@ -35,15 +37,13 @@ using namespace std;
 
 #include "apitest.h"
 
-#include "api_nodb.h"
-#include "api_posdb.h"
-#include "api_db.h"
-#include "api_wrdb.h"
-#include "api_anydb.h"
-#include "api_transdb.h"
-
 static BackendManager backendmanager;
 
+const std::string & get_dbtype()
+{
+    return backendmanager.get_dbtype();
+}
+
 Xapian::Database
 get_database(const string &dbname)
 {
@@ -95,7 +95,8 @@ int main(int argc, char **argv)
 
     backendmanager.set_datadir(srcdir + "/testdata/");
 
-    RUNTESTS("void", nodb);
+    RUNTESTS("none", nodb);
+    RUNTESTS("none", unicode);
 
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
     RUNTESTS("inmemory", anydb);
@@ -103,7 +104,6 @@ int main(int argc, char **argv)
     RUNTESTS("inmemory", writabledb);
     RUNTESTS("inmemory", localdb);
     RUNTESTS("inmemory", positionaldb);
-    RUNTESTS("inmemory", localpositionaldb);
     RUNTESTS("inmemory", doclendb);
     RUNTESTS("inmemory", collfreq);
     RUNTESTS("inmemory", allterms);
@@ -116,7 +116,6 @@ int main(int argc, char **argv)
     RUNTESTS("flint", writabledb);
     RUNTESTS("flint", localdb);
     RUNTESTS("flint", positionaldb);
-    RUNTESTS("flint", localpositionaldb);
     RUNTESTS("flint", doclendb);
     RUNTESTS("flint", collfreq);
     RUNTESTS("flint", allterms);
@@ -131,7 +130,6 @@ int main(int argc, char **argv)
     RUNTESTS("quartz", writabledb);
     RUNTESTS("quartz", localdb);
     RUNTESTS("quartz", positionaldb);
-    RUNTESTS("quartz", localpositionaldb);
     RUNTESTS("quartz", doclendb);
     RUNTESTS("quartz", collfreq);
     RUNTESTS("quartz", allterms);
@@ -141,14 +139,16 @@ int main(int argc, char **argv)
 #endif
 
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
-    RUNTESTS("remote", anydb);
-    RUNTESTS("remote", specchar);
-    RUNTESTS("remote", writabledb);
-    RUNTESTS("remote", remotedb);
-    RUNTESTS("remote", positionaldb);
-    RUNTESTS("remote", doclendb);
-    RUNTESTS("remote", multivalue);
-    RUNTESTS("remote", transactiondb);
+    RUNTESTS("remoteprog", anydb);
+    RUNTESTS("remoteprog", specchar);
+    RUNTESTS("remoteprog", writabledb);
+    RUNTESTS("remoteprog", remotedb);
+    RUNTESTS("remoteprog", positionaldb);
+    RUNTESTS("remoteprog", doclendb);
+    RUNTESTS("remoteprog", collfreq);
+    RUNTESTS("remoteprog", allterms);
+    RUNTESTS("remoteprog", multivalue);
+    RUNTESTS("remoteprog", transactiondb);
 
     RUNTESTS("remotetcp", anydb);
     RUNTESTS("remotetcp", specchar);
@@ -156,35 +156,11 @@ int main(int argc, char **argv)
     RUNTESTS("remotetcp", remotedb);
     RUNTESTS("remotetcp", positionaldb);
     RUNTESTS("remotetcp", doclendb);
+    RUNTESTS("remotetcp", collfreq);
+    RUNTESTS("remotetcp", allterms);
     RUNTESTS("remotetcp", multivalue);
     RUNTESTS("remotetcp", transactiondb);
 #endif
 
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-    // need makeDA, etc tools to build da and db databases
-    if (file_exists("../../makeda/makeDA")) {
-	RUNTESTS("da", anydb);
-	RUNTESTS("da", localdb);
-	RUNTESTS("da", allterms);
-	RUNTESTS("da", mus36);
-    }
-    if (file_exists("../../makeda/makeDAflimsy")) {
-	RUNTESTS("daflimsy", anydb);
-	RUNTESTS("daflimsy", localdb);
-	RUNTESTS("daflimsy", allterms);
-	RUNTESTS("daflimsy", mus36);
-    }
-    if (file_exists("../../makeda/makeDB")) {
-	RUNTESTS("db", anydb);
-	RUNTESTS("db", localdb);
-	RUNTESTS("db", mus36);
-    }
-    if (file_exists("../../makeda/makeDBflimsy")) {
-	RUNTESTS("dbflimsy", anydb);
-	RUNTESTS("dbflimsy", localdb);
-	RUNTESTS("dbflimsy", mus36);
-    }
-#endif
-
     return result;
 }
