@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,10 @@
 
 #include <config.h>
 
+// We have to use the deprecated Quartz::open() method.
+#define XAPIAN_DEPRECATED(D) D
+#include <xapian.h>
+
 #ifdef HAVE_VALGRIND
 # include <valgrind/memcheck.h>
 #endif
@@ -34,11 +38,13 @@
 
 #include <stdio.h>
 
-#include <sys/stat.h>
+#include <sys/types.h>
+#include "safesysstat.h"
 
 #ifdef HAVE_FORK
 # include <signal.h>
 # include <sys/types.h>
+# include <sys/socket.h>
 # include <sys/wait.h>
 # include <unistd.h>
 // Some older systems had SIGCLD rather than SIGCHLD.
@@ -47,98 +53,80 @@
 # endif
 #endif
 
-#include <xapian.h>
+#ifdef __WIN32__
+# include "safefcntl.h"
+# include "safewindows.h"
+#endif
+
+// Paths to xapian-tcpsrv and xapian-progsrv.
+#ifdef __WIN32__
+// Under __WIN32__ we want \ path separators since we pass this path to
+// CreateProcess().
+# ifdef _MSC_VER
+#  ifdef DEBUG
+#   define XAPIAN_BIN_PATH "..\\win32\\Debug\\"
+#  else
+#   define XAPIAN_BIN_PATH "..\\win32\\Release\\"
+#  endif
+# else
+#  define XAPIAN_BIN_PATH "..\\bin\\" // mingw
+# endif
+#else
+# define XAPIAN_BIN_PATH "../bin/"
+#endif
+#define XAPIAN_TCPSRV XAPIAN_BIN_PATH"xapian-tcpsrv"
+#define XAPIAN_PROGSRV XAPIAN_BIN_PATH"xapian-progsrv"
+
 #include "index_utils.h"
 #include "backendmanager.h"
 #include "omdebug.h"
+#include "unixcmds.h"
 #include "utils.h"
 
+#include "noreturn.h"
+
+// We've had problems on some hosts which run tinderbox tests with "localhost"
+// not being set in /etc/hosts - using the IP address equivalent seems more
+// reliable.
+#define LOCALHOST "127.0.0.1"
+
+// Start at DEFAULT port and try higher ports until one isn't already in use.
+#define DEFAULT_PORT 1239
+
 using namespace std;
 
-static void
-index_files_to_database(Xapian::WritableDatabase & database,
-                        const vector<string> & paths)
+void
+BackendManager::index_files_to_database(Xapian::WritableDatabase & database,
+					const vector<string> & dbnames)
 {
     vector<string>::const_iterator p;
-    for (p = paths.begin(); p != paths.end(); ++p) {
-	const string & filename = *p;
+    for (p = dbnames.begin(); p != dbnames.end(); ++p) {
+	if (p->empty()) continue;
+	string filename;
+	if (datadir.empty()) {
+	    filename = *p;
+	} else {
+	    filename = datadir;
+	    filename += '/';
+	    filename += *p;
+	    filename += ".txt";
+	}
+
 	ifstream from(filename.c_str());
 	if (!from)
 	    throw Xapian::DatabaseOpeningError("Cannot open file " + filename +
 		    " for indexing");
 
-	while (from) {
+	while (!from.eof()) {
 	    database.add_document(document_from_stream(from));
 	}
     }
 }
 
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-static void
-index_files_to_m36(const string &prog, const string &dbdir,
-		   const vector<string> & paths)
-{
-    string dump = dbdir + "/DATA";
-    ofstream out(dump.c_str());
-    string valuefile = dbdir + "/keyfile";
-    ofstream values(valuefile.c_str());
-    values << "omrocks!"; // magic word
-    vector<string>::const_iterator p;
-    for (p = paths.begin(); p != paths.end(); ++p) {
-	const string & filename = *p;
-	ifstream from(filename.c_str());
-	if (!from)
-	    throw Xapian::DatabaseOpeningError("Cannot open file " + filename +
-		    " for indexing");
-
-	while (from) {
-	    Xapian::Document doc = document_from_stream(from);
-	    out << "#RSTART#\n" << doc.get_data() << "\n#REND#\n#TSTART#\n";
-	    {
-		Xapian::TermIterator i = doc.termlist_begin();
-		Xapian::TermIterator i_end = doc.termlist_end();
-		for ( ; i != i_end; ++i) {
-		    out << *i << endl;
-		}
-	    }
-	    out << "#TEND#\n";
-	    Xapian::ValueIterator value_i = doc.values_begin();
-	    string value = string("\0\0\0\0\0\0\0", 8);
-	    if (value_i != doc.values_end()) value = (*value_i) + value;
-	    value = value.substr(0, 8);
-	    values << value;
-	}
-    }
-    out.close();
-    string cmd = "../../makeda/" + prog + " -source " + dump +
-	" -da " + dbdir + "/ -work " + dbdir + "/tmp- > /dev/null";
-    system(cmd);
-    unlink(dump);
-}
-#endif
-
-#if defined HAVE_FORK && defined HAVE_WAITPID
-extern "C" void
-on_SIGCHLD(int /*sig*/)
-{
-    int status;
-    while (waitpid(-1, &status, WNOHANG) > 0);
-}
-#endif
-
 BackendManager::BackendManager() :
-    do_getdb(&BackendManager::getdb_void),
-    do_getwritedb(&BackendManager::getwritedb_void)
+    do_getdb(&BackendManager::getdb_none),
+    do_getwritedb(&BackendManager::getwritedb_none)
 {
-#ifdef HAVE_FORK
-    // Clean up child processes forked to close the pipes we open to run
-    // xapian-tcpsrv.
-#ifdef HAVE_WAITPID
-    signal(SIGCHLD, on_SIGCHLD);
-#else
-    signal(SIGCHLD, SIG_IGN);
-#endif
-#endif
 }
 
 void
@@ -151,8 +139,8 @@ BackendManager::set_dbtype(const string &type)
 	do_getdb = &BackendManager::getdb_inmemory;
 	do_getwritedb = &BackendManager::getwritedb_inmemory;
 #else
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
 #endif
 #if 0
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
@@ -168,119 +156,66 @@ BackendManager::set_dbtype(const string &type)
 #else
     } else if (type == "inmemoryerr" || type == "inmemoryerr2" ||
 	       type == "inmemoryerr3") {
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
 #endif
 #endif
     } else if (type == "flint") {
 #ifdef XAPIAN_HAS_FLINT_BACKEND
 	do_getdb = &BackendManager::getdb_flint;
 	do_getwritedb = &BackendManager::getwritedb_flint;
-	rmdir(".flint");
+	rm_rf(".flint");
 #else
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
 #endif
     } else if (type == "quartz") {
 #ifdef XAPIAN_HAS_QUARTZ_BACKEND
 	do_getdb = &BackendManager::getdb_quartz;
 	do_getwritedb = &BackendManager::getwritedb_quartz;
-	rmdir(".quartz");
+	rm_rf(".quartz");
 #else
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
 #endif
-    } else if (type == "remote") {
+    } else if (type == "remoteprog") {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
-	do_getdb = &BackendManager::getdb_remote;
-	do_getwritedb = &BackendManager::getwritedb_remote;
+	do_getdb = &BackendManager::getdb_remoteprog;
+	do_getwritedb = &BackendManager::getwritedb_remoteprog;
 #else
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
 #endif
     } else if (type == "remotetcp") {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 	do_getdb = &BackendManager::getdb_remotetcp;
 	do_getwritedb = &BackendManager::getwritedb_remotetcp;
 #else
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
 #endif
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-    } else if (type == "da") {
-	do_getdb = &BackendManager::getdb_da;
-	do_getwritedb = &BackendManager::getwritedb_da;
-	rmdir(".da");
-    } else if (type == "db") {
-	do_getdb = &BackendManager::getdb_db;
-	do_getwritedb = &BackendManager::getwritedb_db;
-	rmdir(".db");
-    } else if (type == "daflimsy") {
-	do_getdb = &BackendManager::getdb_daflimsy;
-	do_getwritedb = &BackendManager::getwritedb_daflimsy;
-	rmdir(".daflimsy");
-    } else if (type == "dbflimsy") {
-	do_getdb = &BackendManager::getdb_dbflimsy;
-	do_getwritedb = &BackendManager::getwritedb_dbflimsy;
-	rmdir(".dbflimsy");
-#else
-    } else if (type == "da" || type == "db" || type == "daflimsy" ||
-	       type == "dbflimsy") {
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
-#endif
-    } else if (type == "void") {
-	do_getdb = &BackendManager::getdb_void;
-	do_getwritedb = &BackendManager::getwritedb_void;
+    } else if (type == "none") {
+	do_getdb = &BackendManager::getdb_none;
+	do_getwritedb = &BackendManager::getwritedb_none;
     } else {
 	throw Xapian::InvalidArgumentError(
-		"Expected inmemory, flint, quartz, remote, remotetcp, da, db, "
-		"daflimsy, dbflimsy, or void");
+	    "Expected inmemory, flint, quartz, remoteprog, remotetcp, or none");
     }
     current_type = type;
 }
 
-void
-BackendManager::set_datadir(const string &datadir_)
-{
-    datadir = datadir_;
-}
-
-string
-BackendManager::get_datadir()
-{
-    return datadir;
-}
-
 Xapian::Database
-BackendManager::getdb_void(const vector<string> &)
+BackendManager::getdb_none(const vector<string> &)
 {
     throw Xapian::InvalidArgumentError("Attempted to open a disabled database");
 }
 
 Xapian::WritableDatabase
-BackendManager::getwritedb_void(const vector<string> &)
+BackendManager::getwritedb_none(const vector<string> &)
 {
     throw Xapian::InvalidArgumentError("Attempted to open a disabled database");
 }
 
-vector<string>
-BackendManager::change_names_to_paths(const vector<string> &dbnames)
-{
-    vector<string> paths;
-    vector<string>::const_iterator i;
-    for (i = dbnames.begin(); i != dbnames.end(); ++i) {
-	if (!i->empty()) {
-	    if (datadir.empty()) {
-		paths.push_back(*i);
-	    } else {
-		paths.push_back(datadir + "/" + *i + ".txt");
-	    }
-	}
-    }
-    return paths;
-}
-
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
 Xapian::Database
 BackendManager::getdb_inmemory(const vector<string> &dbnames)
@@ -292,7 +227,7 @@ Xapian::WritableDatabase
 BackendManager::getwritedb_inmemory(const vector<string> &dbnames)
 {
     Xapian::WritableDatabase db(Xapian::InMemory::open());
-    index_files_to_database(db, change_names_to_paths(dbnames));
+    index_files_to_database(db, dbnames);
     return db;
 }
 
@@ -308,7 +243,7 @@ BackendManager::getwritedb_inmemoryerr(const vector<string> &dbnames)
 {
     // FIXME: params.set("inmemory_errornext", 1);
     Xapian::WritableDatabase db(Xapian::InMemory::open());
-    index_files_to_database(db, change_names_to_paths(dbnames));
+    index_files_to_database(db, dbnames);
 
     return db;
 }
@@ -324,7 +259,7 @@ BackendManager::getwritedb_inmemoryerr2(const vector<string> &dbnames)
 {
     // FIXME: params.set("inmemory_abortnext", 1);
     Xapian::WritableDatabase db(Xapian::InMemory::open());
-    index_files_to_database(db, change_names_to_paths(dbnames));
+    index_files_to_database(db, dbnames);
 
     return db;
 }
@@ -340,7 +275,7 @@ BackendManager::getwritedb_inmemoryerr3(const vector<string> &dbnames)
 {
     // params.set("inmemory_abortnext", 2);
     Xapian::WritableDatabase db(Xapian::InMemory::open());
-    index_files_to_database(db, change_names_to_paths(dbnames));
+    index_files_to_database(db, dbnames);
 
     return db;
 }
@@ -359,7 +294,7 @@ bool create_dir_if_needed(const string &dirname)
     if (result < 0) {
 	if (errno != ENOENT)
 	    throw Xapian::DatabaseOpeningError("Can't stat directory");
-        if (mkdir(dirname, 0700) < 0)
+	if (mkdir(dirname, 0700) < 0)
 	    throw Xapian::DatabaseOpeningError("Can't create directory");
 	return true; // Successfully created a directory.
     }
@@ -378,13 +313,14 @@ BackendManager::createdb_flint(const vector<string> &dbnames)
     string dbdir = parent_dir + "/db";
     for (vector<string>::const_iterator i = dbnames.begin();
 	 i != dbnames.end(); i++) {
-	dbdir += "=" + *i;
+	dbdir += '=';
+	dbdir += *i;
     }
     // If the database is readonly, we can reuse it if it exists.
     if (create_dir_if_needed(dbdir)) {
 	// Directory was created, so do the indexing.
 	Xapian::WritableDatabase db(Xapian::Flint::open(dbdir, Xapian::DB_CREATE, 2048));
-	index_files_to_database(db, change_names_to_paths(dbnames));
+	index_files_to_database(db, dbnames);
     }
     return dbdir;
 }
@@ -405,12 +341,11 @@ BackendManager::getwritedb_flint(const vector<string> &dbnames)
     // use) from readonly ones (which can be reused).
     string dbdir = parent_dir + "/dbw";
     // For a writable database we need to start afresh each time.
-    rmdir(dbdir);
+    rm_rf(dbdir);
     (void)create_dir_if_needed(dbdir);
-    touch(dbdir + "/log");
     // directory was created, so do the indexing.
     Xapian::WritableDatabase db(Xapian::Flint::open(dbdir, Xapian::DB_CREATE, 2048));
-    index_files_to_database(db, change_names_to_paths(dbnames));
+    index_files_to_database(db, dbnames);
     return db;
 }
 #endif
@@ -425,13 +360,14 @@ BackendManager::createdb_quartz(const vector<string> &dbnames)
     string dbdir = parent_dir + "/db";
     for (vector<string>::const_iterator i = dbnames.begin();
 	 i != dbnames.end(); i++) {
-	dbdir += "=" + *i;
+	dbdir += '=';
+	dbdir += *i;
     }
     // If the database is readonly, we can reuse it if it exists.
     if (create_dir_if_needed(dbdir)) {
 	// Directory was created, so do the indexing.
 	Xapian::WritableDatabase db(Xapian::Quartz::open(dbdir, Xapian::DB_CREATE, 2048));
-	index_files_to_database(db, change_names_to_paths(dbnames));
+	index_files_to_database(db, dbnames);
     }
     return dbdir;
 }
@@ -452,19 +388,19 @@ BackendManager::getwritedb_quartz(const vector<string> &dbnames)
     // use) from readonly ones (which can be reused).
     string dbdir = parent_dir + "/dbw";
     // For a writable database we need to start afresh each time.
-    rmdir(dbdir);
+    rm_rf(dbdir);
     (void)create_dir_if_needed(dbdir);
     touch(dbdir + "/log");
     // directory was created, so do the indexing.
     Xapian::WritableDatabase db(Xapian::Quartz::open(dbdir, Xapian::DB_CREATE, 2048));
-    index_files_to_database(db, change_names_to_paths(dbnames));
+    index_files_to_database(db, dbnames);
     return db;
 }
 #endif
 
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 Xapian::Database
-BackendManager::getdb_remote(const vector<string> &dbnames)
+BackendManager::getdb_remoteprog(const vector<string> &dbnames)
 {
     // Uses xapian-progsrv as the server.
 
@@ -491,19 +427,20 @@ BackendManager::getdb_remote(const vector<string> &dbnames)
 #endif
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
-        return Xapian::Remote::open("./runtest ../bin/xapian-progsrv", args);
+	args.insert(0, XAPIAN_PROGSRV" ");
+	return Xapian::Remote::open("./runsrv", args);
     }
 #endif
-    return Xapian::Remote::open("../bin/xapian-progsrv", args);
+    return Xapian::Remote::open(XAPIAN_PROGSRV, args);
 }
 
 Xapian::WritableDatabase
-BackendManager::getwritedb_remote(const vector<string> &dbnames)
+BackendManager::getwritedb_remoteprog(const vector<string> &dbnames)
 {
     // Uses xapian-progsrv as the server.
 
     // Default to a long (5 minute) timeout so that tests won't fail just
-    // because the host if slow or busy.
+    // because the host is slow or busy.
     string args = "-t300000 --writable ";
 
 #ifdef XAPIAN_HAS_FLINT_BACKEND
@@ -515,33 +452,107 @@ BackendManager::getwritedb_remote(const vector<string> &dbnames)
 #endif
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
-        return Xapian::Remote::open_writable("./runtest ../bin/xapian-progsrv", args);
+	args.insert(0, XAPIAN_PROGSRV" ");
+	return Xapian::Remote::open_writable("./runsrv", args);
     }
 #endif
-    return Xapian::Remote::open_writable("../bin/xapian-progsrv", args);
+    return Xapian::Remote::open_writable(XAPIAN_PROGSRV, args);
 }
 
 #ifdef HAVE_FORK
+
+// We can't dynamically allocate memory for this because it confuses the leak
+// detector.  We only have 1-3 child fds open at once anyway, so a fixed size
+// array isn't a problem, and linear scanning isn't a problem.
+struct pid_fd {
+    pid_t pid;
+    int fd;
+};
+
+static pid_fd pid_to_fd[16];
+
+extern "C" void
+on_SIGCHLD(int /*sig*/)
+{
+    int status;
+    pid_t child;
+    while ((child = waitpid(-1, &status, WNOHANG)) > 0) {
+	for (unsigned i = 0; i < sizeof(pid_to_fd) / sizeof(pid_fd); ++i) {
+	    if (pid_to_fd[i].pid == child) {
+		int fd = pid_to_fd[i].fd;
+		pid_to_fd[i].fd = -1;
+		pid_to_fd[i].pid = -1;
+		// NB close() *is* safe to use in a signal handler.
+		close(fd);
+		break;
+	    }
+	}
+    }
+}
+
 static int
 launch_xapian_tcpsrv(const string & args)
 {
-    int port = 1239;
+    int port = DEFAULT_PORT;
+
+    // We want to be able to get the exit status of the child process we fork
+    // in xapian-tcpsrv doesn't start listening successfully.
+    signal(SIGCHLD, SIG_DFL);
 try_next_port:
-    string cmd = "../bin/xapian-tcpsrv --one-shot --port " + om_tostring(port) + " " + args + " 2>/dev/null";
+    string cmd = XAPIAN_TCPSRV" --one-shot --interface "LOCALHOST" --port " + om_tostring(port) + " " + args;
 #ifdef HAVE_VALGRIND
-    if (RUNNING_ON_VALGRIND) cmd = "./runtest " + cmd;
+    if (RUNNING_ON_VALGRIND) cmd = "./runsrv " + cmd;
 #endif
-    FILE * fh = popen(cmd.c_str(), "r");
+    int fds[2];
+    if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, fds) < 0) {
+	string msg("Couldn't create socketpair: ");
+	msg += strerror(errno);
+	throw msg;
+    }
+
+    pid_t child = fork();
+    if (child == 0) {
+	// Child process.
+	close(fds[0]);
+	// Connect stdout and stderr to the socket.
+	dup2(fds[1], 1);
+	dup2(fds[1], 2);
+	execl("/bin/sh", "/bin/sh", "-c", cmd.c_str(), (void*)NULL);
+	_exit(-1);
+    }
+
+    close(fds[1]);
+    if (child == -1) {
+	// Couldn't fork.
+	int fork_errno = errno;
+	close(fds[0]);
+	string msg("Couldn't fork: ");
+	msg += strerror(fork_errno);
+	throw msg;
+    }
+
+    // Parent process.
+
+    // Wrap the file descriptor in a FILE * so we can read lines using fgets().
+    FILE * fh = fdopen(fds[0], "r");
     if (fh == NULL) {
 	string msg("Failed to run command '");
 	msg += cmd;
-	msg += "'";
+	msg += "': ";
+	msg += strerror(errno);
 	throw msg;
     }
+    string output;
     while (true) {
 	char buf[256];
 	if (fgets(buf, sizeof(buf), fh) == NULL) {
-	    int status = pclose(fh);
+	    fclose(fh);
+	    int status;
+	    if (waitpid(child, &status, 0) == -1) {
+		string msg("waitpid failed: ");
+		msg += strerror(errno);
+		throw msg;
+	    }
 	    if (++port < 65536 && status != 0) {
 		if (WIFEXITED(status) && WEXITSTATUS(status) == 69) {  
 		    // 69 is EX_UNAVAILABLE which xapian-tcpsrv exits
@@ -552,32 +563,144 @@ try_next_port:
 	    }
 	    string msg("Failed to get 'Listening...' from command '");
 	    msg += cmd;
-	    msg += "'";
+	    msg += "' (output: ";
+	    msg += output;
+	    msg += ")";
 	    throw msg;
 	}
 	if (strcmp(buf, "Listening...\n") == 0) break;
+	output += buf;
     }
-    pid_t child = fork();
-    if (child == 0) {
-	// Child process.
-	pclose(fh);
-	_exit(0);
-    }
-    if (child == -1) {
-	string msg("fork() failed: ");
-	msg += strerror(errno);
-	throw msg;
+
+    // dup() the fd we wrapped with fdopen() so we can keep it open so the
+    // xapian-tcpsrv keeps running.
+    int tracked_fd = dup(fds[0]);
+
+    // We must fclose() the FILE* to avoid valgrind detecting memory leaks from
+    // its buffers.
+    fclose(fh);
+
+    // Find a slot to track the pid->fd mapping in.  If we can't find a slot
+    // it just means we'll leak the fd, so don't worry about that too much.
+    for (unsigned i = 0; i < sizeof(pid_to_fd) / sizeof(pid_fd); ++i) {
+	if (pid_to_fd[i].pid == -1) {
+	    pid_to_fd[i].fd = tracked_fd;
+	    pid_to_fd[i].pid = child;
+	    break;
+	}
     }
+
+    // Set a signal handler to clean up the xapian-tcpsrv child process when it
+    // finally exits.
+    signal(SIGCHLD, on_SIGCHLD);
+
     return port;
 }
+
+#elif defined __WIN32__
+
+XAPIAN_NORETURN(static void win32_throw_error_string(const char * str))
+{
+    string msg(str);
+    char * error = 0;
+    DWORD len;
+    len = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER,
+			0, GetLastError(), 0, (CHAR*)&error, 0, 0);
+    if (error) {
+	// Remove any trailing \r\n from output of FormatMessage.
+	if (len >= 2 && error[len - 2] == '\r' && error[len - 1] == '\n')
+	    len -= 2;
+	if (len) {
+	    msg += ": ";
+	    msg.append(error, len);
+	}
+	LocalFree(error);
+    }
+    throw msg;
+}
+
+// This implementation uses the WIN32 API to start xapian-tcpsrv as a child
+// process and read its output using a pipe.
+static int
+launch_xapian_tcpsrv(const string & args)
+{
+    int port = DEFAULT_PORT;
+
+try_next_port:
+    string cmd = XAPIAN_TCPSRV" --one-shot --interface "LOCALHOST" --port " + om_tostring(port) + " " + args;
+
+    // Create a pipe so we can read stdout/stderr from the child process.
+    HANDLE hRead, hWrite;
+    if (!CreatePipe(&hRead, &hWrite, 0, 0))
+	win32_throw_error_string("Couldn't create pipe");
+
+    // Set the write handle to be inherited by the child process.
+    SetHandleInformation(hWrite, HANDLE_FLAG_INHERIT, 1);
+
+    // Create the child process.
+    PROCESS_INFORMATION procinfo;
+    memset(&procinfo, 0, sizeof(PROCESS_INFORMATION));
+
+    STARTUPINFO startupinfo;
+    memset(&startupinfo, 0, sizeof(STARTUPINFO));
+    startupinfo.cb = sizeof(STARTUPINFO);
+    startupinfo.hStdError = hWrite;
+    startupinfo.hStdOutput = hWrite;
+    startupinfo.hStdInput = INVALID_HANDLE_VALUE;
+    startupinfo.dwFlags |= STARTF_USESTDHANDLES;
+
+    // For some reason Windows wants a modifiable copy!
+    BOOL ok;
+    char * cmdline = strdup(cmd.c_str());
+    ok = CreateProcess(0, cmdline, 0, 0, TRUE, 0, 0, 0, &startupinfo, &procinfo);
+    free(cmdline);
+    if (!ok)
+	win32_throw_error_string("Couldn't create child process");
+
+    CloseHandle(hWrite);
+    CloseHandle(procinfo.hThread);
+
+    string output;
+    FILE *fh = fdopen(_open_osfhandle((intptr_t)hRead, O_RDONLY), "r");
+    while (true) {
+	char buf[256];
+	if (fgets(buf, sizeof(buf), fh) == NULL) {
+	    fclose(fh);
+	    DWORD rc;
+	    // This doesn't seem to be necessary on the machine I tested on,
+	    // but I guess it could be on a slow machine...
+	    while (GetExitCodeProcess(procinfo.hProcess, &rc) && rc == STILL_ACTIVE) {
+		Sleep(100);
+	    }
+	    CloseHandle(procinfo.hProcess);
+	    if (++port < 65536 && rc == 69) {
+		// 69 is EX_UNAVAILABLE which xapian-tcpsrv exits
+		// with if (and only if) the port specified was
+		// in use.
+		goto try_next_port;
+	    }
+	    string msg("Failed to get 'Listening...' from command '");
+	    msg += cmd;
+	    msg += "' (output: ";
+	    msg += output;
+	    msg += ")";
+	    throw msg;
+	}
+	if (strcmp(buf, "Listening...\r\n") == 0) break;
+	output += buf;
+    }
+    fclose(fh);
+
+    return port;
+}
+
+#else
+# error Neither HAVE_FORK nor __WIN32__ is defined
 #endif
 
 Xapian::Database
 BackendManager::getdb_remotetcp(const vector<string> &dbnames)
 {
-#ifndef HAVE_FORK
-    throw string("Can't run remotetcp tests without fork()");
-#else
     // Uses xapian-tcpsrv as the server.
 
     vector<string> paths;
@@ -590,7 +713,7 @@ BackendManager::getdb_remotetcp(const vector<string> &dbnames)
 	paths.assign(dbnames.begin() + 2, dbnames.end());
     } else {
 	// Default to a long (5 minute) timeout so that tests won't fail just
-	// because the host if slow or busy.
+	// because the host is slow or busy.
 	args += "300000";
 	paths = dbnames;
     }
@@ -603,20 +726,16 @@ BackendManager::getdb_remotetcp(const vector<string> &dbnames)
 #endif
 
     int port = launch_xapian_tcpsrv(args);
-    return Xapian::Remote::open("127.0.0.1", port);
-#endif
+    return Xapian::Remote::open(LOCALHOST, port);
 }
 
 Xapian::WritableDatabase
 BackendManager::getwritedb_remotetcp(const vector<string> &dbnames)
 {
-#ifndef HAVE_FORK
-    throw string("Can't run remotetcp tests without fork()");
-#else
     // Uses xapian-tcpsrv as the server.
 
     // Default to a long (5 minute) timeout so that tests won't fail just
-    // because the host if slow or busy.
+    // because the host is slow or busy.
     string args = "-t300000 --writable ";
 
 #ifdef XAPIAN_HAS_FLINT_BACKEND
@@ -628,117 +747,7 @@ BackendManager::getwritedb_remotetcp(const vector<string> &dbnames)
 #endif
 
     int port = launch_xapian_tcpsrv(args);
-    return Xapian::Remote::open_writable("127.0.0.1", port);
-#endif
-}
-#endif
-
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-Xapian::Database
-BackendManager::getdb_da(const vector<string> &dbnames)
-{
-    string parent_dir = ".da";
-    create_dir_if_needed(parent_dir);
-
-    string dbdir = parent_dir + "/db";
-    for (vector<string>::const_iterator i = dbnames.begin();
-	 i != dbnames.end();
-	 i++) {
-	dbdir += "=" + *i;
-    }
-    if (create_dir_if_needed(dbdir)) {
-	// directory was created, so do the indexing.
-	// need to build temporary file and run it through makeda (yum)
-	index_files_to_m36("makeDA", dbdir, change_names_to_paths(dbnames));
-    }
-    return Xapian::Muscat36::open_da(dbdir + "/R", dbdir + "/T",
-				     dbdir + "/keyfile");
-}
-
-Xapian::Database
-BackendManager::getdb_daflimsy(const vector<string> &dbnames)
-{
-    string parent_dir = ".daflimsy";
-    create_dir_if_needed(parent_dir);
-
-    string dbdir = parent_dir + "/db";
-    for (vector<string>::const_iterator i = dbnames.begin();
-	 i != dbnames.end();
-	 i++) {
-	dbdir += "=" + *i;
-    }
-    if (create_dir_if_needed(dbdir)) {
-	// directory was created, so do the indexing.
-	// need to build temporary file and run it through makeda (yum)
-	index_files_to_m36("makeDAflimsy", dbdir, change_names_to_paths(dbnames));
-    }
-    return Xapian::Muscat36::open_da(dbdir + "/R", dbdir + "/T",
-				     dbdir + "/keyfile", false);
-}
-
-Xapian::WritableDatabase
-BackendManager::getwritedb_da(const vector<string> &/*dbnames*/)
-{
-    throw Xapian::InvalidArgumentError("Attempted to open writable da database");
-}
-
-Xapian::WritableDatabase
-BackendManager::getwritedb_daflimsy(const vector<string> &/*dbnames*/)
-{
-    throw Xapian::InvalidArgumentError("Attempted to open writable daflimsy database");
-}
-
-Xapian::Database
-BackendManager::getdb_db(const vector<string> &dbnames)
-{
-    string parent_dir = ".db";
-    create_dir_if_needed(parent_dir);
-
-    string dbdir = parent_dir + "/db";
-    for (vector<string>::const_iterator i = dbnames.begin();
-	 i != dbnames.end();
-	 i++) {
-	dbdir += "=" + *i;
-    }
-    if (create_dir_if_needed(dbdir)) {
-	// directory was created, so do the indexing.
-	// need to build temporary file and run it through makedb (yum)
-	index_files_to_m36("makeDB", dbdir, change_names_to_paths(dbnames));
-    }
-    return Xapian::Muscat36::open_db(dbdir + "/DB", dbdir + "/keyfile");
-}
-
-Xapian::Database
-BackendManager::getdb_dbflimsy(const vector<string> &dbnames)
-{
-    string parent_dir = ".dbflimsy";
-    create_dir_if_needed(parent_dir);
-
-    string dbdir = parent_dir + "/db";
-    for (vector<string>::const_iterator i = dbnames.begin();
-	 i != dbnames.end();
-	 i++) {
-	dbdir += "=" + *i;
-    }
-    // should autodetect flimsy - don't specify to test this
-    if (create_dir_if_needed(dbdir)) {
-	// directory was created, so do the indexing.
-	// need to build temporary file and run it through makedb (yum)
-	index_files_to_m36("makeDBflimsy", dbdir, change_names_to_paths(dbnames));
-    }
-    return Xapian::Muscat36::open_db(dbdir + "/DB", dbdir + "/keyfile");
-}
-
-Xapian::WritableDatabase
-BackendManager::getwritedb_db(const vector<string> &/*dbnames*/)
-{
-    throw Xapian::InvalidArgumentError("Attempted to open writable db database");
-}
-
-Xapian::WritableDatabase
-BackendManager::getwritedb_dbflimsy(const vector<string> &/*dbnames*/)
-{
-    throw Xapian::InvalidArgumentError("Attempted to open writable dbflimsy database");
+    return Xapian::Remote::open_writable(LOCALHOST, port);
 }
 #endif
 
@@ -763,3 +772,14 @@ BackendManager::get_writable_database(const string &dbname)
     dbnames.push_back(dbname);
     return (this->*do_getwritedb)(dbnames);
 }
+
+const char *
+BackendManager::get_xapian_progsrv_command()
+{
+#ifdef HAVE_VALGRIND
+    if (RUNNING_ON_VALGRIND) {
+	return "./runsrv "XAPIAN_PROGSRV;
+    }
+#endif
+    return XAPIAN_PROGSRV;
+}
