@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,10 +27,12 @@
 # include <valgrind/memcheck.h>
 # include <stdio.h>
 # include <sys/types.h>
-# include <sys/stat.h>
 # include "safefcntl.h"
+# include "safeunistd.h"
 #endif
 
+#include <algorithm>
+#include <iomanip>
 #include <iostream>
 
 #ifdef HAVE_STREAMBUF
@@ -41,6 +43,8 @@
 
 #include <set>
 
+#include <float.h> // For DBL_DIG.
+#include <math.h> // For ceil, fabs, log10.
 #include <stdlib.h>
 
 #include "gnu_getopt.h"
@@ -53,6 +57,7 @@
 #include <xapian/error.h>
 #include "testsuite.h"
 #include "omdebug.h"
+#include "stringutils.h"
 #include "utils.h"
 
 using namespace std;
@@ -75,8 +80,8 @@ om_ostringstream tout;
 
 int test_driver::runs = 0;
 test_driver::result test_driver::total = {0, 0, 0};
-string test_driver::argv0 = "";
-string test_driver::opt_help = "";
+string test_driver::argv0;
+string test_driver::opt_help;
 map<int, string *> test_driver::short_opts;
 vector<string> test_driver::test_names;
 bool test_driver::abort_on_error = false;
@@ -90,29 +95,41 @@ test_driver::get_srcdir()
     char *p = getenv("srcdir");
     if (p != NULL) return string(p);
 
-    string srcdir = argv0;
-    // default srcdir to everything leading up to the last "/" on argv0
-    string::size_type i = srcdir.find_last_of('/');
+#ifdef __WIN32__
+    // The path on argv[0] will always use \ for the directory separator.
+    const char ARGV0_SEP = '\\';
+#else
+    const char ARGV0_SEP = '/';
+#endif
+    // Default srcdir to the pathname of argv[0].
+    string srcdir(argv0);
+    string::size_type i = srcdir.find_last_of(ARGV0_SEP);
     string srcfile;
     if (i != string::npos) {
 	srcfile = srcdir.substr(i + 1);
 	srcdir.erase(i);
-	// libtool puts the real executable in .libs...
-	i = srcdir.find_last_of('/');
+	// libtool may put the real executable in .libs.
+	i = srcdir.find_last_of(ARGV0_SEP);
 	if (srcdir.substr(i + 1) == ".libs") {
 	    srcdir.erase(i);
-	    if (srcfile.substr(0, 3) == "lt-") srcfile.erase(0, 3);
+	    // And it may have an "lt-" prefix.
+	    if (startswith(srcfile, "lt-")) srcfile.erase(0, 3);
 	}
     } else {
-	// default to current directory - probably won't work if libtool
-	// is involved as the true executable is usually in .libs
+	// No path of argv[0], so default srcdir to the current directory.
+	// This may not work if libtool is involved as the true executable is
+	// sometimes in ".libs".
 	srcfile = srcdir;
 	srcdir = ".";
     }
-    // sanity check
-    if (!file_exists(srcdir + "/" + srcfile + ".cc")) {
+
+    // Remove any trailing ".exe" suffix, since some platforms add this.
+    if (endswith(srcfile, ".exe")) srcfile.resize(srcfile.size() - 4);
+
+    // Sanity check.
+    if (!file_exists(srcdir + '/' + srcfile + ".cc")) {
 	cout << argv0
-	     << ": srcdir not in the environment and I can't guess it!\n"
+	     << ": srcdir is not in the environment and I can't guess it!\n"
 		"Run test programs using the runtest script - see HACKING for details"
 	     << endl;
 	exit(1);
@@ -203,12 +220,16 @@ test_driver::runtest(const test_desc *test)
 	    try {
 		expected_exception = NULL;
 #ifdef HAVE_VALGRIND
-		VALGRIND_DO_LEAK_CHECK;
-		int vg_errs = VALGRIND_COUNT_ERRORS;
-		long vg_leaks = 0, vg_dubious = 0, vg_reachable = 0, dummy;
-		VALGRIND_COUNT_LEAKS(vg_leaks, vg_dubious, vg_reachable, dummy);
-		// Skip past any unread log output.
-		lseek(vg_log_fd, 0, SEEK_END);
+		int vg_errs = 0;
+		long vg_leaks = 0, vg_dubious = 0, vg_reachable = 0;
+		if (vg_log_fd != -1) {
+		    VALGRIND_DO_LEAK_CHECK;
+		    vg_errs = VALGRIND_COUNT_ERRORS;
+		    long dummy;
+		    VALGRIND_COUNT_LEAKS(vg_leaks, vg_dubious, vg_reachable, dummy);
+		    // Skip past any unread log output.
+		    lseek(vg_log_fd, 0, SEEK_END);
+		}
 #endif
 		if (!test->run()) {
 		    string s = tout.str();
@@ -221,11 +242,12 @@ test_driver::runtest(const test_desc *test)
 		    return FAIL;
 		}
 #ifdef HAVE_VALGRIND
-		// We must empty tout before asking valgrind to perform its
-		// leak checks, otherwise the buffers holding the output may
-		// be identified as a memory leak (especially if >1K of output
-		// has been buffered it appears...)
-		tout.str("");
+		if (vg_log_fd != -1) {
+		    // We must empty tout before asking valgrind to perform its
+		    // leak checks, otherwise the buffers holding the output
+		    // may be identified as a memory leak (especially if >1K of
+		    // output has been buffered it appears...)
+		    tout.str("");
 #define REPORT_FAIL_VG(M) do { \
     if (verbose) { \
 	while (true) { \
@@ -236,80 +258,82 @@ test_driver::runtest(const test_desc *test)
     } \
     out << " " << col_red << M << col_reset; \
 } while (0)
-		char buf[1024];
-		while (true) {
-		    ssize_t c = read(vg_log_fd, buf, sizeof(buf));
-		    if (c == 0 || (c < 0 && errno != EINTR)) {
-			buf[0] = 0;
-			break;
+		    char buf[1024];
+		    while (true) {
+			ssize_t c = read(vg_log_fd, buf, sizeof(buf));
+			if (c == 0 || (c < 0 && errno != EINTR)) {
+			    buf[0] = 0;
+			    break;
+			}
+			if (c > 0) {
+			    ssize_t i = 0;
+			    do {
+				while (i < c && buf[i] != ' ') ++i;
+			    } while (++i < c && buf[i] == '\n');
+			    char *p = reinterpret_cast<char *>(memchr(buf + i, '\n', c - i));
+			    if (p == NULL) p = buf + c;
+			    c = p - buf - i;
+			    if (c > 1024) c = 79;
+			    memmove(buf, buf + i, c);
+			    buf[c] = '\0';
+			    break;
+			}
 		    }
-		    if (c > 0) {
-			ssize_t i = 0;
-			do {
-			    while (i < c && buf[i] != ' ') ++i;
-			} while (++i < c && buf[i] == '\n');
-			char *p = reinterpret_cast<char *>(memchr(buf + i, '\n', c - i));
-			if (p == NULL) p = buf + c;
-			c = p - buf - i;
-			if (c > 1024) c = 79;
-			memmove(buf, buf + i, c);
-			buf[c] = '\0';
-			break;
+		    VALGRIND_DO_LEAK_CHECK;
+		    int vg_errs2 = VALGRIND_COUNT_ERRORS;
+		    vg_errs = vg_errs2 - vg_errs;
+		    long vg_leaks2 = 0, vg_dubious2 = 0, vg_reachable2 = 0;
+		    long dummy;
+		    VALGRIND_COUNT_LEAKS(vg_leaks2, vg_dubious2, vg_reachable2,
+					 dummy);
+		    vg_leaks = vg_leaks2 - vg_leaks;
+		    vg_dubious = vg_dubious2 - vg_dubious;
+		    vg_reachable = vg_reachable2 - vg_reachable;
+		    if (vg_errs) {
+			string fail_msg(buf);
+			if (fail_msg.empty())
+			    fail_msg = "VALGRIND DETECTED A PROBLEM";
+			REPORT_FAIL_VG(fail_msg);
+			return FAIL;
+		    }
+		    if (vg_leaks > 0) {
+			REPORT_FAIL_VG("LEAKED " << vg_leaks << " BYTES");
+			return FAIL;
+		    }
+		    if (vg_dubious > 0) {
+			// If code deliberately holds onto blocks by a pointer
+			// not to the start (e.g. languages/utilities.c does)
+			// then we need to rerun the test to see if the leak is
+			// real...
+			if (runcount == 0) {
+			    out << " " << col_yellow << "PROBABLY LEAKED MEMORY - RETRYING TEST" << col_reset;
+			    ++runcount;
+			    continue;
+			}
+			REPORT_FAIL_VG("PROBABLY LEAKED " << vg_dubious << " BYTES");
+			return FAIL;
+		    }
+		    if (vg_reachable > 0) {
+			// FIXME:
+			// C++ STL implementations often "horde" released
+			// memory - perhaps we can supply our own allocator
+			// so we can tell the difference?
+			//
+			// See also:
+			// http://valgrind.org/docs/FAQ/#faq.reports
+			//
+			// For now, just use runcount to rerun the test and see
+			// if more is leaked - hopefully this shouldn't give
+			// false positives.
+			if (runcount == 0) {
+			    out << " " << col_yellow << "POSSIBLE UNRELEASED MEMORY - RETRYING TEST" << col_reset;
+			    ++runcount;
+			    continue;
+			}
+			REPORT_FAIL_VG("FAILED TO RELEASE " << vg_reachable << " BYTES");
+			return FAIL;
 		    }
 		}
-		VALGRIND_DO_LEAK_CHECK;
-		int vg_errs2 = VALGRIND_COUNT_ERRORS;
-		vg_errs = vg_errs2 - vg_errs;
-		long vg_leaks2 = 0, vg_dubious2 = 0, vg_reachable2 = 0;
-		VALGRIND_COUNT_LEAKS(vg_leaks2, vg_dubious2, vg_reachable2,
-				     dummy);
-		vg_leaks = vg_leaks2 - vg_leaks;
-		vg_dubious = vg_dubious2 - vg_dubious;
-		vg_reachable = vg_reachable2 - vg_reachable;
-		if (vg_errs) {
-		    string fail_msg(buf);
-		    if (fail_msg.empty())
-			fail_msg = "VALGRIND DETECTED A PROBLEM";
-		    REPORT_FAIL_VG(fail_msg);
-		    return FAIL;
-		}
-		if (vg_leaks > 0) {
-		    REPORT_FAIL_VG("LEAKED " << vg_leaks << " BYTES");
-		    return FAIL;
-		}
-		if (vg_dubious > 0) {
-		    // If code deliberately holds onto blocks by a pointer not
-		    // to the start (e.g. languages/utilities.c does) then we
-		    // need to rerun the test to see if the leak is real...
-		    if (runcount == 0) {
-			out << " " << col_yellow << "PROBABLY LEAKED MEMORY - RETRYING TEST" << col_reset;
-			++runcount;
-			continue;
-		    }
-		    REPORT_FAIL_VG("PROBABLY LEAKED " << vg_dubious << " BYTES");
-		    return FAIL;
-		}
-		if (vg_reachable > 0) {
-		    // FIXME:
-		    // C++ STL implementations often "horde" released
-		    // memory - perhaps we can supply our own allocator
-		    // so we can tell the difference?
-		    //
-		    // See also:
-		    // http://valgrind.org/docs/FAQ/#faq.reports
-		    //
-		    // For now, just use runcount to rerun the test and see
-		    // if more is leaked - hopefully this shouldn't give
-		    // false positives.
-		    if (runcount == 0) {
-			out << " " << col_yellow << "POSSIBLE UNRELEASED MEMORY - RETRYING TEST" << col_reset;
-			++runcount;
-			continue;
-		    }
-		    REPORT_FAIL_VG("FAILED TO RELEASE " << vg_reachable << " BYTES");
-		    return FAIL;
-		}
-REPORT_FAIL_VG("NO PROBLEM");
 #endif
 	    } catch (const TestFail &) {
 		string s = tout.str();
@@ -342,14 +366,7 @@ REPORT_FAIL_VG("NO PROBLEM");
 		    tout.str("");
 		}
 		out << " " << col_red << errclass << col_reset;
-		if (verbose) {
-		    out << err.get_type() << " exception: " << err.get_msg();
-		    if (!err.get_context().empty())
-			out << " (context:" << err.get_context() << ")";
-		    if (err.get_errno())
-			out << " (errno:" << strerror(err.get_errno()) << ")";
-		    out << endl;
-		}
+		if (verbose) out << err.get_description() << endl;
 		return FAIL;
 	    } catch (const string & msg) {
 		string s = tout.str();
@@ -358,10 +375,14 @@ REPORT_FAIL_VG("NO PROBLEM");
 		    if (s[s.size() - 1] != '\n') out << endl;
 		    tout.str("");
 		}
-		out << " " << col_red << "EXCEPT" << col_reset;
-		if (verbose) {
-		    out << msg << endl;
-		}
+		out << " " << col_red << "EXCEPTION: ";
+		size_t cutoff = min(size_t(40), msg.size());
+		cutoff = find(msg.begin(), msg.begin() + cutoff, '\n') - msg.begin();
+		if (verbose || cutoff == msg.size())
+		    out << msg;
+		else
+		    out << msg.substr(0, cutoff) << "...";
+		out << col_reset;
 		return FAIL;
 	    } catch (...) {
 		string s = tout.str();
@@ -370,10 +391,7 @@ REPORT_FAIL_VG("NO PROBLEM");
 		    if (s[s.size() - 1] != '\n') out << endl;
 		    tout.str("");
 		}
-		out << " " << col_red << "EXCEPT" << col_reset;
-		if (verbose) {
-		    out << "Unknown exception!" << endl;
-		}
+		out << " " << col_red << "UNKNOWN EXCEPTION" << col_reset;
 		return FAIL;
 	    }
 	} else {
@@ -591,24 +609,26 @@ test_driver::parse_command_line(int argc, char **argv)
 
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
-	// Open the valgrind log file, and unlink it.
-	char fname[64];
-	sprintf(fname, ".valgrind.log.%lu", (unsigned long)getpid());
-	vg_log_fd = open(fname, O_RDONLY|O_NONBLOCK);
-	if (vg_log_fd == -1 && errno == ENOENT) {
-	    // Older valgrind versions named the log output differently.
-	    sprintf(fname, ".valgrind.log.pid%lu", (unsigned long)getpid());
+	if (getenv("XAPIAN_TESTSUITE_VALGRIND") != NULL) {
+	    // Open the valgrind log file, and unlink it.
+	    char fname[64];
+	    sprintf(fname, ".valgrind.log.%lu", (unsigned long)getpid());
 	    vg_log_fd = open(fname, O_RDONLY|O_NONBLOCK);
+	    if (vg_log_fd == -1 && errno == ENOENT) {
+		// Older valgrind versions named the log output differently.
+		sprintf(fname, ".valgrind.log.pid%lu", (unsigned long)getpid());
+		vg_log_fd = open(fname, O_RDONLY|O_NONBLOCK);
+	    }
+	    if (vg_log_fd != -1) unlink(fname);
 	}
-	unlink(fname);
     }
 #endif
 
+#ifdef XAPIAN_DEBUG_VERBOSE
     // We need to display something before we start, or the allocation
     // made when the first debug message is displayed is (wrongly) picked
     // up on as a memory leak.
     DEBUGLINE(UNKNOWN, "Starting testsuite run.");
-#ifdef XAPIAN_DEBUG_VERBOSE
     om_debug.initialise();
 #endif /* XAPIAN_DEBUG_VERBOSE */
 }
@@ -634,3 +654,9 @@ test_driver::run(const test_desc *tests)
 
     return bool(myresult.failed); // if 0, then everything passed
 }
+
+bool
+TEST_EQUAL_DOUBLE_(double a, double b)
+{
+    return (ceil(log10(max(fabs(a), fabs(b)))) - log10(fabs(a - b)) > DBL_DIG);
+}
