@@ -2,8 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2006 Olly Betts
- * Copyright 2006 Richard Boulton
+ * Copyright 2002,2003,2006,2007 Olly Betts
+ * Copyright 2006 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,20 +23,27 @@
 
 #include <config.h>
 
+// Need to call Error::get_errno() for testing.
+#define XAPIAN_DEPRECATED(D) D
+#include <xapian.h>
+
 #include <float.h>
+#include "safeerrno.h"
 
 #include <string>
 #include <list>
 
 using namespace std;
 
-#include <xapian.h>
+#include "autoptr.h"
 #include "testsuite.h"
+#include "testutils.h"
 #include "omstringstream.h"
 
 #include "serialise.h"
 #include "serialise-double.h"
 #include "omqueryinternal.h"
+#include "utils.h"
 
 static bool test_except1()
 {
@@ -140,6 +147,44 @@ static bool test_refcnt2()
     return true;
 }
 
+// Class for testing AutoPtr<>.
+class test_autoptr {
+    bool &deleted;
+  public:
+    test_autoptr(bool &deleted_) : deleted(deleted_) {
+	tout << "test_autoptr constructor\n";
+    }
+    ~test_autoptr() {
+	deleted = true;
+	tout << "test_autoptr destructor\n";
+    }
+};
+
+// Test autoptr self-assignment.
+static bool test_autoptr1()
+{
+    bool deleted = false;
+
+    test_autoptr * raw_ptr = new test_autoptr(deleted);
+    {
+	AutoPtr<test_autoptr> ptr(raw_ptr);
+
+	TEST_EQUAL(ptr.get(), raw_ptr);
+
+	TEST(!deleted);
+
+	ptr = ptr;
+
+	TEST_EQUAL(ptr.get(), raw_ptr);
+
+	TEST(!deleted);
+    }
+
+    TEST(deleted);
+    
+    return true;
+}
+
 // test string comparisions
 static bool test_stringcomp1()
 {
@@ -190,6 +235,32 @@ static bool test_omstringstream1()
     return true;
 }
 
+static bool test_tostring1()
+{
+    TEST_EQUAL(om_tostring(0), "0");
+    TEST_EQUAL(om_tostring(10), "10");
+    TEST_EQUAL(om_tostring(10u), "10");
+    TEST_EQUAL(om_tostring(-10), "-10");
+    TEST_EQUAL(om_tostring(0xffffffff), "4294967295");
+    TEST_EQUAL(om_tostring(0x7fffffff), "2147483647");
+    TEST_EQUAL(om_tostring(0x7fffffffu), "2147483647");
+    TEST_EQUAL(om_tostring(-0x7fffffff), "-2147483647");
+
+#ifdef __WIN32__
+    /* Test the 64 bit integer conversion to string.
+     * (Currently only exists for windows.)
+     */
+    TEST_EQUAL(om_tostring(10ll), "10");
+    TEST_EQUAL(om_tostring(-10ll), "-10");
+    TEST_EQUAL(om_tostring(0x200000000ll), "8589934592");
+// We don't currently have an "unsigned long long" version since it's not required
+// anywhere in the library.
+//    TEST_EQUAL(om_tostring(0x200000000ull), "8589934592");
+#endif
+
+    return true;
+}
+
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 // Check serialisation of lengths.
 static bool test_serialiselength1()
@@ -199,7 +270,7 @@ static bool test_serialiselength1()
 	string s = encode_length(n);
 	const char *p = s.data();
 	const char *p_end = p + s.size();
-	size_t decoded_n = decode_length(&p, p_end);
+	size_t decoded_n = decode_length(&p, p_end, false);
 	if (n != decoded_n || p != p_end) tout << "[" << s << "]" << endl;
 	TEST_EQUAL(n, decoded_n);
 	TEST_EQUAL(p_end - p, 0);
@@ -212,6 +283,82 @@ static bool test_serialiselength1()
 
     return true;
 }
+
+// Regression test: vetting the remaining buffer length
+static bool test_serialiselength2()
+{
+    // Special case tests for 0
+    {
+	string s = encode_length(0);
+	{
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == 0);
+	    TEST(p == p_end);
+	}
+	s += 'x';
+	{
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == 0);
+	    TEST_EQUAL(p_end - p, 1);
+	}
+    }
+    // Special case tests for 1
+    {
+	string s = encode_length(1);
+	TEST_EXCEPTION(Xapian::NetworkError,
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == 1);
+	);
+	s += 'x';
+	{
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == 1);
+	    TEST_EQUAL(p_end - p, 1);
+	}
+	s += 'x';
+	{
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == 1);
+	    TEST_EQUAL(p_end - p, 2);
+	}
+    }
+    // Nothing magic here, just test a range of odd and even values.
+    for (size_t n = 2; n < 1000; n = (n + 1) * 2 + (n >> 1)) {
+	string s = encode_length(n);
+	TEST_EXCEPTION(Xapian::NetworkError,
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == n);
+	);
+	s.append(n-1, 'x');
+	TEST_EXCEPTION(Xapian::NetworkError,
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == n);
+	);
+	s += 'x';
+	{
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == n);
+	    TEST_EQUAL(size_t(p_end - p), n);
+	}
+	s += 'x';
+	{
+	    const char *p = s.data();
+	    const char *p_end = p + s.size();
+	    TEST(decode_length(&p, p_end, true) == n);
+	    TEST_EQUAL(size_t(p_end - p), n + 1);
+	}
+    }
+
+    return true;
+}
 #endif
 
 static void check_double_serialisation(double u)
@@ -283,34 +430,31 @@ static bool test_serialisedoc1()
 
     return true;
 }
-#endif
 
 // Check serialisation of queries.
 static bool test_serialisequery1()
 {
     string s;
     list<Xapian::Query> queries;
-    
+
     queries.push_back(Xapian::Query("foo"));
 
+    // Regression test for bug in 0.9.10 and earlier.
+    queries.push_back(Xapian::Query("foo", 1, 1));
+
+    queries.push_back(Xapian::Query(Xapian::Query::OP_OR,
+                                    Xapian::Query("foo", 1, 1),
+                                    Xapian::Query("bar", 1, 1)));
+
     const char * words[] = { "paragraph", "word" };
     queries.push_back(Xapian::Query(Xapian::Query::OP_OR, words, words + 2));
 
     list<Xapian::Query>::const_iterator query;
-    for(query = queries.begin(); query != queries.end(); query++) {
+    for (query = queries.begin(); query != queries.end(); query++) {
         Xapian::Query::Internal * qint;
 
-        try {
-            s = query->internal->serialise();
-        } catch (const Xapian::InternalError & e) {
-            SKIP_TEST("query serialisation not compiled in");
-        }
-
-        try {
-            qint = Xapian::Query::Internal::unserialise(s);
-        } catch (const Xapian::InternalError & e) {
-            SKIP_TEST("query unserialisation not compiled in");
-        }
+	s = query->internal->serialise();
+	qint = Xapian::Query::Internal::unserialise(s);
 
         TEST(qint->serialise() == s);
         delete qint;
@@ -319,6 +463,46 @@ static bool test_serialisequery1()
     return true;
 }
 
+// Check serialisation of Xapian::Error.
+static bool test_serialiseerror1()
+{
+    string enoent_msg(strerror(ENOENT));
+    Xapian::DatabaseOpeningError e("Failed to open database", ENOENT);
+    // Regression test for bug in 1.0.0 - it didn't convert errno values for
+    // get_description() if they hadn't already been converted.
+    TEST_STRINGS_EQUAL(e.get_description(), "DatabaseOpeningError: Failed to open database (" + enoent_msg + ")");
+
+    TEST_EQUAL(e.get_errno(), ENOENT);
+    TEST_STRINGS_EQUAL(e.get_error_string(), enoent_msg);
+
+    string serialisation = serialise_error(e);
+
+    // Test if unserialise_error() throws with a flag to avoid the possibility
+    // of an "unreachable code" warning when we get around to marking
+    // unserialise_error() as "noreturn".
+    bool threw = false;
+    try {
+	// unserialise_error throws an exception.
+	unserialise_error(serialisation, "", "");
+    } catch (Xapian::Error & ecaught) {
+	TEST_EQUAL(ecaught.get_errno(), 0); // errno values aren't portable.
+	TEST_STRINGS_EQUAL(ecaught.get_error_string(), enoent_msg);
+	threw = true;
+    }
+    TEST(threw);
+
+    // Check that the original is still OK.
+    TEST_STRINGS_EQUAL(e.get_error_string(), enoent_msg);
+
+    // Regression test - in 1.0.0, copying used to duplicate the error_string
+    // pointer, resulting in double calls to free().
+    Xapian::DatabaseOpeningError ecopy(e);
+    TEST_STRINGS_EQUAL(ecopy.get_error_string(), enoent_msg);
+
+    return true;
+}
+#endif
+
 // By default Sun's C++ compiler doesn't call the destructor on a
 // temporary object until the end of the block (contrary to what
 // ISO C++ requires).  This is done in the name of "compatibility".
@@ -353,15 +537,19 @@ test_desc tests[] = {
     {"exception1",              test_exception1},
     {"refcnt1",			test_refcnt1},
     {"refcnt2",			test_refcnt2},
+    {"autoptr1",		test_autoptr1},
     {"stringcomp1",		test_stringcomp1},
     {"temporarydtor1",		test_temporarydtor1},
     {"omstringstream1",		test_omstringstream1},
+    {"tostring1",		test_tostring1},
     {"serialisedouble1",	test_serialisedouble1},
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
     {"serialiselength1",	test_serialiselength1},
+    {"serialiselength2",	test_serialiselength2},
     {"serialisedoc1",		test_serialisedoc1},
-#endif
     {"serialisequery1",		test_serialisequery1},
+    {"serialiseerror1",		test_serialiseerror1},
+#endif
     {0, 0}
 };
 
