@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,8 +22,12 @@
 
 #include <config.h>
 
+// We have to use the deprecated Quartz::open() method.
+#define XAPIAN_DEPRECATED(D) D
+
 #include "safeerrno.h"
 
+#include "unixcmds.h"
 #include "testsuite.h"
 #include "testutils.h"
 #include <xapian/error.h>
@@ -39,7 +43,7 @@
 using namespace std;
 
 #include <sys/types.h>
-#include <sys/stat.h>
+#include "safesysstat.h"
 
 static string tmpdir;
 
@@ -53,7 +57,7 @@ static void makedir(const string &filename)
 
 static void removedir(const string &filename)
 {
-    rmdir(filename);
+    rm_rf(filename);
     struct stat buf;
     if (stat(filename, &buf) == 0 || errno != ENOENT) {
 	FAIL_TEST("Failed to remove directory `" << filename << "' (" <<
@@ -61,29 +65,6 @@ static void removedir(const string &filename)
     }
 }
 
-static void unlink_table(const string & path)
-{
-    unlink(path + "DB");
-    unlink(path + "baseA");
-    unlink(path + "baseB");
-}
-
-/// Test opening of a quartz database
-static bool test_open1()
-{
-    const string dbdir = tmpdir + "testdb_open1";
-    removedir(dbdir);
-
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Internal::RefCntPtr<Xapian::Database::Internal> database_0 = new QuartzDatabase(dbdir));
-
-    Xapian::Internal::RefCntPtr<Xapian::Database::Internal> database_w =
-	    new QuartzWritableDatabase(dbdir, Xapian::DB_CREATE, 2048);
-    Xapian::Internal::RefCntPtr<Xapian::Database::Internal> database_r = new QuartzDatabase(dbdir);
-
-    return true;
-}
-
 /// Test creating and opening of quartz databases
 static bool test_create1()
 {
@@ -91,40 +72,7 @@ static bool test_create1()
     removedir(dbdir);
 
     Xapian::Internal::RefCntPtr<Xapian::Database::Internal> db;
-
-    // (1) db doesn't exist (no create)
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   db = new QuartzDatabase(dbdir));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   db = new QuartzWritableDatabase(dbdir, Xapian::DB_OPEN, 2048));
-
-    // (2) db doesn't exist, basedir doesn't exist (create)
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   db = new QuartzDatabase(dbdir));
     db = new QuartzWritableDatabase(dbdir, Xapian::DB_CREATE, 2048);
-    db = 0; // Close the database - Cygwin can't delete a locked file...
-    removedir(dbdir);
-
-    // (3) db doesn't exist, basedir exists (no create)
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   db = new QuartzDatabase(dbdir));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   db = new QuartzWritableDatabase(dbdir, Xapian::DB_OPEN, 2048));
-
-    // (4) db doesn't exist, basedir exists (create)
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   db = new QuartzDatabase(dbdir));
-    db = new QuartzWritableDatabase(dbdir, Xapian::DB_CREATE, 2048);
-    db = new QuartzDatabase(dbdir);
-
-    // (5) db exists (create, no overwrite)
-    db = new QuartzDatabase(dbdir);
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-		   db = new QuartzWritableDatabase(dbdir, Xapian::DB_CREATE, 2048));
-    db = new QuartzDatabase(dbdir);
-
-    // (6) db exists (no create)
-    db = new QuartzWritableDatabase(dbdir, Xapian::DB_OPEN, 2048);
 
     // (7) db exists (create, overwrite)
     db = new QuartzDatabase(dbdir);
@@ -311,9 +259,8 @@ static bool test_adddoc2()
 		TEST_NOT_EQUAL(j, document_out.termlist_end());
 		TEST_EQUAL(*i, *j);
 		TEST_EQUAL(i.get_wdf(), j.get_wdf());
-		// FIXME: MapTermList::get_termfreq asserts in a debug build
-		//TEST_NOT_EQUAL(i.get_termfreq(), j.get_termfreq());
-		//TEST_EQUAL(0, i.get_termfreq());
+		TEST_EXCEPTION(Xapian::InvalidOperationError,
+			       (void)i.get_termfreq());
 		TEST_NOT_EQUAL(0, j.get_termfreq());
 		if (*i == "foobar") {
 		    // termfreq of foobar is 2
@@ -386,7 +333,7 @@ static bool test_adddoc3()
 /// Test packing integers into strings
 static bool test_packint1()
 {
-    TEST_EQUAL(pack_uint(0u), string("\000", 1));
+    TEST_EQUAL(pack_uint(0u), string("\0", 1));
     TEST_EQUAL(pack_uint(1u), string("\001", 1));
     TEST_EQUAL(pack_uint(127u), string("\177", 1));
     TEST_EQUAL(pack_uint(128u), string("\200\001", 2));
@@ -572,7 +519,6 @@ static bool test_postlist1()
     {
 	QuartzPostList pl2(db_w, &table, &positiontable, "foo");
 	TEST_EQUAL(pl2.get_termfreq(), 0);
-	TEST_EQUAL(pl2.get_collection_freq(), 0);
 	pl2.next(0);
 	TEST(pl2.at_end());
     }
@@ -582,7 +528,6 @@ static bool test_postlist1()
     {
 	QuartzPostList pl2(db_w, &table, &positiontable, "foo");
 	TEST_EQUAL(pl2.get_termfreq(), 1);
-	TEST_EQUAL(pl2.get_collection_freq(), 7);
 	pl2.next(0);
 	TEST(!pl2.at_end());
 	TEST_EQUAL(pl2.get_docid(), 5);
@@ -596,7 +541,6 @@ static bool test_postlist1()
     {
 	QuartzPostList pl2(db_w, &table, &positiontable, "foo");
 	TEST_EQUAL(pl2.get_termfreq(), 2);
-	TEST_EQUAL(pl2.get_collection_freq(), 8);
 	pl2.next(0);
 	TEST(!pl2.at_end());
 	TEST_EQUAL(pl2.get_docid(), 5);
@@ -631,7 +575,6 @@ static bool test_postlist2()
     {
 	QuartzPostList pl2(db_w, &bufftable, &positiontable, "foo");
 	TEST_EQUAL(pl2.get_termfreq(), 0);
-	TEST_EQUAL(pl2.get_collection_freq(), 0);
 	pl2.next(0);
 	TEST(pl2.at_end());
     }
@@ -657,7 +600,6 @@ static bool test_postlist2()
     {
 	QuartzPostList pl2(db_w, &bufftable, &positiontable, "foo");
 	TEST_EQUAL(pl2.get_termfreq(), testdata.size());
-	TEST_EQUAL(pl2.get_collection_freq(), collfreq);
 	pl2.next(0);
 	vector<unsigned int>::const_iterator i3 = testdata.begin();
 
@@ -837,7 +779,6 @@ static bool test_packstring1()
 
 /// The lists of tests to perform
 test_desc tests[] = {
-    {"open1",		test_open1},
     {"create1",		test_create1},
     {"adddoc1",		test_adddoc1},
     {"adddoc2",		test_adddoc2},
